/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#include <stdio.h>
#include "CECRangeVar.h"
#include "CyrException.h"

CECRangeVar::CECRangeVar(const string& name,int options,const string& values,unsigned long seed)
        :CECProcessVar(options,name),_seed(seed),_cur_seed(seed)
{
    if(!(_options & VTINT))
        throw CyrException("CECRangeVar::CECRangeVar","Range variable not integer values , "+_name,-1);
    if(_options & VORAND )
        _cur_seed=CECVar::GetRandomSeed();
    string::size_type pos=values.find_first_of("-");
    if(pos==string::npos)
        throw CyrException("CECRangeVar::CECRangeVar","Invalid values for Range variable , "+_name,-1);
    _begin=atol(values.substr(0,pos).c_str());
    _end=atol(values.substr(pos+1,values.length()-pos).c_str());
    if(_end<_begin)
    {
        _increment = false;
        _cur_int=_begin+1;
    }
    else
    {
        _increment = true;
        _cur_int=_begin-1;
    }
    _cur_value="";

}
CECRangeVar::~CECRangeVar()
{}

string  CECRangeVar::get_value(const string& expression)
{
    if(_cur_value=="")//have never done a next
    {
        string msg="No Next or Generate performed on variable "+_name;
        throw CyrException("CECRangeVar::get_value",msg,-1);
    }
    return CECProcessVar::get_value(expression);
}
void    CECRangeVar::next_value()
{
    if(_increment)
    {
        _cur_int++;
        if(_cur_int > _end )
            _cur_int=_begin;
    }
    else
    {
        _cur_int--;
        if(_cur_int < _end )
            _cur_int=_begin;
    }
    char res[13];
    //
    // LINUX Port Changes
    //
    // No ltoa on linux so ...
    sprintf(res,"%d",_cur_int);
    _cur_value = res;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   Generate a random value in the range _begin to _end and put the
//          result in _cur_value (as a string) and _cur_int (as an integer).
//
void CECRangeVar::generate_value()
{
    // Set seed.
    srand (_cur_seed);      // Use current seed to randomize
    _cur_seed = rand();     // Set current seed to random for next srand()

    // Generate a random integer value in the range _begin to _end.
    if (_end > _begin)
    {
        _cur_int = (_cur_seed % (_end - _begin + 1)) + _begin;
    }
    else
    {
        _cur_int = (_cur_seed % (_begin - _end + 1)) + _end;
    }
    
    // Get random integer value as a string.
    // LINUX Port Changes
    // No ltoa on Linux so...
    char res[13];
    sprintf (res, "%d", _cur_int);
    _cur_value = res;
}

void    CECRangeVar::reset_seed(unsigned long seed)
{
    if(seed==0)seed=_seed;//use original seed if required
    _cur_seed=seed;//set seed up for next generate_value
    if(_end<_begin)//set back to beginning state ready for next or generate
    {
        _cur_int=_begin+1;
    }
    else
    {
        _cur_int=_begin-1;
    }
    _cur_value="";

}
