/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrFFApi
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECFFFile_h
#define _CECFFFile_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include "CECFFLink.h"
#include "CECFFFolder.h"

///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Constants
const int FILE_STATUS_CURRENT     = 0;
const int FILE_STATUS_MERGE       = 1;
const int FILE_STATUS_MISSING     = 2;
const int FILE_STATUS_MODIFIED    = 3;
const int FILE_STATUS_NEW         = 4;
const int FILE_STATUS_OUT_OF_DATE = 5;
const int FILE_STATUS_UNKNOWN     = 6;
const int FILE_STATUS_ERROR       = 7;

#define FILE_TYPE                   "File"

///////////////////////////////////////////////////////////////////////////////
// @class   CECFFFile | This class wraps up functionality available via the
//          IFile interface.  It is intended purely to hide the COM
//          mechanisms from products such as Impact.
// @base    public | CECFFLink
class CECFFFile : public CECFFLink
{
    // @access Public Members
public:
    // @cmember Constructor used to retrieve a File
    CECFFFile( CECFFFolder* parent, const char* name );
    // @cmember Constructor used to create a new File
    CECFFFile( CECFFFolder* parent, const char* name, const char* desc );
    virtual ~CECFFFile(); // @cmember Destructor

    string GetName() const;          // @cmember Returns File name
    string GetDescription() const;   // @cmember Returns File description
    string GetComment() const;       // @cmember Returns File comment
    void SetDescription( const char* desc ); // @cmember Set new File description
    void SetComment( const char* comment );  // @cmember Set new File comment
    string GetFullPath() const;      // @cmember Returns full path of file

    void Update();  // @cmember Persists any changes into OpenSTARep database
    void Remove();  // @cmember Removes File from OpenSTARep database

    bool Lock();            // @cmember Locks File access
    void Unlock();          // @cmember Unlocks File access

    // @cmember Retrieve File contents from database
    bool CheckOut( bool lock );
    // @cmember Transfer File contents to database
    bool CheckIn( bool release );


    CECFFServer*  GetParentServer();
    CECFFProject* GetParentProject();
    CECFFFolder*  GetParentFolder();

    // @access Protected Members
protected:
    CECFFFolder* m_parent;    // @cmember Reference to parent Folder object
    bool RetrieveFile( CECFFFolder* parent, const char* name );
    string m_name;
    string m_desc;
    // @cmember Returns underlying servant reference as an item for linking
}
;

#endif
