/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrFFApi
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECFFLink_h
#define _CECFFLink_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include "CECFFServer.h"
#include "CECFFProject.h"

///////////////////////////////////////////////////////////////////////////////
// Pre-declared classes
class CECFFFolder;

///////////////////////////////////////////////////////////////////////////////
// @class   CECLink | The CECLink class is intended as a base class for all
//          OpenSTARep items that can be linked together.
// @class   public | CECFFBean
class CECFFLink : public CECFFBean
{
    // @access Public Members
public:
    virtual ~CECFFLink();  // @cmember Destructor

    void AddLink( CECFFLink* link );     // @cmember Creates new link
    void RemoveLink( CECFFLink* link );  // @cmember Deletes existing link

    string GetLinkPath();       // @cmember Returns link path of this object
    string GetFirstLinkPath();  // @cmember Returns first linked item path
    string GetNextLinkPath();   // @cmember Returns next linked item path

    virtual CECFFServer*  GetParentServer() = 0;
    virtual CECFFProject* GetParentProject() = 0;
    virtual CECFFFolder*  GetParentFolder() = 0;


    // @access Protected Members
protected:
    CECFFLink();           // @cmember Constructor
    long m_linkIndex;
};

#endif
