/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc Cyrdmn
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrDmn_h
#define _CyrDmn_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <OSTAORB/OSTAORB.h>
#include <Factory/Factory_i.h>
#include <library.hpp>

#include <list>
using std::list;

///////////////////////////////////////////////////////////////////////////////
// Library function prototypes
typedef int (*PFN_LIB_INIT)( const char*, CORBAServer* );
typedef int (*PFN_LIB_UNINIT)();

///////////////////////////////////////////////////////////////////////////////
// Type definitions
// @type    LibList | A list of library modules.
typedef list<CTLibrary*>   LibList;

///////////////////////////////////////////////////////////////////////////////
// @class   This class contains all functionality required to implement a
//          Cyrano daemon process.  This includes logging facilities as well as
//          automatic library loading, etc.
//
//          This class only throws exceptions of type <c CyrDaemonException>.
//          All other exception types are caught within this class.
class CyrDaemon
{
    // @access Public Members
public:
    CyrDaemon( CORBAServer* pServer, const char* name );  // @cmember Constructor.
    virtual ~CyrDaemon();                                 // @cmember Destructor.

    void Run( bool cont = true );       // @cmember Starts daemon execution.

    // @cmember,mfunc Returns true if Daemon is running.
    bool IsRunning() const
    {
        return m_running;
    };
    void StartReaper();         // @cmember Start reaper thread.
    void StopReaper();          // @cmember Stop reaper thread.
    void ReaperThread();        // @cmember Actual reaper thread method.
    void StartIPCheck();         // @cmember Start IPCheck thread.
    void StopIPCheck();          // @cmember Stop IPCheck thread.
    void IPCheckThread();        // @cmember Actual IPCheck thread method.
    // @access Private Members
private:
    void StartFactory();            // @cmember Starts CORBA Factory for this object.
    void StopFactory();             // @cmember Stops this object's CORBA Factory.
    void LoadProviderModules();     // @cmember Loads Provider modules.
    void UnloadProviderModules();   // @cmember Unloads Provider modules.

    string    m_name;        // @cmember The name of the contained Factory object.
    FactoryImpl* m_factory;     // @cmember Pointer to Factory object.
    CORBAServer* m_server;      // @cmember Pointer to CORBA server functionality.
    LibList      m_lstLib;      // @cmember A list of Provider libraries.
    bool         m_running;     // @cmember If true the daemon is running.

    omni_thread* reaper;    // @cmember Handle to the reaper method.
    omni_thread* ipcheck;    // @cmember Handle to the ipcheck method.

}
;

#endif

