/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc ProviderReg
///////////////////////////////////////////////////////////////////////////////

#ifndef _ProviderReg_h
#define _ProviderReg_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes

///////////////////////////////////////////////////////////////////////////////
#ifdef _WIN32
#   pragma warning (disable : 4786)
#endif
#include <string>
#include <list>
using std::list;
using std::string;
///////////////////////////////////////////////////////////////////////////////
// @struct  PROV_S | This structure contains information about registered
//          Provider classes.
// @field   string| name | Name of the Provider class.
// @field   string| provider | Name of the Provider component.
// @field   int | type | Type of the Provider class.
typedef struct tagPROV_S
{
    string name;
    string provider;
    long type;
}
PROV_S, *PPROV_S;
// @type    PPROV_S | This type is a pointer to a <t PROV_S> structure.

// @type    ProviderList | This type is a list of <t PROV_S> structures.
// @type    SearchList | This type represents the results of a search.
typedef list<PPROV_S>    ProviderList;
typedef list<string>  SearchList;

///////////////////////////////////////////////////////////////////////////////
// @class   The Provider Register is responsible for maintaining a list of
//          classes with a list of Providers.  A Provider will be registered
//          with this class when it is constructed and deregister itself when
//          it is destructed.
class ProviderReg
{
    // @access Public Members
public:
    ProviderReg()
    {}

    ;         // @cmember Constructor.
    virtual ~ProviderReg();     // @cmember Destructor.

    void Register(const char* name, const char* provider, int type );  // @cmember Register a class entry.
    void Deregister(const char* name, const char* provider );          // @cmember Remove a single class entry.
    void DeregisterAll(const char* provider );                       // @cmember Remove all class entries.

    string GetFirstClass();                      // @cmember Start iterating through all classes.
    string GetFirstClass( const char* provider );  // @cmember Start iterating through classes.
    string GetFirstClass( int type );            // @cmember Start iterating through classes.
    string GetFirstProvider();                   // @cmember Start iterating through all Providers.
    string GetFirstProvider( const char* name );   // @cmember Start iterating through Providers.
    string GetNext();                            // @cmember Get next entry in search list.

    // @access Private Members
private:
    ProviderList m_listProvider;            // @cmember List of all Provider entries

    // Search members
    SearchList      m_listSearch;       // @cmember Contains results for current search.
    SearchList::iterator m_searchIndex;                 // @cmember Search list iterator.
    bool InSearchList(const char* entry );   // @cmember Search for entry in search list.
}
;

#endif

