/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4514)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __CYRTOOLS_TIME__
#  include "ctime.hpp"
#endif  // __CYRTOOLS_TIME__

#ifdef  WIN32
#  include  <sys/timeb.h>
#endif  // WIN32

#if defined( AIX) || defined (ALPHA) || defined(SOLARIS)
# include <sys/time.h>
# include <iostream.h>
#endif


#ifdef HPUX
# include <sys/timeb.h>
#endif

//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////// Class CTTime //
///////////////////////////////////////////////////////////////////////////////
// Class representing the time.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTTime::CTTime
//
// Default constructor.
//
// ulSeconds      : number of seconds.
// ulMilliSeconds : number of milliseconds.
// Return    : none.
//-----------------------------------------------------------------------------


CTTime::CTTime (const unsigned long ulSeconds /* = 0L */, const unsigned long ulMilliSeconds /* = 0L */)
        : ulSeconds_ (ulSeconds), ulMilliSeconds_ (ulMilliSeconds)
{
    ulSeconds_      += ulMilliSeconds_ / 1000L ;
    ulMilliSeconds_ %= 1000L ;
} // CTTime::CTTime

//-----------------------------------------------------------------------------
// CTTime::CTTime
//
// Copy constructor.
//
// time   : reference to a time object.
// Return : none.
//-----------------------------------------------------------------------------
CTTime::CTTime (const CTTime& time)
        : ulSeconds_ (time.ulSeconds_), ulMilliSeconds_ (time.ulMilliSeconds_)
{
    ulSeconds_      += ulMilliSeconds_ / 1000L ;
    ulMilliSeconds_ %= 1000L ;
} // CTTime::CTTime

//-----------------------------------------------------------------------------
// CTTime::~CTTime
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTTime::~CTTime ()
{} // CTTime::CTTime

//================================================================< Operators >
//-----------------------------------------------------------------------------
// operator==
//
// time1  : reference to the first time object.
// time2  : reference to the second time object.
// Return : TRUE or CFALSE.
//-----------------------------------------------------------------------------

bool operator== (const CTTime& time1, const CTTime& time2)
{
    return ( ( (time1.ulSeconds_ + (time1.ulMilliSeconds_ / 1000L) ) == (time2.ulSeconds_ + (time2.ulMilliSeconds_ / 1000L) ) ) && ( (time1.ulMilliSeconds_ % 1000L) == (time2.ulMilliSeconds_ % 1000L) ) ) ;
} // operator==

//-----------------------------------------------------------------------------
// operator<
//
// time1  : reference to the first time object.
// time2  : reference to the second time object.
// Return : TRUE or CFALSE.
//-----------------------------------------------------------------------------
bool operator< (const CTTime& time1, const CTTime& time2)
{
    return ( ( (time1.ulSeconds_ + (time1.ulMilliSeconds_ / 1000L) ) == (time2.ulSeconds_ + (time2.ulMilliSeconds_ / 1000L) ) ) ? ( (time1.ulMilliSeconds_ % 1000L) < (time2.ulMilliSeconds_ % 1000L) ) : ( (time1.ulSeconds_ + (time1.ulMilliSeconds_ / 1000L) ) < (time2.ulSeconds_ + (time2.ulMilliSeconds_ / 1000L) ) ) ) ;
} // operator<

//-----------------------------------------------------------------------------
// operator<=
//
// time1  : reference to the first time object.
// time2  : reference to the second time object.
// Return : TRUE or CFALSE.
//-----------------------------------------------------------------------------
bool operator<= (const CTTime& time1, const CTTime& time2)
{
    return ( (time1 < time2) || (time1 == time2) ) ;
} // operator<=

//-----------------------------------------------------------------------------
// operator>
//
// time1  : reference to the first time object.
// time2  : reference to the second time object.
// Return : TRUE or CFALSE.
//-----------------------------------------------------------------------------
bool operator> (const CTTime& time1, const CTTime& time2)
{
    return ( ( (time1.ulSeconds_ + (time1.ulMilliSeconds_ / 1000L) ) == (time2.ulSeconds_ + (time2.ulMilliSeconds_ / 1000L) ) ) ? ( (time1.ulMilliSeconds_ % 1000L) > (time2.ulMilliSeconds_ % 1000L) ) : ( (time1.ulSeconds_ + (time1.ulMilliSeconds_ / 1000L) ) > (time2.ulSeconds_ + (time2.ulMilliSeconds_ / 1000L) ) ) ) ;
} // operator>

//-----------------------------------------------------------------------------
// operator>=
//
// time1  : reference to the first time object.
// time2  : reference to the second time object.
// Return : TRUE or CFALSE.
//-----------------------------------------------------------------------------
bool operator>= (const CTTime& time1, const CTTime& time2)
{
    return ( (time1 > time2) || (time1 == time2) ) ;
} // operator>=

//-----------------------------------------------------------------------------
// operator+
//
// time1  : reference to the first time object.
// time2  : reference to the second time object.
// Return : a new CTTime object.
//-----------------------------------------------------------------------------
CTTime operator+ (const CTTime& time1, const CTTime& time2)
{
    CTTime  res = time1 ;

    res += time2 ;

    return res ;
} // operator+

//-----------------------------------------------------------------------------
// operator-
//
// time1  : reference to the first time object.
// time2  : reference to the second time object.
// Return : a new CTTime object.
//-----------------------------------------------------------------------------
CTTime operator- (const CTTime& time1, const CTTime& time2)
{
    CTTime  res = time1 ;

    res -= time2 ;

    return res ;
} // operator-

//-----------------------------------------------------------------------------
// CTTime::operator=
//
// time   : reference to a time object.
// Return : *this.
//-----------------------------------------------------------------------------
CTTime& CTTime::operator= (const CTTime& time)
{
    ulSeconds_      = time.ulSeconds_ + (time.ulMilliSeconds_ / 1000L) ;
    ulMilliSeconds_ = time.ulMilliSeconds_ % 1000L;

    return *this ;
} // CTTime::operator=

//-----------------------------------------------------------------------------
// CTTime::operator=
//
// ulMilliSeconds : number of milliseconds.
// Return         : *this.
//-----------------------------------------------------------------------------
CTTime& CTTime::operator= (const unsigned long ulMilliSeconds)
{
    ulSeconds_      = ulMilliSeconds / 1000L ;
    ulMilliSeconds_ = ulMilliSeconds % 1000L;

    return *this ;
} // CTTime::operator=

//-----------------------------------------------------------------------------
// CTTime::operator+=
//
// time   : reference to a time object to add.
// Return : *this.
//-----------------------------------------------------------------------------
CTTime& CTTime::operator+= (const CTTime& time)
{
    ulSeconds_      += time.ulSeconds_ + ( (ulMilliSeconds_ + time.ulMilliSeconds_) / 1000L) ;
    ulMilliSeconds_ = (ulMilliSeconds_ + time.ulMilliSeconds_) % 1000L ;

    return *this ;
} // CTTime::operator+=

//-----------------------------------------------------------------------------
// CTTime::operator+=
//
// ulMilliSeconds : number of milliseconds.
// Return         : *this.
//-----------------------------------------------------------------------------
CTTime& CTTime::operator+= (const unsigned long ulMilliSeconds)
{
    ulSeconds_      += (ulMilliSeconds_ + ulMilliSeconds) / 1000L ;
    ulMilliSeconds_ = (ulMilliSeconds_ + ulMilliSeconds) % 1000L ;

    return *this ;
} // CTTime::operator+=

//-----------------------------------------------------------------------------
// CTTime::operator-=
//
// time   : reference to a time object to substract.
// Return : *this.
//-----------------------------------------------------------------------------
CTTime& CTTime::operator-= (const CTTime& time)
{
    unsigned long   ulSeconds       = time.ulSeconds_ + (time.ulMilliSeconds_ / 1000L) ;
    unsigned long   ulMilliSeconds  = time.ulMilliSeconds_ % 1000L ;

    ulSeconds_      += ulMilliSeconds_ / 1000L ;
    ulMilliSeconds_ %= 1000L ;

    if (ulSeconds > ulSeconds_)
    {
        ulSeconds_      = 0L ;
        ulMilliSeconds_ = 0L ;
    }
    else
    {
        ulSeconds_      -= ulSeconds ;
        if (ulMilliSeconds > ulMilliSeconds_)
        {
            if (ulSeconds_ == 0)
            {
                ulMilliSeconds_ = 0L ;
            }
            else
            {
                -- ulSeconds_ ;
                ulMilliSeconds_ += 1000L - ulMilliSeconds ;
            }
        }
        else
        {
            ulMilliSeconds_ -= ulMilliSeconds ;
        }
    }

    return *this ;
} // CTTime::operator-=

//-----------------------------------------------------------------------------
// CTTime::operator-=
//
// ulMilliSeconds : number of milliseconds.
// Return         : *this.
//-----------------------------------------------------------------------------
CTTime& CTTime::operator-= (const unsigned long ulMilliSecondsToSub)
{
    unsigned long   ulSeconds       = ulMilliSecondsToSub / 1000L ;
    unsigned long   ulMilliSeconds  = ulMilliSecondsToSub % 1000L ;

    ulSeconds_      += ulMilliSeconds_ / 1000L ;
    ulMilliSeconds_ %= 1000L ;

    if (ulSeconds > ulSeconds_)
    {
        ulSeconds_      = 0L ;
        ulMilliSeconds_ = 0L ;
    }
    else
    {
        ulSeconds_      -= ulSeconds ;
        if (ulMilliSeconds > ulMilliSeconds_)
        {
            if (ulSeconds_ == 0)
            {
                ulMilliSeconds_ = 0L ;
            }
            else
            {
                -- ulSeconds_ ;
                ulMilliSeconds_ += 1000L - ulMilliSeconds ;
            }
        }
        else
        {
            ulMilliSeconds_ -= ulMilliSeconds ;
        }
    }

    return *this ;
} // CTTime::operator-=

//-----------------------------------------------------------------------------
// CTTime::operator++
//
// Arguments : none.
// Return    : *this.
//-----------------------------------------------------------------------------
CTTime& CTTime::operator++ ()
{
    ++ ulMilliSeconds_ ;
    if (ulMilliSeconds_ >= 1000L)
    {
        ++ ulSeconds_ ;
        ulMilliSeconds_ = 0L ;
    }

    return *this ;
} // CTTime::operator++

//-----------------------------------------------------------------------------
// CTTime::operator--
//
// Arguments : none.
// Return    : *this.
//-----------------------------------------------------------------------------
CTTime& CTTime::operator-- ()
{
    if (ulMilliSeconds_ == 0L)
    {
        if (ulSeconds_ > 0L)
        {
            -- ulSeconds_ ;
            ulMilliSeconds_ = 1000L - 1L ;
        }
    }
    else
    {
        -- ulMilliSeconds_ ;
    }

    return *this ;
} // CTTime::operator--

//-----------------------------------------------------------------------------
// CTTime::operator unsigned long
//
// Arguments : none.
// Return    : an unsigned long.
//-----------------------------------------------------------------------------
CTTime::operator unsigned long () const
{
    return (ulSeconds_ * 1000L + ulMilliSeconds_ ) ;
} // CTTime::operator unsigned long

//=========================================================< Control methods >
//-----------------------------------------------------------------------------
// CTTime::now
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTTime::now ()
{
#if (defined (WIN32) || defined (__OS2__) || defined (hp800) || defined (HPUX) || defined (dgux) || defined (sun4) )
    struct timeb    time ;
    ftime (&time) ;
    ulSeconds_      = (unsigned long) time.time ;
    ulMilliSeconds_ = (unsigned long) time.millitm ;
    //#endif
#elif (defined (aix32) || defined (aix42) || defined (AIX) || defined (unixware) || defined (SOLARIS))
struct timeval  tp ;
    char            unused ;
    gettimeofday (&tp, (void*) &unused) ;
    ulSeconds_      = (unsigned long) tp.tv_sec ;
    ulMilliSeconds_ = (unsigned long) (tp.tv_usec / 1000L) ;
    //#endif

#elif (defined (dpx20) || defined (alpha) || defined (ALPHA) )
    struct timeval  tp ;
    struct timezone tzp ;

    gettimeofday (&tp, &tzp) ;
    ulSeconds_      = (unsigned long) tp.tv_sec ;
    ulMilliSeconds_ = (unsigned long) (tp.tv_usec / 1000L) ;
    //#endif

#elif defined (sun_svr4)
struct timeval  tp ;

    gettimeofday (&tp) ;
    ulSeconds_      = (unsigned long) tp.tv_sec ;
    ulMilliSeconds_ = (unsigned long) (tp.tv_usec / 1000L) ;
#endif
} // CTTime::now


char *CTTime::getPrintable()
{

    return ctime((time_t*) &ulSeconds_);
}
string  CTTime::Format(const string& fmtstr)
{
    char time_buffer[256];
    size_t len;
    len=strftime(time_buffer,256,fmtstr.c_str(),localtime((time_t*) &ulSeconds_));
    if(len==0)
        return "";
    else
        return string(time_buffer);
}

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////// End of Class CTTime //
///////////////////////////////////////////////////////////////////////////////


