/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*             **** C O M M O N ****                 */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: LIST_SUBS_WIN32                 */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*    This module contains routines which are specific to WIN32.         */
/*    There are equivalent routines for use under UNIX & VAX/VMS.        */
/*                                       */
/*===========================================================================*/

/*
 *   PSL Macros
 */
#include <psl_macros.h>

#ifdef WIN32

/*
 *   Standard ANSI header files
 */
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <io.h>

/*
 *  Function prototypes
 */
#include psl_common_file (psl_rtndef.h) /* PSL Routines */
#include "list_intrtndef.h"     /* Internal routines */

/*
 *  Structure Definitions.
 */
#include psl_common_file(shrmsg.h)  /* PSL Messages */
#include psl_common_file (pslerr_prmdef.h)
#include psl_common_file (psldiag_prmdef.h)

/*
 *  Global data
 */
#include psl_common_file (list_ctxdef.h)
#define   WIN32_PAGESIZE 66


/************************************************************************/
/*                                  */
/*    LIST__OPEN_LISTING                        */
/*    ==================                        */
/*                                  */
/*    This routine is responsible for opening the listing file      */
/*                                      */
/*    It is a WIN32 specific version of this routine.           */
/*                                  */
/*    Calling Convention:                       */
/*                                  */
/*      iret = LIST__OPEN_LISTING (context_ptr,             */
/*                 def_file_ptr,            */
/*                 rel_file_ptr)            */
/*                                  */
/*    Inputs:                               */
/*  context_ptr    = Pointer to the context block           */
/*  def_file_ptr   = Pointer to CHAR Default file name STRING   */
/*  rel_file_ptr   = Pointer to CHAR Related file name STRING   */
/*                                  */
/*    Outputs:                              */
/*                                  */
/*    Returns:                              */
/*     iret     = unsigned long error status return         */
/*                                  */
/************************************************************************/

unsigned long   LIST__OPEN_LISTING (struct list_ctx *context_ptr,
                                    const char *def_file_ptr,
                                    const char *rel_file_ptr)
{
    /*
     *    Local data
     */
    unsigned long    iret;      /* Status return variable */

    /*
     *    If the listing value is empty then we should output to stdout.
     */
    if ((*context_ptr).lstvalc256[0] != '\0')
    {

        /*
         *  Call the file name parsing routine passing the listing file name,
         *  default file and related file names and retrieving the expanded file.
         */
        iret = PSL_FILE_PARSE ((*context_ptr).lstvalc256, def_file_ptr,
                               rel_file_ptr, "",
                               (*context_ptr).prtfilc256,
                               sizeof ((*context_ptr).prtfilc256),
                               0);
        if (iret != PSL_NORMAL) return iret;

        /*
         *  Now open the file.
         */
        (*context_ptr).filepf = fopen ((*context_ptr).prtfilc256, "w");
    }
    else
    {

        /*
         *  Now open the 'terminal' listing file.
         */
        (*context_ptr).filepf = stdout;

        /*
         *  Get the current device name for the 'expanded specification'.
         */
        /*  wrk_ptr = ttyname (0); */

        /*
         *  Copy the name.
         */
        /*  (void) strcpy ((*context_ptr).prtfilc256, wrk_ptr); */
        (void) strcpy ((*context_ptr).prtfilc256, "test");
    }

    /*
     *    If we have an error then return the error status held in errno
     */
    if ((*context_ptr).filepf == NULL)
    {
        return PSL_OPENOUT;
    }
    else
    {
        return PSL_NORMAL;
    }
}

/************************************************************************/
/*                                  */
/*    LIST__CLOSE_KEEP                          */
/*    ================                          */
/*                                  */
/*    This routine is responsible for closing the listing file      */
/*                                      */
/*    It is a WIN32 specific version of this routine.           */
/*                                  */
/*    Calling Convention:                       */
/*                                  */
/*      iret = LIST__CLOSE_KEEP (context_ptr)               */
/*                                  */
/*    Inputs:                               */
/*  context_ptr    = Pointer to the context block           */
/*                                  */
/*    Outputs:                              */
/*  None                                */
/*                                  */
/*    Return Status:                            */
/*  Unsigned long PSL error status                  */
/*                                  */
/************************************************************************/

unsigned long   LIST__CLOSE_KEEP (struct list_ctx *context_ptr)
{
    /*
     *    Local data
     */
    long   status;

    /*
     *    Close the listing file - if not to stdout
     */
    if ((*context_ptr).filepf != stdout)
    {
        status = fclose ((*context_ptr).filepf);
        if (status == EOF)
        {
            return PSL_CLOSEOUT;
        }
        else
        {
            return PSL_NORMAL;
        }
    }
    else
    {
        return PSL_NORMAL;
    }
}

/************************************************************************/
/*                                  */
/*    LIST__CLOSE_PRINT                         */
/*    =================                         */
/*                                  */
/*    This routine is responsible for closing the listing file      */
/*    and spooling it to the print queue                    */
/*                                  */
/*    It is a WIN32 specific version of this routine.           */
/*                                  */
/*    Calling Convention:                       */
/*                                  */
/*      iret = LIST__CLOSE_PRINT (context_ptr)              */
/*                                  */
/*    Inputs:                               */
/*  context_ptr    = Pointer to the context block           */
/*                                  */
/*    Outputs:                              */
/*  None                                */
/*                                  */
/*    Return Status:                            */
/*  Unsigned long PSL status                    */
/*                                  */
/************************************************************************/

unsigned long   LIST__CLOSE_PRINT (struct list_ctx *context_ptr)
{
    /*
     *    Local data
     */
    long   status;

    /*
     *    Close the listing file.
     */
    if ((*context_ptr).filepf != stdout)
    {
        status = fclose ((*context_ptr).filepf);
        if (status == EOF)
        {
            return PSL_CLOSEOUT;
        }
        else
        {
            return PSL_NORMAL;
        }
    }
    else
    {
        return PSL_NORMAL;
    }
}


/************************************************************************/
/*                                  */
/*    LIST__WRITE                           */
/*    ===========                           */
/*                                  */
/*    This routine is responsible for writing a record to the       */
/*    listing file                          */
/*                                  */
/*    It is a WIN32 specific version of this routine.           */
/*                                  */
/*    Calling Convention:                       */
/*                                  */
/*  LIST__WRITE (context_ptr, write_type, string, number)       */
/*                                  */
/*    Inputs:                               */
/*  context_ptr = Pointer to the context block          */
/*  write_type  = Unsigned long "Type" of write this is (1 to 5)    */
/*  string      = Pointer to CHAR string to write           */
/*  number      = Unsigned long value (page or line number)     */
/*                                  */
/*    Outputs:                              */
/*  None                                */
/*                                  */
/*    Returns:                              */
/*  unsigned long PSL status value                  */
/*                                  */
/************************************************************************/

unsigned long   LIST__WRITE (struct list_ctx *context_ptr,
                             unsigned long write_type,
                             const char *string_ptr,
                             unsigned long number)
{
    /*
     *    Local data
     */
    long    status;        /* Status return variable */
    char    temp_buf [256];    /* Temporary buffer for string */
    unsigned long    length;        /* Length of string */
    unsigned long    isub;          /* Loop counter */

    /*
     *    Strip off any trailing non-printing characters or spaces in the string
     */
    if (write_type != 5)        /* Only if there is a string */

    {
        length = strlen (string_ptr);   /* Length of string to output   */
        for (isub = length - 1;
                ((isub != 0)
                 && (*(string_ptr+isub) == ' ')
                 && (isprint (*(string_ptr+isub)) == 0));
                isub--);           /* Loop stripping spaces and    */
        /* non-printing characters  */
        length = isub + 1;      /* Calculate length     */
        (void) memcpy (temp_buf, string_ptr, length);
        /* Copy this much to local buffer */
        temp_buf [length] = '\0';   /* ...and null terminate it.    */
    }

    /*
     *    Write the char string formatted to the listing file depending upon
     *    the value of the write_type
     */
    switch (write_type)
    {
    case 1:
        {
            status = fprintf ((*context_ptr).filepf, "        %s\n", temp_buf);

            break;
        }

    case 2:
        {
            status = fprintf ((*context_ptr).filepf , "%5ld   %s\n"
                              ,number, temp_buf);
            break;
        }

    case 3:
        {
            status = fprintf ((*context_ptr).filepf, "\f\n%s\n", temp_buf);

            break;
        }

    case 4:
        {
            status = fprintf ((*context_ptr).filepf, "%s\n", temp_buf);

            break;
        }

    case 5:
        {
            status = fprintf ((*context_ptr).filepf, "\n");
            break;
        }

    default:
        {
            break;
        }
    }

    /*
     *    Now check the status of the writes.
     */
    if (status < 0)         /* If we have had an error */
    {
        return PSL_WRITEERR;        /* return an error status */
    }
    else
    {
        return PSL_NORMAL;      /* ...else return success */
    }
}

/************************************************************************/
/*                                  */
/*    LIST__GET_PAGESIZE                        */
/*    ==================                        */
/*                                  */
/*    This routine is responsible for returning the size of the page    */
/*    in lines.                             */
/*                                      */
/*    It is a WIN32 specific version of this routine.           */
/*                                  */
/*    Calling Convention:                       */
/*                                  */
/*      lines = LIST__GET_PAGESIZE ()                   */
/*                                  */
/*    Inputs:                               */
/*  None                                */
/*                                  */
/*    Outputs:                              */
/*  None                                */
/*                                  */
/*    Return Status:                            */
/*  lines = Number of lines on page                 */
/*                                  */
/************************************************************************/

unsigned short  LIST__GET_PAGESIZE (void)
{
    /*
     *    The number of lines on a WIN32 page is currently hardwired to 66
     */
    return WIN32_PAGESIZE;
}


/************************************************************************/
/*                                  */
/*    LIST__FILE_ERROR                          */
/*    ================                          */
/*                                  */
/*    Calling Convention:                       */
/*                                  */
/*  LIST__FILE_ERROR (diag_ptr, mess_no, filename_ptr)      */
/*                                  */
/*    Inputs:                               */
/*  diag_ptr     = Pointer to diagnostics stack             */
/*  mess_no      = UNSIGNED LONG message identifier         */
/*  filename_ptr = Pointer to Expanded file specification       */
/*                                  */
/*    Outputs:                              */
/*  None                                */
/*                                  */
/*    Returns:                              */
/*  None                                */
/*                                  */
/*    This routine is called to report a file error under WIN32     */
/*                                  */
/************************************************************************/

void   LIST__FILE_ERROR (struct diagstk *diag_ptr,
                         unsigned long mess_no,
                         char *filename_ptr)
{
    /*
     *  Output the appropriate error.
     */
    PSL_LOG_CIOERR (diag_ptr, mess_no, filename_ptr);
    PSL_SIGNAL_ERROR (diag_ptr, PSLERRCLS_LIST, mess_no);

    return;
}
#endif
