/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/* scl_subs_unix.c:
 * ================
 *
 * This module contains all the UNIX-specific routines used by SCL that
 * do not belong in any other SCL module.
 */

/*
 *  PSL Macros.
 */
#include <psl_macros.h>
#include <vtest_macros.h>

/*
 *  Standard ANSI header files.
 */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>

/*
 *  Function Prototypes.
 */
#include <psl_rtndef.h>         /* PSL Routines */
#include "scl_rtndef.h"         /* SCL Routines */

/*
 *  Structure definitions and parameters.
 */
#include <shrmsg.h>         /* PSL messages */
#include "scl_cmd.h"            /* Command Line Data Structure */
#include "scl_comdef.h"         /* Global Data structure */
#include <psldiag_prmdef.h>     /* Diagnostics paramters */
#include <pslspn_prmdef.h>      /* PSL_SPAWN symbols */

#define BLOCK_SIZE 512          /* Bytes per block */

/************************************************************************/
/*                                  */
/*  SCL_GETNEXT_FILE - Get next file specified on command line  */
/*  =================                       */
/*                                  */
/*  Calling Convention:                     */
/*                                  */
/*     iret = SCL_GETNEXT_FILE (scl_com_ptr);           */
/*                                  */
/*  Where:                              */
/*                                  */
/*     iret     = unsigned long routine status          */
/*          = PSL_NORMAL - if success           */
/*          = PSL_ENDMATCH - if no more files match     */
/*          = PSL_OPENIN - if unable to open next file  */
/*          = any return from SCL_TRIM_FILESPEC     */
/*     scl_com_ptr  = Pointer to the SCL Common Data Record     */
/*                                  */
/*  This routine is called to pick up the file name of the next     */
/*  file to compile. It is based on the name specified on the   */
/*  command line.                           */
/*                                  */
/************************************************************************/
unsigned long SCL_GETNEXT_FILE (struct scl_com *scl_com_ptr)
{
    /*
     *  Local data
     */
    unsigned long       iret;       /* Status Return variable         */
    char       *src_name_ptr;  /* Pointer to Source File Name    */
    struct   scl_cmd    *scl_cmd_ptr;   /* Pointer to Command Line Struct */

    static Boolean      first_time = PTRUE; /* First time through flag        */
    static   char       **src_ptr_ptr;  /* Pointer to pointer to string   */
    static unsigned long scl_count;     /* SCL file count                 */

    /*
     *  Copy pointers for ease of reference.
     */
    scl_cmd_ptr = (*scl_com_ptr).cmd_ptrps;

    /*
     *  If first time, initialize pointer to the source file list and
     *  clear count.
     */
    if (first_time)
    {
        src_ptr_ptr = (*scl_cmd_ptr).scl_value_ptrppc;
        scl_count = 0;
        first_time = PFALSE;
    }

    /*
     *  If there are no more files return status to indicate thsi.
     */
    if (scl_count == (*scl_cmd_ptr).scl_value_countl)
    {
        iret = PSL_ENDMATCH;
        goto end;
    }

    /*
     *  Copy source file name to buffers in Common Data Structure.
     *  Let's hope these buffers are big enough!
     */
    src_name_ptr = *src_ptr_ptr;

    (void) strcpy ((*scl_com_ptr).input_namec256, src_name_ptr);

    (void) strcpy ((*scl_com_ptr).scl_namec256, src_name_ptr);

    /*
     *  Increment counter and point to next pointer.
     */
    scl_count += 1;
    src_ptr_ptr += 1;

    /*
     *  Trim the filespec down 'sensibly' if required.
     */
    if (strlen ((*scl_com_ptr).scl_namec256) > 40)
        /* If trimming required           */
    {
        iret = PSL_TRIM_FILESPEC ((*scl_com_ptr).scl_namec256,
                                  (*scl_com_ptr).trimmed_namec40, 39,0);
        /* Get NULL-terminated trimmed string */
        if (!(psl_test_status (iret))) goto end;
    }
    else            /* Else trimming not required         */
    {
        (void) strcpy ((*scl_com_ptr).trimmed_namec40,
                       (*scl_com_ptr).scl_namec256);
        /* Just copy file name            */
    }

    iret = PSL_NORMAL;      /* Signal Success             */

    /*
     *  END:
     *
     *  Routine exit point.
     */
end:
    return iret;
}
