/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TFakeRecords.cpp
/**
 * File to contain all the method definitions of those classes dervived
 * from TRecord's.
 *
 * The class methods in this file are intended as examples and also to
 * allow the atvf functionality to be built into working test utilities
 * within the package
 **/
#include "TFakeRecords.h"
#include "TRecMake.h"

/**
 * The keys for our 'real' records,  these examples come the tof
 * specification
 * 'static' so they must come before any methods.
 **/
const AtvfKey TRcrdConn::KEY("Connect");
const AtvfKey TRcrdLang::KEY("Languge");
const AtvfKey TRcrdDisc::KEY("Disconnect");

/**
 * C linkage procedure required by TScriptRule
 * this can recognise a transaction key and match to one of below
 * known/real TRecord type.
 **/
SharedPtr<TRecCreator> getTRecCreator(const AtvfKey& key)
{
    SharedPtr<TRecCreator> creatorP;
    if(key == TRcrdConn::KEY)
    {
        creatorP = SharedPtr<TRecCreator>(new TRecMake<TRcrdConn>());
    }
    else if(key == TRcrdLang::KEY)
    {
        creatorP = SharedPtr<TRecCreator>(new TRecMake<TRcrdLang>());
    }
    else if(key  == TRcrdDisc::KEY)
    {
        creatorP = SharedPtr<TRecCreator>(new TRecMake<TRcrdDisc>());
    }
    else
    { // Any record we don't recognise
        creatorP = SharedPtr<TRecCreator>(new TRecMake<TRecord>());
    }
    return creatorP;
}

/**
 * Execute method for a Connect Record
 **/
int TRcrdConn::execute(TContext& ctxt)
{
    return 0;
}

/**
 * Execute method for a Language Record
 **/
int TRcrdLang::execute(TContext& ctxt)
{
    return 1;
}

/**
 * Execute method for a Language Record
 **/
int TRcrdDisc::execute(TContext& ctxt)
{
    return 0;
}


