/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TRecord.h
#ifndef _CYR_TRECORD_H
#define _CYR_TRECORD_H
/**
 * Class definition for a Transaction Record
 **/
#ifdef WIN32
#   pragma warning(disable:4786)
#endif

#include <string>
#include <vector>
#include <set>

#ifndef _WIN32_WINNT
#   define _WIN32_WINNT 0x0404
#endif
#ifndef WINVER
#   define WINVER       0x0400
#endif
#include "windows.h"
#include "FastLock.hpp"


#ifdef unix
#   elif defined (__VMS)
#   elif defined (WIN32)

using namespace std;
//++
class TRecord;
ostream& operator<<(ostream&, const TRecord*);
//--
#else //unknown
#   error Unknown Platform
#endif // unknown

// Define mentioned classes that we won't be using
class TContext;
class Cresultset;
class TScript;
class CRegExp;


// Local includes
#include "AtvfRule.h"
#include "TParam.h"

typedef vector<AtvfRecord*> recp_cont_t;
typedef vector<TParam*> prmp_vect_t;

typedef enum { TOF_RES_OK=0, TOF_RES_FAILED=-1, TOF_RES_ASYNC_STARTED=-2, TOF_SKIP_TO_NEXT=-3, TOF_RES_OVER=-4, TOF_RES_KILL_THREAD=-5, TOF_RES_FINISHED=-6 } ETofResult;

class TRecord
{
    TParam* _headPrmP;
    AtvfRecord* _headRecP;
    AtvfRecord* _delimRecP;
    recp_cont_t _recs;
    TOptions _options; // Only really used if no _headPrmP
    prmp_cont_t _params;
    prmp_vect_t _prmidx;

    Cresultset* _results; // Results obtained by running underlying command

protected:
    static CFastLock *m_pFastLock;

public:
    // constructors & destructor
    TRecord(const TParamRule* headRuleP = NULL);
    TRecord(const TRecord& trec);
    virtual ~TRecord();

    // Initialization / Uninitialization.
    static bool InitClass();    // Must be called on program startup.
    static void UninitClass();  // Must be called on program completion.

    // accessors
    const TOptions& options() const;


    Cresultset* Results() const;
    virtual string token() const;
    virtual bool isType(const AtvfKey& key) const;
    virtual bool isKnown() const;
    virtual bool isDeclaration() const;
    virtual bool findInVal(const string& text,
                           int& prmidx,
                           bool matchCase = true,
                           bool forward = true) const;

    virtual bool findInVal(CRegExp& regex,
                           int& prmidx,
                           bool matchCase = true,
                           bool forward = true) const;
    virtual void save(ostream& out) const;
    virtual void saveDelim(ostream& out) const;
    virtual const TParam* param(const string& key) const;
    virtual const TParam* param(unsigned int idx) const;
    virtual int intPrmVal(const string& key,
                          TContext& ctxt,
                          unsigned int idx = 0) const;
    virtual string strPrmVal(const string& key,
                             TContext& ctxt,
                             unsigned int idx = 0) const;
    virtual unsigned int numParams() const;
    virtual TRecord* duplicate() const;

    // mutators


    void SetResults(Cresultset* results);

    virtual int execute(TContext& ctxt);

    virtual int prepare(TScript* script, int objIndex, bool verbose=false);

    virtual void addOptions(const TOptions& opts);
    virtual void addParam(TParamRule* prP);
    virtual void addHead(AtvfRecord* recP = NULL);
    virtual bool addData(AtvfRecord* recP);
    void deleteData(AtvfRecord* recP);
    virtual void setPrmVal(const string& val,
                           const string& key,
                           unsigned int idx = 0);

    // friends
    friend ostream& operator<<(ostream& os, const TRecord* rP);

private:
    //// Forbidden
    operator=(const TRecord&);
}
; // TRecord

ostream& operator<<(ostream& os, const TRecord* rP);
#endif // _CYR_TRECORD_H

