/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "StdAfx.h"

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

#include "CECInterProcessVar.h"
#include "TEMainStructure.hpp"

extern CMainExecuterStructure *g_pTExecuter;

CECInterProcessVar::CECInterProcessVar(const string& name, int size, const string& file,
    const string& fileid, const string& dimensions, const string& values, int options,
    unsigned long seed, const string& scriptname)
        : CECVar(name),_options(options),_size(size)
{
    // Connect to the Correct Variable Factory depending on scope.
    CECInetInfo inet;
    CORBA::Object_var obj = pCServer->FindObjectWithName(VarFactory::CLASS_NAME,
        inet.GetIPAddressAsString(g_pTExecuter->GetRepositoryHost()).c_str());
    VarFactory_var VF=VarFactory::_narrow(obj.inout());

    //Create a New GlobalFileVar_ptr and assign to member variable
    CORBA::String_var cb_name=CORBA::string_dup(name.c_str());
    CORBA::String_var cb_file=CORBA::string_dup(file.c_str());
    CORBA::String_var cb_fileid=CORBA::string_dup(fileid.c_str());
    CORBA::String_var cb_dimensions=CORBA::string_dup(dimensions.c_str());
    CORBA::String_var cb_values=CORBA::string_dup(values.c_str());
    CORBA::String_var cb_scriptname=CORBA::string_dup(scriptname.c_str());
    obj=VF->GetVariable(cb_name.in(),_size,cb_file.in(),cb_fileid.in(),cb_dimensions.in(),cb_values.in(),_options,seed,cb_scriptname.in());
    m_variable=GlobalVariable::_narrow(obj.inout());
}
CECInterProcessVar::~CECInterProcessVar()
{
    //Released Reference to Global Variable
}


string  CECInterProcessVar::get_value(const string& expression)
{
    //Return the value stored in CORBA member variable
    CORBA::String_var cb_expression=CORBA::string_dup(expression.c_str());
    CORBA::String_var Cstr=m_variable->get_value(cb_expression.in());
    string retstr(Cstr);
    return retstr;
}
int CECInterProcessVar::set_value(const string& newval,const string& expression)
{
    CORBA::String_var cb_newval=CORBA::string_dup(newval.c_str());
    CORBA::String_var cb_expression=CORBA::string_dup(expression.c_str());
    // Should Set Value be allowed on a range variable!!!!!!
    //Set the value in the CORBA member variable
    m_variable->set_value(cb_newval.in(),cb_expression.in());
    return 0;
}
void    CECInterProcessVar::next_value()
{
    m_variable->next_value();
}
void    CECInterProcessVar::generate_value()
{
    m_variable->generate_value();
}
void    CECInterProcessVar::reset_seed(unsigned long seed)
{
    m_variable->reset_seed();
}
