/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#include "StdAfx.h"

#include "CECMutex.h"
#include "TEMainStructure.hpp"

extern CORBAServer*		pCServer;//This has become a Global variable due to
extern CMainExecuterStructure *g_pTExecuter;

//compilation problems if included in some headers
CECMutex::CECMutex(const string& name,int scope)
#ifndef CYRANO_TP_STANDALONE
    :   m_key(0)
#endif
{
#ifndef CYRANO_TP_STANDALONE
    string strName(name);
    switch(scope)
    {
    case TOF_MTXSCP_TESTWIDE:
        
        strName=g_pTExecuter->GetTestRun()+"."+name+".TEST-WIDE";
        break;
    case TOF_MTXSCP_LOCAL:
    default:
        strName=g_pTExecuter->GetTestRun()+"."+name+".LOCAL";
        break;
    }

    CORBA::Object_var obj;
    try
    {
        obj=pCServer->FindObjectWithName(TPMutex::CLASS_NAME,strName.c_str());
    }
    catch(...)//no mutex of that name available so get one from TestManager Variable Factory
    {
        CECInetInfo inet;
        CORBA::Object_var Ofac = pCServer->FindObjectWithName(VarFactory::CLASS_NAME,
            inet.GetIPAddressAsString(g_pTExecuter->GetRepositoryHost()).c_str());
        VarFactory_var VF=VarFactory::_narrow(Ofac.inout());
        CORBA::String_var cb_name=CORBA::string_dup(strName.c_str());
        obj=VF->CreateMutex(cb_name.in());

    }
    m_mutex=TPMutex::_narrow(obj.inout());
#endif
}
CECMutex::~CECMutex()
{}

bool CECMutex::lock()
{
#ifndef CYRANO_TP_STANDALONE
    m_key=m_mutex->lock();
#else
    m_mutex.Lock();
#endif
    return true;
}

bool CECMutex::unlock()
{
#ifndef CYRANO_TP_STANDALONE
    m_mutex->unlock(m_key);
#else
    m_mutex.Unlock();
#endif
    return true;
}

#ifndef CYRANO_TP_STANDALONE

bool CECMutex::trylock()
{
    m_key=m_mutex->trylock();
    return m_key != -1; //failed to lock
}

#endif
