/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "StdAfx.h"

#include "CECSemaphore.h"
#include "TEMainStructure.hpp"

extern CORBAServer*		pCServer;//This has become a Global variable due to
extern CMainExecuterStructure *g_pTExecuter;

//compilation problems if included in some headers
CECSemaphore::CECSemaphore(const string& name,int scope)
{
    string strName;
    switch(scope)
    {
    case TOF_SEMSCP_TESTWIDE:
        strName=g_pTExecuter->GetTestRun()+"."+name+".TEST-WIDE";
        break;
    case TOF_SEMSCP_LOCAL:
    default:
        strName=g_pTExecuter->GetTestRun()+"."+name+".LOCAL";
        break;
    }

    CORBA::Object_var obj;
    try
    {
        obj=pCServer->FindObjectWithName(TPSemaphore::CLASS_NAME,strName.c_str());
    }
    catch(...)//no Semaphore of that name available so get one from TestManager Variable Factory
    {
        CECInetInfo inet;
        CORBA::Object_var Ofac = pCServer->FindObjectWithName(VarFactory::CLASS_NAME,
            inet.GetIPAddressAsString(g_pTExecuter->GetRepositoryHost()).c_str());
        VarFactory_var VF=VarFactory::_narrow(Ofac.inout());
        CORBA::String_var cb_name=CORBA::string_dup(strName.c_str());
        obj=VF->CreateSimpleSemaphore(cb_name.in());//Create CYRANO Test type semaphore

    }

    m_Semaphore=TPSemaphore::_narrow(obj.inout());
}
CECSemaphore::~CECSemaphore()
{}

bool CECSemaphore::acquire()
{
    m_Semaphore->acquire();
    return true;
}
bool CECSemaphore::release()
{
    m_Semaphore->release();
    return true;
}
bool CECSemaphore::tryacquire()
{
    bool bret=false;
    bret=m_Semaphore->tryacquire();
    return bret;
}
