/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"

#include "TestExecuter_i.h"

#ifndef CYRANO_TP_STANDALONE
#   include "InjectorControl.h"
#endif

#include "AsyncVirtualUserController.h"
#include "TEMainStructure.hpp"


#include "ctime.hpp"

extern CAsyncVirtualUserController* _threadPoolController;
extern CMainExecuterStructure *g_pTExecuter;

CORBAServer* pCServer = NULL;


TestExecuterImpl::TestExecuterImpl(CORBAServer* pserver)
    : m_pServer(pserver)
#ifndef CYRANO_TP_STANDALONE
    , m_pObjid(NULL), m_bOpen(TRUE)
#endif
{
    char strReg[64];
    CECInetInfo inet;
    pCServer = m_pServer;
    m_pObjid = m_pServer->ActivateObject(this);
    wsprintf(strReg, "%s_%8.8lX", inet.GetIPAddressAsString().c_str(), GetCurrentProcessId());
    m_strRegisterName = strReg;
    m_pServer->NameObject(this->_this(), TestExecuter::CLASS_NAME, strReg);

    // Connect to the injector control object.
	CORBA::Object_ptr obj = m_pServer->FindObjectWithName(InjectorControl::CLASS_NAME, inet.GetIPAddressAsString().c_str());
	m_pControl = InjectorControl::_narrow(obj);
}

TestExecuterImpl::~TestExecuterImpl()
{
    // Disconnect from the injector control object.
    if(m_pControl)
    {
        CORBA::release(m_pControl);
        m_pControl = NULL;
    }

    m_pServer->UnNameObject(TestExecuter::CLASS_NAME,m_strRegisterName.c_str());
    Deactivate();
}

void TestExecuterImpl::Deactivate()
{
    if(m_pObjid)
    {
        m_pServer->DeActivateObject(*m_pObjid);
        m_pObjid = NULL;
    }
}

CORBA::Boolean TestExecuterImpl::StartTaskGroup(TaskGroupExecInfo& tgExec)
{
    if(!m_bOpen)
        return false;

    char zTgNum[64];
    wsprintf(zTgNum, "Incoming call: start task group %d.", tgExec.lTaskGroupId);
    g_pTExecuter->TraceMsg(zTgNum);

    // Create internal task group structure.
    TaskGroupDefinition* pTaskGroup = new TaskGroupDefinition(
        (int)tgExec.lTaskGroupId,           // iTaskGroupNumber
             tgExec.lTaskGroupDuration,     // lTaskGroupDuration
             tgExec.lNumVUs,                // lNumVirtualUsers
             tgExec.lTaskGroupIterations,   // lNumTaskGroupIterations
        (int)tgExec.lInterBatchDelay,       // iInterBatchDelay
             tgExec.lVUsPerBatch,           // lVUsPerBatch
        (int)tgExec.lBatchRampUpTime,       // iBatchRampUpTime
        (TaskGroupDefinition::LogLev)tgExec.lStatsLevel, // iLoggingLevel
             tgExec.bUsePageTimers);        // bUsePageTimers

    // Add the tasks.
    CORBA::ULong ulLength = tgExec.seqTasks.length();
    for(CORBA::ULong ul = 0; ul < ulLength; ++ul)
    {
        TaskExecInfo& tei = tgExec.seqTasks[ul];
        if(tei.bIsFixedDelay)
        {
            TaskGroupDefinition::TaskDefinition task(
                (const char*)tei.strScriptName,                     // Script name.
                             tei.bIsDuration ? tei.lDuration : 0,   // Duration.
                             tei.bIsDuration ? 0 : tei.lIterations, // Iterations.
                (int)        tei.lFixedDelay);                      // Fixed delay.

            pTaskGroup->m_tasks.push_back(task);
        }
        else
        {
            TaskGroupDefinition::TaskDefinition task(
                (const char*)tei.strScriptName,                     // Script name.
                             tei.bIsDuration ? tei.lDuration : 0,   // Duration.
                             tei.bIsDuration ? 0 : tei.lIterations, // Iterations.
                (int)        tei.lMinDelay,                         // Min delay.
                (int)        tei.lMaxDelay);                        // Max delay.

            pTaskGroup->m_tasks.push_back(task);
        }
    }

    return g_pTExecuter->StartTaskGroup(SharedPtr<TaskGroupDefinition>(pTaskGroup)) != FALSE;
}

CORBA::Boolean TestExecuterImpl::StopTaskGroup(CORBA::Long lTaskGroupId)
{
    if(!m_bOpen)
        return false;

    char zTgNum[64];
    wsprintf(zTgNum, "Incoming call: stop task group %d.", lTaskGroupId);
    g_pTExecuter->TraceMsg(zTgNum);

    g_pTExecuter->StopTaskGroup((int)lTaskGroupId);
    return true;
}

CORBA::Boolean TestExecuterImpl::Stop()
{
    if(!m_bOpen)
        return false;

    // Stop accepting incoming calls.
    InterlockedExchange((LPLONG)&m_bOpen, (LONG)FALSE);

    g_pTExecuter->TraceMsg("Incoming call: stop Test Executer.");

    g_pTExecuter->Stop();
    return true;
}

CORBA::Boolean TestExecuterImpl::GetStatsSnapshot(TestExecuterSnapshot& teSnapshot)
{
    if(!m_bOpen)
        return false;

    g_pTExecuter->TraceMsg("Incoming call: get task group statistics snapshot.");

    // Get the snapshot.
    CAsyncVirtualUserController::TestExecuterSnapShot sn;
    CORBA::Boolean bOK = g_pTExecuter->GetStatsSnapShot(sn) != FALSE;

    // Convert the type.
    teSnapshot.lNumberOfActiveVUs = sn.m_lNumberOfActiveVUs;
    CORBA::ULong ulLength = sn.m_taskGroupSnapShots.size();
    teSnapshot.seqTaskGroupSnaps.length(ulLength);
    if(ulLength)
    {
        CAsyncVirtualUserController::TaskGroupSnapShots::const_iterator pTsn = sn.m_taskGroupSnapShots.begin();
        for(CORBA::ULong ul = 0; ul < ulLength; ++ul)
        {
            TaskGroupSnap& tgsn       = teSnapshot.seqTaskGroupSnaps[ul];
            tgsn.lTaskGroupId         = pTsn->first;
            tgsn.lCompletedIterations = pTsn->second.m_lCompletedIterations;
            tgsn.lRunTime             = pTsn->second.m_lRunTime;
            tgsn.lBytes               = pTsn->second.m_lBytes;
            tgsn.lRequests            = pTsn->second.m_lRequests;
            CORBA::ULong ulNbTask     = pTsn->second.m_activeUsers.size();
            CORBA::Long lActiveUsers  = 0;
            tgsn.seqTaskSnaps.length(ulNbTask);
            if(ulNbTask)
            {
                std::vector<long>::const_iterator pAU = pTsn->second.m_activeUsers.begin();
                for(CORBA::ULong ulTask = 0; ulTask < ulNbTask; ++ulTask)
                {
                    TaskSnap& ts    = tgsn.seqTaskSnaps[ulTask];
                    ts.lTaskId      = ulTask;
                    ts.lActiveUsers = *pAU;
                    lActiveUsers    += ts.lActiveUsers;

                    ++pAU;
                }
            }
            tgsn.lActiveUsers = lActiveUsers;

            ++pTsn;
        }
    }

    return bOK;
}

CORBA::Long TestExecuterImpl::GetTEVUs(CORBA::Long lPreviousTestVUCount)
{
    if(!m_bOpen)
        return 0;

    char zTgNum[128];

    // Compute the VU count for this Test Executer.
    CORBA::Long lVUs = g_pTExecuter->GetTestExecuterVUs();

    // If the previous count reports less than the current number of VUs in this
    // Test Executer, than this latter count is surely more accurate!
    if(lVUs > lPreviousTestVUCount)
        lPreviousTestVUCount = lVUs;

    // Cache the previous VU count for the whole test.
    g_pTExecuter->SetCachedTestVUCount(lPreviousTestVUCount);

    wsprintf(zTgNum, "Incoming call: get number of active VUs. Previous value = %ld. Returning %ld.", lPreviousTestVUCount, lVUs);
    g_pTExecuter->TraceMsg(zTgNum);

    return lVUs;
}

CORBA::Boolean TestExecuterImpl::GetTaskSnapshot(const TaskIdList& taskIDs, TaskSnap2List& taskSnapshots)
{
    g_pTExecuter->TraceMsg("Incoming call: get task statistics snapshot.");
    return g_pTExecuter->GetTaskSnapshot(taskIDs, taskSnapshots) != FALSE;
}
