/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// LocalRepository.cpp: implementation of the CLocalRepository class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "RegFunctions.h"
#include "LocalRepository.h"
#include "ManagerMain.h"
#include "OstaTraceDef.h"
#include <CECInetInfo.h>
#include <ctime.hpp>
#include <iostream>
using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
#define SERVERNAME_LITERAL "Server"
const string TESTRUNSTRING_SEP="~";

CLocalRepository::CLocalRepository(CORBAServer* corba, string strTest, int nPID)
{
	m_pCorba = corba;
	m_strTest = strTest;
	m_nPID = nPID;
	m_pRootFolder = NULL;
	m_pProject = NULL;
	m_pReposServer = NULL;
	m_pTestFolder = NULL;
	m_pTestStoreFolder = NULL;
	m_pTestRun = NULL;
	m_strTestRun = "";

	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Constructing Local Repository");
    CECInetInfo info;
    string underscore_ip_name=info.GetIPAddressAsString();
	
    string factory=underscore_ip_name+".CyranoFactory";
    string dataservername=underscore_ip_name+"_CyrStProvider_001";
    string msg="Node "+underscore_ip_name+" is not running a DataServer";

	// Attempt to connect to factory on the repository host machine
    try
    {
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Attempt to connect to factory");
		CORBA::Object_ptr obj;
		obj=m_pCorba->FindObjectWithName(Provider::CLASS_NAME,dataservername.c_str());
    }
    catch(...)
    {
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception connecting to factory");
    }

	// Create the OrbServer and create connections to the main folders
    try
    {
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Connect to repository");
		m_pReposServer = new CECOrbServer(m_pCorba, underscore_ip_name.c_str(), "", "");
		m_pProject = new CECOrbProject(m_pReposServer, "Repository");
		m_pRootFolder = new CECOrbFolder(m_pProject);
		m_pTestFolder = new CECOrbFolder(m_pRootFolder, "Tests");
		m_pTestStoreFolder = new CECOrbFolder(m_pTestFolder, strTest.c_str());
    }
    catch(...)
    {
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "CLocalRepository::CLocalRepository");
    }

	// Create TestRun
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, "Create TestRun folder");
	MakeActualTestRun();
}

CLocalRepository::~CLocalRepository()
{
	try
	{
		m_pCorba->UnNameObject(CyrStTestRun::CLASS_NAME,m_strTestRun.c_str());

		if (!CORBA::is_nil(m_pTR))
		{
			CORBA::release(m_pTR);
		}

		if (m_pTestRun)
			m_pTestRun = NULL; // DONT DELETE: CAUSES RESULTS TO GO TO INCORRECT TESTRUN
		if (m_pTestStoreFolder)
			delete m_pTestStoreFolder;
		if (m_pTestFolder)
			delete m_pTestFolder;
		if (m_pRootFolder)
			delete m_pRootFolder;
		if (m_pProject)
			delete m_pProject;
		if (m_pReposServer)
			delete m_pReposServer;
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE,"Exception: CLocalRepository::~CLocalRepository");
	}
}

void CLocalRepository::MakeActualTestRun()
{
	try
	{
		CTTime time;
		time.now();
		string desc="Started at ";
		desc+=time.Format("%H:%M:%S %d-%m-%Y");

		char *c_pid=new char[64];
		(void)sprintf(c_pid,"%d",m_nPID);

		m_pTestRun = new CECOrbTestRun(m_pTestStoreFolder ,desc.c_str(), "localhost",c_pid);

		delete c_pid;

		//Register this new testrun with naming service for pick up by testexecuters
		m_pTR = m_pTestRun->GetServant();
		m_strTestRun = GetTestRunString();
	
		string strMsg = "Register CyrStTestRun_var with name ";
		strMsg += m_strTestRun;
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_MEDIUM, strMsg);
		m_pCorba->NameObject(m_pTR, CyrStTestRun::CLASS_NAME,m_strTestRun.c_str());
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE,"Exception: CLocalRepository::MakeActualTestRun");
	}
}

string CLocalRepository::GetTestRunString()
{
	if (m_strTestRun != "")
		return m_strTestRun;

	m_strTestRun = SERVERNAME_LITERAL;
    m_strTestRun += TESTRUNSTRING_SEP + m_pProject->GetName() + TESTRUNSTRING_SEP + m_pTestFolder->GetName();
	m_strTestRun += TESTRUNSTRING_SEP + m_pTestStoreFolder->GetName() + TESTRUNSTRING_SEP + m_pTestRun->GetName();

	return m_strTestRun;
}

string CLocalRepository::GetRunFolderName()
{
	if(m_pTestRun)
		return m_pTestRun->GetName();
	return "";
}

long CLocalRepository::GetNumResultsFiles()
{
	long nFiles = 0;
	try
	{
		// Get TestRunFolder Path
		CString strTRPath = GetRepositoryPath();
		strTRPath += "Tests\\";
		strTRPath += m_strTest.c_str();
		strTRPath += "\\";
		strTRPath += m_pTestRun->GetName().c_str();

		CFileFind FF;
		if(!FF.FindFile(strTRPath + "\\*.*"))
		return FALSE;

		BOOL bContinue=true;
		while(bContinue)
		{
			bContinue = FF.FindNextFile();
			if(FF.IsDots() || FF.IsDirectory())
				continue;
			else
				nFiles++;
		}
		FF.Close();	
	}
	catch(...)
	{
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Exception: CLocalRepository::GetNumResultsFiles");
	}
	return nFiles;
}
