/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// DataNameCtrls.cpp : implementation file
//

#include "stdafx.h"
#include "DataNameCtrls.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

inline BOOL CheckDataName(char *szText,int nMaxLength)
{
	int nLength = strlen(szText);
	if(!nLength)
		return true;

	//Check the length is less than the maximum
	if(nLength>nMaxLength)
		return false;

	//Cannot start with a number
	if(isdigit(*szText))
		return false;

	BOOL bHyphen=false;
	while(*szText)
	{
		if((*szText == '-') || (*szText == '_'))
		{
			if(bHyphen)		//Found two concurrent hyphens(error)
				return false;
			bHyphen = true;		//found hyphen so set hyphen flag
		}
		else
		if(iscsym(*szText))
			bHyphen = false;	//reset hyphen flag
		else
			return false;

		szText++;	//go to next char
	}
	return true;
}

/////////////////////////////////////////////////////////////////////////////
// CDataNameEdit

CDataNameEdit::CDataNameEdit(int nMaxLength/*=16*/):
	m_nMaxLength(nMaxLength)
{
}

CDataNameEdit::~CDataNameEdit()
{
}


BEGIN_MESSAGE_MAP(CDataNameEdit, CEdit)
	//{{AFX_MSG_MAP(CDataNameEdit)
	ON_CONTROL_REFLECT(EN_CHANGE, OnChange)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDataNameEdit message handlers

void CDataNameEdit::OnChange() 
{
	CString strText;
	GetWindowText(strText);

	if(CheckDataName(strText.GetBuffer(0),m_nMaxLength))
	{
		m_strPrevText = strText;	//Name is correct
		return;
	}

	int nStart,nEnd;
	GetSel(nStart,nEnd);
	nStart--;
	nEnd--;
	
	//Restore last name...
	SetWindowText(m_strPrevText);
	
	//and selection
	SetSel(nStart,nEnd);
}

void CDataNameEdit::PreSubclassWindow() 
{
	ModifyStyle(0,ES_UPPERCASE);
	
	GetWindowText(m_strPrevText);
		
	CEdit::PreSubclassWindow();
}

