/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestPlugin.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "TestPlugin.h"

#include "MainFrm.h"
#include "ChildFrm.h"
#include "IpFrame.h"
#include "TestPluginDoc.h"
#include "TestPluginView.h"
#include "CECGridCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BOOL bFirstLoad = FALSE;
/////////////////////////////////////////////////////////////////////////////
// CTestPluginApp

BEGIN_MESSAGE_MAP(CTestPluginApp, CWinApp)
	//{{AFX_MSG_MAP(CTestPluginApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestPluginApp construction

CTestPluginApp::CTestPluginApp()
{
	//Look for key created by commander that tells us that 
	//this is the first load
	HKEY hKey;
	if(RegOpenKeyEx(HKEY_CURRENT_USER,"SOFTWARE\\OpenSTA\\Loading",0,KEY_READ,&hKey)==ERROR_SUCCESS)
	{
		bFirstLoad = true;
		RegCloseKey(hKey);
		return;
	}

	m_pPluginManager = NULL;
	try
	{
		m_pInfo = new CCORBAInfo();
	}
	catch(...)
	{
		m_pInfo = NULL;
		MessageBox(NULL, "Unable to connect with the OpenSTA Name Server.\rYou will not be able to execute any tests until\rthe name server has been re-started.", "OpenSTA", MB_OK);
	}
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CTestPluginApp object

CTestPluginApp theApp;

// This identifier was generated to be statistically unique for your app.
// You may change it if you prefer to choose a specific identifier.

// {0298A255-D4CA-11D4-94C7-00C04F89E392}
static const CLSID clsid =
{ 0x298a255, 0xd4ca, 0x11d4, { 0x94, 0xc7, 0x0, 0xc0, 0x4f, 0x89, 0xe3, 0x92 } };

/////////////////////////////////////////////////////////////////////////////
// CTestPluginApp initialization

BOOL CTestPluginApp::InitInstance()
{
	// Initialize OLE libraries
	if (!AfxOleInit())
	{
		AfxMessageBox(IDP_OLE_INIT_FAILED);
		return FALSE;
	}

	CyrtoolsInitialize();
	
	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	SetRegistryKey(_T("OpenSTA"));

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CMultiDocTemplate* pDocTemplate;
	pDocTemplate = new CMultiDocTemplate(
		IDR_TESTPLTYPE,
		RUNTIME_CLASS(CTestPluginDoc),
		RUNTIME_CLASS(CChildFrame), // custom MDI child frame
		RUNTIME_CLASS(CTestPluginView));
	pDocTemplate->SetServerInfo(
		IDR_TESTPLTYPE_SRVR_EMB, IDR_TESTPLTYPE_SRVR_IP,
		RUNTIME_CLASS(CInPlaceFrame));
	AddDocTemplate(pDocTemplate);

	// Connect the COleTemplateServer to the document template.
	//  The COleTemplateServer creates new documents on behalf
	//  of requesting OLE containers by using information
	//  specified in the document template.
	m_server.ConnectTemplate(clsid, pDocTemplate, FALSE);

	// Register all OLE server factories as running.  This enables the
	//  OLE libraries to create objects from other applications.
	COleTemplateServer::RegisterAll();
		// Note: MDI applications register all server objects without regard
		//  to the /Embedding or /Automation on the command line.

	// When a server application is launched stand-alone, it is a good idea
	//  to update the system registry in case it has been damaged.
	m_server.UpdateRegistry(OAT_DOC_OBJECT_SERVER);
	COleObjectFactory::UpdateRegistryAll();

	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes(TRUE);

	//Check if /Register is on the command line 
	//if it is we can now shutdown as registration 
	//has now been completed
	if(IsRegisterMode())
		return false;

	// create main MDI Frame window
	CMainFrame* pMainFrame = new CMainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();
	
	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Check to see if launched as OLE server
	if (cmdInfo.m_bRunEmbedded || cmdInfo.m_bRunAutomated)
	{
		// Application was run with /Embedding or /Automation.  Don't show the
		//  main window in this case.
		return TRUE;
	}

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	InitPlugins();

	// The main window has been initialized, so show and update it.
	pMainFrame->ShowWindow(m_nCmdShow);
	pMainFrame->UpdateWindow();

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CTestPluginApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CTestPluginApp message handlers


int CTestPluginApp::ExitInstance() 
{
	if (m_pInfo)
		delete m_pInfo;

	if (m_pPluginManager)
		delete m_pPluginManager;

	CyrtoolsUninitialize();

	return CWinApp::ExitInstance();
}

CPluginManager* CTestPluginApp::GetPluginManager()
{
	return m_pPluginManager;
}

void CTestPluginApp::InitPlugins()
{
	if(bFirstLoad)
		return;

	CString strPlugDir = getenv("OPENSTA_HOME");
	strPlugDir += "Plugins\\";
	if(!m_pPluginManager)
		m_pPluginManager = new CPluginManager(strPlugDir);
}

//Check command line for /Register option
BOOL CTestPluginApp::IsRegisterMode()
{
	for(int nArg=1;nArg<__argc;nArg++)
	{
		if(stricmp("/Register",__argv[nArg])==0)
			return true;
	}
	return false;
}

CCORBAInfo* CTestPluginApp::GetInfo()
{
 	return m_pInfo;
}
