

#include "stdafx.h"
#include <afxdllx.h>
#include "PluginTypes.h"
#include "resource.h"       // main symbols
#include "TestFile.h"
#include "PropertiesBar.h"
#include "ConfigurationDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CConfigurationDlg *m_pConfigDlg=NULL;
CPropertiesBar *m_pPropertiesBar = NULL;

CCORBAInfo *m_pCorbaInfo = NULL;
CTestFile *m_pTestFile = NULL;
CDocument *m_pDocument = NULL;
CToolBar *m_pToolbar = NULL;

static AFX_EXTENSION_MODULE CConfigurationTabDll = { NULL, NULL };

HINSTANCE m_hInstance = NULL;
extern "C" int APIENTRY
    DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	m_hInstance = hInstance;
    // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        TRACE0("ConfigurationTabDll.DLL Initializing!\n");

        // Extension DLL one-time initialization
        if (!AfxInitExtensionModule(CConfigurationTabDll, hInstance))
            return 0;

        // Insert this DLL into the resource chain
        // NOTE: If this Extension DLL is being implicitly linked to by
        //  an MFC Regular DLL (such as an ActiveX Control)
        //  instead of an MFC application, then you will want to
        //  remove this line from DllMain and put it in a separate
        //  function exported from this Extension DLL.  The Regular DLL
        //  that uses this Extension DLL should then explicitly call that
        //  function to initialize this Extension DLL.  Otherwise,
        //  the CDynLinkLibrary object will not be attached to the
        //  Regular DLL's resource chain, and serious problems will
        //  result.
		CyrtoolsInitialize();

		new CDynLinkLibrary(CConfigurationTabDll);
    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        TRACE0("CConfigurationTabDll.DLL Terminating!\n");

        // Terminate the library before destructors are called
        AfxTermExtensionModule(CConfigurationTabDll);
		
		delete m_pPropertiesBar;
		m_pPropertiesBar = NULL;
		delete m_pConfigDlg;
		m_pConfigDlg = NULL;
		delete m_pToolbar;
		m_pToolbar = NULL;

		CyrtoolsUninitialize();
    }

    return 1;   // ok
}

BOOL IsInitialised()
{
	if(m_pConfigDlg && m_pPropertiesBar &&
		m_pTestFile && m_pDocument)
		return true;

	return false;
}

extern "C" AFX_EXT_API
BOOL OpenSTAVerify()
{
	return true;
}

extern "C" AFX_EXT_API
void GetTitle(CString& strTitle)
{
	strTitle = "Test Configuration Tab";
}

extern "C" AFX_EXT_API
int GetType()
{
	return TEST_PLUGIN_TAB;
}

extern "C" AFX_EXT_API
void GetPackage(CString& strPackage)
{
	strPackage = "TEST CONFIGURATION TAB";
}

extern "C" AFX_EXT_API
void SetTestFile(CTestFile *pTestFile)
{
	m_pTestFile = pTestFile;
	
	if(IsInitialised())
		m_pConfigDlg->Activate();
}

extern "C" AFX_EXT_API
void SetDocument(CDocument *pDoc)
{
	m_pDocument = pDoc;
	
	if(IsInitialised())
		m_pConfigDlg->Activate();
}

extern "C" AFX_EXT_API
int GetNumTabs()
{
	return 1;
}

extern "C" AFX_EXT_API
CDialog* CreateTab(int nTab,CWnd *pParent,CString& strTabName)
{
	UINT uiIDD = 0;
	CDialog *pDialog = NULL;
	switch(nTab)
	{
	case 0:
		if(!m_pConfigDlg)
			m_pConfigDlg = new CConfigurationDlg;
		pDialog = m_pConfigDlg;
		uiIDD = CConfigurationDlg::IDD;
		strTabName = "Configuration";
	break;
	default:
		ASSERT(FALSE);
	break;
	}

	AfxSetResourceHandle(m_hInstance);
	if(!pDialog || !pDialog->Create(uiIDD,pParent))
	{
		AfxSetResourceHandle(AfxGetInstanceHandle());
		return NULL;
	}
	AfxSetResourceHandle(AfxGetInstanceHandle());

	if(IsInitialised())
		m_pConfigDlg->Activate();

	return pDialog;
}

extern "C" AFX_EXT_API
void GetTabImages(CBitmap& bmpTabs)
{
	AfxSetResourceHandle(m_hInstance);
	bmpTabs.LoadBitmap(IDB_TAB_IMAGES);
	AfxSetResourceHandle(AfxGetInstanceHandle());
}

extern "C" AFX_EXT_API
void SetCorbaInfo(CCORBAInfo *pCorbaInfo)
{
	m_pCorbaInfo = pCorbaInfo;

	if(IsInitialised())
		m_pConfigDlg->Activate();
}

extern "C" AFX_EXT_API
BOOL LoadToolbars(CFrameWnd* pWndFrame,CFrameWnd *pOwner,HMENU hMainMenu)
{
	if(!m_pPropertiesBar)
		m_pPropertiesBar = new CPropertiesBar;
	else
		m_pPropertiesBar->DestroyWindow();

	m_pPropertiesBar->SetOwner(pOwner);
	AfxSetResourceHandle(m_hInstance);

	if (!m_pPropertiesBar->Create(_T("Properties"), pWndFrame, 131))
	{
	    TRACE0("Failed to create Properties bar\n");
	    return false;
	    // fail to create
	}

	AfxSetResourceHandle(AfxGetInstanceHandle());
	m_pPropertiesBar->SetBarStyle(m_pPropertiesBar->GetBarStyle() |
	    CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	m_pPropertiesBar->EnableDocking(CBRS_ALIGN_ANY);

	pWndFrame->DockControlBar(m_pPropertiesBar,AFX_IDW_DOCKBAR_BOTTOM);
	pWndFrame->ShowControlBar(m_pPropertiesBar,true,false);

	m_pToolbar = new CToolBar;
	m_pToolbar->SetOwner(pOwner);

	AfxSetResourceHandle(m_hInstance);
	
	// Create toolbar on client's frame window
	if (!m_pToolbar->CreateEx(pWndFrame, TBSTYLE_FLAT,WS_CHILD| CBRS_TOP|WS_VISIBLE
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_pToolbar->LoadToolBar(IDR_CONFIG_TOOLBAR))
	{
		TRACE0("Failed to create toolbar\n");
		return FALSE;
	}

	AfxSetResourceHandle(AfxGetInstanceHandle());

	m_pToolbar->EnableDocking(CBRS_ALIGN_ANY);
	pWndFrame->DockControlBar(m_pToolbar,AFX_IDW_DOCKBAR_TOP);
	
	//This code will dock the new tool bar to the right of the plugins ones
	CDockBar *pDockBar = (CDockBar*)pWndFrame->GetControlBar(AFX_IDW_DOCKBAR_TOP);
	if(pDockBar)
	{
		for(int nBar = 0;nBar<pDockBar->m_arrBars.GetSize();nBar++)
		{
			CToolBar *pToolBar = (CToolBar*)pDockBar->m_arrBars[nBar];
			
			if(pToolBar == m_pToolbar)
				pDockBar->m_arrBars.RemoveAt(nBar--);
			else
			if(pToolBar)
			{
				while(pDockBar->m_arrBars[++nBar]);
				pDockBar->m_arrBars.InsertAt(nBar,m_pToolbar);
			}
		}
	}

	pWndFrame->ShowControlBar(m_pToolbar,true,false);

	if(IsInitialised())
		m_pConfigDlg->Activate();

	return true;
}	

/*
extern "C" AFX_EXT_API
void SetPluginManager(CPluginManager* pPluginManager)
{

}*/

extern "C" AFX_EXT_API
void SetTestActive(BOOL bActive,const CString& strReg)
{
	if(m_pConfigDlg)
	{
		if(bActive)
			m_pConfigDlg->SetTestActive(strReg);
		else
			m_pConfigDlg->StopActiveTest();
	}
}

extern "C" AFX_EXT_API
void LoadMenus(HMENU hMenu)
{
	AfxSetResourceHandle(m_hInstance);

	CMenu *pMenu = CMenu::FromHandle(hMenu);
	
	UINT uiTestPos = -1;
	UINT uiCount = pMenu->GetMenuItemCount();

	CString strItem;
	//Look for test menu
	for(UINT uiItem=0;uiItem<uiCount;uiItem++)
	{
		pMenu->GetMenuString(uiItem,strItem,MF_BYPOSITION);
		if(strItem == "&Test")
		{
			uiTestPos = uiItem;
			break;
		}
	}
	
	if(uiTestPos == -1)
	{
		uiTestPos = uiCount-2;
		pMenu->InsertMenu(uiTestPos,MF_BYPOSITION|MF_STRING,0,"&Test");
	}
	
	CMenu *pTestMenu = pMenu->GetSubMenu(uiTestPos);
	pTestMenu->AppendMenu(MF_STRING,ID_DELETESELECTION,"&Delete Selection\tDel");
	
	pTestMenu->Detach();
	pMenu->Detach();
		
	AfxSetResourceHandle(AfxGetInstanceHandle());
}

//void MergeMenu(