/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// MonitoringErrorLog.cpp : implementation file
//

#include "stdafx.h"
#include "MonitoringErrorLog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDT_POLL_DATA 1
static int ID_REFRESH_RATE = 5000;

/////////////////////////////////////////////////////////////////////////////
// CMonitoringErrorLog dialog


CMonitoringErrorLog::CMonitoringErrorLog(CWnd* pParent /*=NULL*/)
	: CDialog(CMonitoringErrorLog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMonitoringErrorLog)
	//}}AFX_DATA_INIT

	m_bDataFileOpen = false;
	m_bHeadersLoaded = false;
	m_nFilePos=0;
	m_strTestRunPath="";
}


void CMonitoringErrorLog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMonitoringErrorLog)
	DDX_Control(pDX, IDC_ERROR_LIST, m_ctlErrorLogList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMonitoringErrorLog, CDialog)
	//{{AFX_MSG_MAP(CMonitoringErrorLog)
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMonitoringErrorLog message handlers

void CMonitoringErrorLog::OnDestroy() 
{
	CDialog::OnDestroy();
	UpdateErrorLogWindow();
	KillTimer(IDT_POLL_DATA);
	//set pointer to be at start of file NULL
}

void CMonitoringErrorLog::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);	

	if(m_ctlErrorLogList.GetSafeHwnd())
	{
		m_ctlErrorLogList.MoveWindow(0,0,cx,cy);
	}
	
}


BOOL CMonitoringErrorLog::OnInitDialog() 
{

	CDialog::OnInitDialog();
	CRect r;
	GetClientRect(r);
	m_ctlErrorLogList.InsertColumn(0,"Time",LVCFMT_LEFT,r.Width()/4);
	m_ctlErrorLogList.InsertColumn(1,"Test Name",LVCFMT_LEFT,r.Width()/4);
	m_ctlErrorLogList.InsertColumn(2,"Location",LVCFMT_LEFT,r.Width()/4);
	m_ctlErrorLogList.InsertColumn(3,"Message",LVCFMT_LEFT,r.Width());
	//this makes the message window be too wide to be all seen without resizing the window

	SetTimer(IDT_POLL_DATA, ID_REFRESH_RATE, NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CMonitoringErrorLog::OnTimer(UINT nIDEvent) 
{
	UpdateErrorLogWindow();
	CDialog::OnTimer(nIDEvent);
}

void CMonitoringErrorLog::UpdateErrorLogWindow()
{
	try
    {
		if (!m_bDataFileOpen)
		{
			//open the file
			CFileStatus fs;
			CString Path = 	GetPath();
			if (!CFile::GetStatus(Path, fs))
			{
				return;
			}
			m_bDataFileOpen = m_dataFile.Open(Path, CFile::modeRead | CFile::typeText | CFile::shareDenyNone );
			if (!m_bDataFileOpen)
			{
				return;
			}
		}
		m_dataFile.Seek(m_nFilePos, CFile::begin);

        CString strData;

		while(m_dataFile.ReadString(strData))
		{
			CString time, name, location, message;
			GetNextField(',', strData, time);
			GetNextField(',', strData, name);
			GetNextField(',', strData, location);
			GetNextField(',', strData, message);

			m_ctlErrorLogList.InsertItem(0,time);
			m_ctlErrorLogList.SetItemText(0,1,name);
			m_ctlErrorLogList.SetItemText(0,2,location);
			m_ctlErrorLogList.SetItemText(0,3,message);
		}
		m_nFilePos = m_dataFile.GetPosition();
    }
    catch(...)
    {
    }	
	return;
}

int CMonitoringErrorLog::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr == "")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep, 0);

    if (nPos == -1)
    {
        field = inStr;
        inStr = "";
        return 0;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr = inStr.Right(len - nPos - 1);
        return 0;
    }
}

CString CMonitoringErrorLog::GetPath()
{
	CString answer=m_strTestRunPath+"\\ErrLog.txt";
	return answer;
}

void CMonitoringErrorLog::Init(CString path)
{
	m_strTestRunPath=path;
	return;
}
