/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ResultsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ResultsDlg.h"
#include "TestFile.h"
#include "PluginTypes.h"
#include "TestResultsPlugin.h"
#include "XAuditViewer.h"
#include "DeleteRunDlg.h"
#include "OstaDefs.h"
#include "TestSummaryWnd.h"
#include "TestRunBar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg dialog
extern HINSTANCE m_hInstance;
extern CToolBar *m_pToolbar;
extern CPluginManager* m_pPluginManager;
extern CTestFile* m_pTestFile;
extern CTestRunBar *m_pTestRunBar;

CResultsDlg::CResultsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CResultsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CResultsDlg)
	//}}AFX_DATA_INIT
	m_pFrame = NULL;
	m_nLastGraphAdded=0;
	m_pCurrentGraph =NULL;
	
	m_bRunsVisible = true;
}

CResultsDlg::~CResultsDlg()
{
}


void CResultsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CResultsDlg)
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CResultsDlg, CDialog)
	//{{AFX_MSG_MAP(CResultsDlg)
	ON_WM_SIZE()
	ON_COMMAND(ID_DEF_OPERATION, OnDefaultOperation)
	ON_WM_SHOWWINDOW()
	ON_COMMAND(ID_DELETE_RUNS, OnDeleteRuns)
	ON_COMMAND(ID_CASCADE, OnCascade)
	ON_COMMAND(ID_TILEHORIZONTALLY, OnTilehorizontally)
	ON_COMMAND(ID_TILEVERTICALLY, OnTilevertically)
	ON_COMMAND(ID_TOGGLE_SHOWRUNS, OnToggleShowruns)
	ON_UPDATE_COMMAND_UI(ID_TOGGLE_SHOWRUNS, OnUpdateToggleShowruns)
	ON_COMMAND(ID_CLOSEALL, OnCloseall)
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI(ID_DEF_OPERATION,OnUpdateDefOperation)
	ON_WM_ERASEBKGND()
	ON_MESSAGE(WMU_MDISTATECHANGED,OnMDIStateChanged)
	ON_MESSAGE(WMU_GRAPH_SELCHANGED,OnGraphSelChanged)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg message handlers

BOOL CResultsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	AfxSetResourceHandle(m_hInstance);

	m_pFrame = new CMDIFrameWnd;
	m_pFrame->LoadFrame(IDR_MAINFRAME,FWS_ADDTOTITLE|WS_CHILD|WS_VISIBLE,this);
	m_pFrame->UpdateWindow();

	AfxSetResourceHandle(AfxGetInstanceHandle());
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CResultsDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	if(m_pFrame)
		m_pFrame->SetWindowPos(NULL,0,0,cx,cy,SWP_NOZORDER);

}

void CResultsDlg::OnDefaultOperation() 
{
	OpenGraph OG;
	if(GetCurrentGraph(OG))
	{
		CTestResultsPlugin plug(OG.strPlugin, m_pPluginManager);
		plug.LoadRun(OG.strRun);
		plug.StartDefaultOperation(OG.pGraphWnd->GetGraph());
	}
}

void CResultsDlg::Activate()
{
	CString strTestFile(m_pTestFile->GetTestFilePath());
	int nPos = strTestFile.ReverseFind('\\');
	
	if(nPos!=-1)
	{
		m_strTestFolder = strTestFile.Left(nPos+1);
		if(m_pTestRunBar)
		{
			m_pTestRunBar->SetTestFolder(m_strTestFolder);
			m_pTestRunBar->SetNotifyWindow(m_hWnd);
		}
	}
}

BOOL CResultsDlg::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)
	{
	case WM_KEYDOWN:
		switch(pMsg->wParam)
		{
		case VK_F5:
			if(m_pTestRunBar)
				m_pTestRunBar->Refresh();
		break;
		}
	break;
	}
	
	return CDialog::PreTranslateMessage(pMsg);
}

void CResultsDlg::OnUpdateDefOperation(CCmdUI *pCmdUI)
{
	OpenGraph OG;
	if(GetCurrentGraph(OG))
		pCmdUI->Enable(OG.bHasDefOperation);
	else
		pCmdUI->Enable(FALSE);
}

void CResultsDlg::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);

	if(!m_pTestRunBar)
		return;

	if(bShow)
	{
		if(m_bRunsVisible)
		{
			m_pTestRunBar->Refresh();
			m_pTestRunBar->m_pDockSite->ShowControlBar(m_pTestRunBar,TRUE,TRUE);
		}
		if(m_pToolbar)
			m_pToolbar->m_pDockSite->ShowControlBar(m_pToolbar,TRUE,TRUE);
	}
	else
	{
		m_pTestRunBar->m_pDockSite->ShowControlBar(m_pTestRunBar,FALSE,TRUE);
		if(m_pToolbar)
			m_pToolbar->m_pDockSite->ShowControlBar(m_pToolbar,FALSE,TRUE);
	}
	((CFrameWnd*)AfxGetMainWnd())->RecalcLayout(FALSE);
}

void CResultsDlg::OnDeleteRuns()
{
	AfxSetResourceHandle(m_hInstance);
	CDeleteRunDlg dlg(m_strTestFolder);
	if(dlg.DoModal()==IDOK && m_pTestRunBar)
		m_pTestRunBar->Refresh();

	AfxSetResourceHandle(AfxGetInstanceHandle());
}

BOOL CResultsDlg::OnEraseBkgnd(CDC *pDC)
{
	return true;
//	return CDialog::OnEraseBkgnd(pDC);
}

BOOL CResultsDlg::GetCurrentGraph(OpenGraph &OG)
{
	for(int nNdx=0;nNdx<m_arOpenGraphs.GetSize();nNdx++)
	{
		if(m_arOpenGraphs[nNdx].pGraphWnd == m_pCurrentGraph)
		{
			OG = m_arOpenGraphs[nNdx];
			return TRUE;
		}
	}
	return false;
}
	
void CResultsDlg::OnOK() 
{
}

void CResultsDlg::OnCancel() 
{
}

CGraphWnd *CResultsDlg::AddNewChild(const CString& strTitle)
{
	HINSTANCE hInst = AfxGetResourceHandle();
	HMENU hMDIMenu = ::LoadMenu(hInst, MAKEINTRESOURCE(IDR_MDITESTYPE));
	HACCEL hMDIAccel = ::LoadAccelerators(hInst, MAKEINTRESOURCE(IDR_MDITESTYPE));

	CRuntimeClass *pClass = RUNTIME_CLASS(CGraphWnd);
	ASSERT(pClass != NULL);
	CGraphWnd *pFrame = (CGraphWnd*) pClass->CreateObject();
	ASSERT_KINDOF(CGraphWnd, pFrame);

	// load the frame
	CCreateContext context;
	context.m_pCurrentFrame = m_pFrame;

	if(!pFrame->Create(NULL,strTitle,WS_CHILD|WS_VISIBLE|WS_OVERLAPPEDWINDOW,CFrameWnd::rectDefault,m_pFrame,&context))
	{
		TRACE0("Couldn't load frame window.\n");
		delete (LPVOID)pFrame;
		return NULL;
	}
	pFrame->SetHandles(hMDIMenu, hMDIAccel);
	pFrame->SetNotifyWindow(m_hWnd);

	return pFrame;
}

LRESULT CResultsDlg::OnMDIStateChanged(WPARAM wParam,LPARAM lParam)
{
	HWND hWnd = (HWND)wParam;
	
	for(int nNdx=0;nNdx<m_arOpenGraphs.GetSize();nNdx++)
	{
		if(m_arOpenGraphs[nNdx].pGraphWnd->m_hWnd == hWnd)
		{
			switch(lParam)
			{
			case CGraphWnd::enActive:
				m_pCurrentGraph = m_arOpenGraphs[nNdx].pGraphWnd;
	
				UpdateDialogControls(this,TRUE);
			return 0;
			case CGraphWnd::enInactive:
			return 0;
			case CGraphWnd::enDestroyed:
				//Remove check from closed graph
				if(m_pTestRunBar)
					m_pTestRunBar->SetCheck(m_arOpenGraphs[nNdx].strRun,m_arOpenGraphs[nNdx].strName,false);
				
				CWnd *pWnd = m_arOpenGraphs[nNdx].pGraphWnd;
				m_arOpenGraphs.RemoveAt(nNdx);
				if(m_pCurrentGraph == pWnd)
					m_pCurrentGraph = NULL;

					
				UpdateDialogControls(this,TRUE);
			return 0;
			}
		}
	}
	
	return 0;
}

void CResultsDlg::OnCascade() 
{
	m_pFrame->MDICascade();
}

void CResultsDlg::OnTilehorizontally() 
{
	m_pFrame->MDITile(MDITILE_HORIZONTAL);
}

void CResultsDlg::OnTilevertically() 
{
	m_pFrame->MDITile(MDITILE_VERTICAL);
}

void CResultsDlg::OnToggleShowruns() 
{
	if(!m_pTestRunBar)
		return;

	if(m_pTestRunBar->IsVisible())
	{
		m_bRunsVisible = false;
		m_pTestRunBar->m_pDockSite->ShowControlBar(m_pTestRunBar,FALSE,TRUE);
	}
	else
	{
		m_bRunsVisible = true;
		m_pTestRunBar->m_pDockSite->ShowControlBar(m_pTestRunBar,TRUE,TRUE);
	}
	
	((CFrameWnd*)AfxGetMainWnd())->RecalcLayout();
}

void CResultsDlg::OnUpdateToggleShowruns(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable( IsWindowVisible() );
	if(m_pTestRunBar)
		pCmdUI->SetCheck( m_pTestRunBar->IsWindowVisible() );
}

LRESULT CResultsDlg::OnGraphSelChanged(WPARAM wParam,LPARAM lParam)
{
	CTestRunBar::NM_TESTRUN *pTestRun = (CTestRunBar::NM_TESTRUN*)wParam;
	BOOL bSelected = (BOOL) lParam;

	const CString &strRunName = pTestRun->strTestRun;
	const CString &strGraphTitle = pTestRun->strGraph;
	const CString &strPlugin = pTestRun->strPlugin;

	if(bSelected)
	{
		CRect rcChild;
		OpenGraph OG;
		OG.strRun = strRunName; 
		OG.strName = strGraphTitle;
		OG.strPlugin = strPlugin;
		OG.bHasDefOperation = false;
		
		CString strTitle(strRunName);
		strTitle.TrimRight("\\");
		int nSlashPos = strTitle.ReverseFind('\\');
		if(nSlashPos != -1)
			strTitle = strGraphTitle + ":- " + strTitle.Mid(nSlashPos+1);
		else
			strTitle = strGraphTitle + ":- " + strTitle;

		OG.pGraphWnd = AddNewChild(strTitle);
		OG.pGraphWnd->GetClientRect(rcChild);

		if(strPlugin.IsEmpty())
		{
			if(strGraphTitle == CTestRunBar::m_strAuditLogName ||
			   strGraphTitle == CTestRunBar::m_strHistoryLogName||
			   strGraphTitle == CTestRunBar::m_strReportLogName)
			{
				CXAuditViewer *pAudit = new CXAuditViewer;
				pAudit->Create(NULL,WS_CHILD|WS_VISIBLE,rcChild,OG.pGraphWnd,0);
				pAudit->SetRunFolder(OG.strRun);
				
				int nType = 0;
				if(strGraphTitle == CTestRunBar::m_strHistoryLogName)
					nType = 1;
				else
				if(strGraphTitle == CTestRunBar::m_strReportLogName)
					nType = 2;
				
				pAudit->SetLogType(nType);
				OG.pGraphWnd->SetGraph(pAudit);
			}
			else
			if(strGraphTitle == CTestRunBar::m_strSummaryName)
			{
				CTestSummaryWnd *pWnd = new CTestSummaryWnd;
				pWnd->Create(rcChild,OG.pGraphWnd);
				pWnd->SetFile(OG.strRun + strSummaryFile);
				pWnd->SetFont(GetFont());
				OG.pGraphWnd->SetGraph(pWnd);
			}
		}
		else
		{
			CTestResultsPlugin plug(strPlugin, m_pPluginManager);
			plug.LoadRun(strRunName);
			CWnd *pGraph = plug.DisplayGraph(strGraphTitle,OG.pGraphWnd,rcChild);
			if(!pGraph)
			{
				OG.pGraphWnd->MDIDestroy();
				return 0;
			}

			OG.pGraphWnd->SetGraph(pGraph);
			OG.bHasDefOperation = plug.GetDefaultOperationName(pGraph,OG.strDefOpName);
		}

		m_arOpenGraphs.Add(OG);
		OnMDIStateChanged((WPARAM)OG.pGraphWnd->GetSafeHwnd(),CGraphWnd::enActive);
		OG.pGraphWnd->InitialUpdateFrame(NULL,TRUE);
		OG.pGraphWnd->RedrawWindow(NULL,NULL,RDW_FRAME|RDW_INVALIDATE);
	}
	else
	{
		for(int nGraph=0;nGraph<m_arOpenGraphs.GetSize();nGraph++)
		{
			OpenGraph &OG = m_arOpenGraphs[nGraph];
			if((OG.strRun == strRunName) && 
				(OG.strName == strGraphTitle) &&
				(OG.strPlugin == strPlugin))
			{
				OG.pGraphWnd->MDIDestroy();
			}
		}
	}
	
	UpdateDialogControls(this,TRUE);
	
	return 0;
}

void CResultsDlg::OnCloseall() 
{
	//calling MDIDestroy truncates the array! so beware
	//each message to OnMDIStateChanged() truncates the array
	int numberofwindows=m_arOpenGraphs.GetSize();

	for(int nGraph=0;nGraph<numberofwindows;nGraph++)
	{
		OpenGraph &OG = m_arOpenGraphs[0];
		OG.pGraphWnd->MDIDestroy();
	}
}


