/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4710)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifdef BUILDDLLGWCONSCMD
#  define DLLSPEC_GWCONSCMD __EXPORT__
#else
#  define DLLSPEC_GWCONSCMD __IMPORT__
#endif  // BUILDDLLGWCONSCMD

#ifndef __GATEWAY_CMD__
#define DLLSPEC_GWCMD   DLLSPEC_GWCONSCMD
#  include "gwcmd.hpp"
#endif  // __GATEWAY_CMD__

#ifndef __GATEWAY_CONSOLE_COMMAND__
#  include <gwconscmd.hpp>
#endif  // __GATEWAY_CONSOLE_COMMAND__


//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////// Class CGCmdConsole //
///////////////////////////////////////////////////////////////////////////////
class CGCmdConsole : public CGCmdUser
{
    //-----------------------------------------------------------------------------
public:                         // Types.
    enum enumCmdConsole
    {
        CMD_GET_STATUS
    } ;

public:                         // Constructor(s) and destructor.
    CGCmdConsole ()
: CGCmdUser ()
    { }

    CGCmdConsole (enumCmdConsole eCommandConsole)
: CGCmdUser ( (unsigned long) eCommandConsole)
    { }

    CGCmdConsole (const CGCmdConsole& consoleCommand)
: CGCmdUser (consoleCommand)
    { }

    ~CGCmdConsole ()
    { }


public:                         // Operators.
    CGCmdConsole&               operator = (const CGCmdConsole&)
    {
        return *this ;
    }

public:                         // Get methodes.
    enumCmdConsole&             getConsoleCommand (void)
    {
        return (enumCmdConsole&) getUserCommand () ;
    }

    //-----------------------------------------------------------------------------
private:                        // Members.

    //-----------------------------------------------------------------------------
}

; // CGCmdConsole

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////// End of Class CGCmdConsole //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CGConsole //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWCMD CGConsole : public CGWParseCommand
{
    //-----------------------------------------------------------------------------
public:                         // Virtual methodes.
    void                        parseCommand (const string& command, CGCmd*& pCmd) ;
    void                        commandResult (CGCmd*& cmd) ;

    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGConsole (const void* pConnectionId, const string& taskName, bool& bTraceOn, CTOLogFileList& logFileList, CTSocket* pClient) ;
    ~CGConsole ()
    { }
    ;

public:                         // Operators.
    CGConsole&                  operator = (const CGConsole&) ;

    //-----------------------------------------------------------------------------
protected:                      // Control Methodes.
    void                        getWhoList (long lParentId, CGCmdWho* pCmdWho, CGWhoList& whoList) ;

protected:                      // Get Methodes.

    //-----------------------------------------------------------------------------
private:                        // Members.
    int                         iConnectionId_ ;
    static int                  iNbConnections_ ;

    //-----------------------------------------------------------------------------
}

; // CGConsole

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CGConsole //
///////////////////////////////////////////////////////////////////////////////

//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CGConsole //
///////////////////////////////////////////////////////////////////////////////

int CGConsole::iNbConnections_ = 0 ;

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CGConsole::CGConsole
//
// Constructor.
//
// Arguments : pConnectionId : connection identifier.
//             taskName      : name of the task.
//             bTraceOn      : reference on the trace mode.
//             logFileList   : reference on the log file list.
//             pClient       : pointer to client socket.
// Return    : none.
//-----------------------------------------------------------------------------
CGConsole::CGConsole (const void* pConnectionId, const string& taskName, bool& bTraceOn, CTOLogFileList& logFileList, CTSocket* pClient)
        : CGWParseCommand (pConnectionId, taskName, bTraceOn, logFileList, pClient)
{
    try
    {
        iConnectionId_ = ++ iNbConnections_ ;
        traceLine (taskName_ + ": new command object.") ;
    }
    catch (CTException& e)
    {
        throw CGCmdException ( (CGCmdException&) e) ;
    }
} // CGConsole::CGConsole

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CGConsole::parseCommand
//
// Parse the command.
//
// Arguments : command : command string to parse.
//             pCmd    : reference to pointer to command to execute at return.
// Return    : none.
//-----------------------------------------------------------------------------
void CGConsole::parseCommand (const string& command, CGCmd*& pCmd)
{
    try
    {
        traceFlush (taskName_ + ": " + command) ;
        if (command == "CONSOLE:WHO")
        {   // Who is connected..
            pCmd = new CGCmdWho () ;
            if (pCmd == NULL)
            {   // Error from new.
                throw CGCmdException ("parseCommand: new CGCmdWho failed.", __FILE__, __LINE__) ;
            }
        }
        else if (command == "CONSOLE:LISTEN")
        {   // Create a new gateway.
            // Receive configuration informations.
            CGListen    listenInfo ;
            client_ >> listenInfo ;

            // Create a gateway.
            port_t  listenPort ;
            sscanf (listenInfo.getPort ().data (), "%hu", &listenPort) ;
            pCmd = new CGCmdNewListenTask (listenInfo.getIsAdministrationListen (), listenInfo.getTargetAddress (), listenPort, listenInfo.getLibrary (), listenInfo.getTargetAddressAutoDetectionMode () ) ;
            if (pCmd == NULL)
            {   // Error from new.
                throw CGCmdException ("parseCommand: new CGCmdNewListenTask failed.", __FILE__, __LINE__) ;
            }
            traceFlush (string (":\tlisten for <") + listenInfo.getTargetAddress () + "> at <" + listenInfo.getPort () + "> using '" + listenInfo.getLibrary () + "'") ;
        }
        else if (command == "CONSOLE:KILL")
        {   // Kill a task.
            // Receive kill informations.
            CGKill  killInfo ;
            client_ >> killInfo ;

            pCmd = new CGCmdKillTaskId (killInfo.getTaskId (), killInfo.getChildrenAlso () ) ;
            if (pCmd == NULL)
            {   // Error from new.
                throw CGCmdException ("parseCommand: new CGCmdKillTaskId failed.", __FILE__, __LINE__) ;
            }
            char zBuf[16];
            sprintf(zBuf, "%d", killInfo.getTaskId () );
            traceFlush (string (":\tkill ") + string((char*) (killInfo.getChildrenAlso () ? "-r " : "")) + string(zBuf)) ;
        }
        else if (command == "CONSOLE:SHUTDOWN")
        {   // Shutdown the gateway.
            // Receive shutdown informations.
            CGShutdown  shutdownInfo ;
            client_ >> shutdownInfo ;

            pCmd = new CGCmdShutDown (shutdownInfo.getWithNoWait () ) ;
            if (pCmd == NULL)
            {   // Error from new.
                throw CGCmdException ("parseCommand: new CGCmdShutDown failed.", __FILE__, __LINE__) ;
            }
            if (shutdownInfo.getWithNoWait () )
            {
                traceFlush (string (":\tshutdown with no wait") ) ;
            }
        }
        else if (command == "CONSOLE:TRACE")
        {   // Set the trace on.
            // Receive trace informations.
            CGTrace traceInfo ;
            client_ >> traceInfo ;

            pCmd = new CGCmdTrace (traceInfo.getOn () ) ;
            if (pCmd == NULL)
            {   // Error from new.
                throw CGCmdException ("parseCommand: new CGCmdTrace failed.", __FILE__, __LINE__) ;
            }
            traceFlush (string (":\ttrace ") + (char*) (traceInfo.getOn () ? "on" : "off") ) ;
        }
        else if (command == "CONSOLE:GOODBYE")
        {   // Close connection.
            pCmd = new CGCmdQuit () ;
            if (pCmd == NULL)
            {   // Error from new.
                throw CGCmdException ("parseCommand: new CGCmdQuit failed.", __FILE__, __LINE__) ;
            }
        }
        else
        {   // Error from command.
            throw CGCmdException (string ("parseCommand: command ") + command + " is unknow.", __FILE__, __LINE__) ;
        }
        traceLine (string (".") ) ;
    }
    catch (CTException& e)
    {
        throw CGCmdException ( (CGCmdException&) e) ;
    }
} // CGConsole::parseCommand

//-----------------------------------------------------------------------------
// CGConsole::commandResult
//
// Send the result of the command.
//
// Arguments : pCmd    : reference to pointer to command to execute at return.
// Return    : none.
//-----------------------------------------------------------------------------
void CGConsole::commandResult (CGCmd*& pCmd)
{
    try
    {
        string  command ;
        if (pCmd == NULL)
        {   // Error from pCmd.
            throw CGCmdException ("commandResult: pCmd is null.", __FILE__, __LINE__) ;
        }
        if (pCmd->getCommand () == CGCmd::CMD_WHO)
        {   // Send the client list.
            if (pCmd->getResult () )
            {
                CGWhoList   whoList ;
                getWhoList (0L, (CGCmdWho*) pCmd, whoList) ;
                client_ << whoList ;
            }

            // Send command execution status.
            command = (char*) (pCmd->getResult () ? "OK" : "FAIL") ;
            client_ << command ;
        }
        else if (pCmd->getCommand () == CGCmd::CMD_NEW_LISTEN_TASK)
        {   // Create a new gateway.
            // Send command execution status.
            command = (char*) (pCmd->getResult () ? "OK" : "FAIL") ;
            client_ << command ;
        }
        else if ( (pCmd->getCommand () == CGCmd::CMD_KILL_TASK_ID) || (pCmd->getCommand () == CGCmd::CMD_KILL_TASK_TYPE) )
        {   // kill a task.
            // Send command execution status.
            command = (char*) (pCmd->getResult () ? "OK" : "FAIL") ;
            client_ << command ;
        }
        else if (pCmd->getCommand () == CGCmd::CMD_SHUTDOWN)
        {   // Shutdown.
            // Send command execution status.
            command = (char*) (pCmd->getResult () ? "OK" : "FAIL") ;
            client_ << command ;
        }
        else if (pCmd->getCommand () == CGCmd::CMD_TRACE)
        {   // Set the trace.
            // Send command execution status.
            command = (char*) (pCmd->getResult () ? "OK" : "FAIL") ;
            client_ << command ;
        }
        else if (pCmd->getCommand () == CGCmd::CMD_QUIT)
        {   // Close connection.
        }

        else if (pCmd->getCommand () == CGCmd::CMD_USER)
        {
            // Error from command.
            // Send command execution status.
            command = "FAIL" ;
            client_ << command ;
        }
        else
        {   // Error from command.
            // Send command execution status.
            command = "FAIL" ;
            client_ << command ;
        }
        delete pCmd ;
        pCmd = NULL ;
    }
    catch (CTException& e)
    {
        throw CGCmdException ( (CGCmdException&) e) ;
    }
} // CGConsole::commandResult

void CGConsole::getWhoList (long lParentId, CGCmdWho* pCmdWho, CGWhoList& whoList)
{
    if (pCmdWho)
    {
        whoList.push_back (CGWho (pCmdWho->getId (), lParentId, (long) pCmdWho->getType (), pCmdWho->getClientAddress (), pCmdWho->getGatewayAddress (), pCmdWho->getTargetAddress (), pCmdWho->getLibrary () ) ) ;

        for (int i=0; i<(int)pCmdWho->getChildrenList ().size(); i++)
        {
            getWhoList (pCmdWho->getId (), pCmdWho->getChildrenList ()[i], whoList) ;
        }
    }
} // CGConsole::getWhoList

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CGConsole //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Globals Functions //
///////////////////////////////////////////////////////////////////////////////
extern "C"
{
    //=====================================================================< Data >

    //================================================================< Functions >
    void DLLSPEC_GWCONSCMD LGW_NewGatewayParseCommand (const void* pConnectionId, const string& taskName, bool& bTraceOn, CTOLogFileList& logFileList, CTSocket* pClient, CGWParseCommand*& pGWParseCommand)
    {
        try
        {
            pGWParseCommand = (CGWParseCommand*) new CGConsole (pConnectionId, taskName, bTraceOn, logFileList, pClient) ;
            if (pGWParseCommand == NULL)
            {   // Error from new.
                throw CGCmdException ("LGW_NewGatewayParseCommand: new failed.", __FILE__, __LINE__) ;
            }
        }
        catch (CTException& e)
        {
            pGWParseCommand = NULL ;
            throw CGCmdException ( (CGCmdException&) e) ;
        }
    } // LGW_NewGatewayParseCommand

} // extern "C"

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Globals Functions //
///////////////////////////////////////////////////////////////////////////////
