#ifndef __VARSTRUCTURES_H__
#define __VARSTRUCTURES_H__
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include <AfxDB.h>
#include "ConcertLib.h"

//---------------------------------------------------------------------------
// Structures utilises dans le wizard de cration de variable
//---------------------------------------------------------------------------

class CVarAutoGen
{
public:
    CVarAutoGen();
    virtual ~CVarAutoGen();

	CString m_strSuffix;
    CString m_strPrefix ;
    long    m_lLowerLimit ;
    long    m_lUpperLimit ;
    long    m_lStepValue ;
};

//---------------------------------------------------------------------------

class CVarDBSession
{
public:
    CVarDBSession();
    virtual ~CVarDBSession();

    CDatabase m_Database ;
    BOOL Connect(CString _strDSN, CString  _strUserLogin, CString _strUserPwd) ;
    void Disconnect() ;
    BOOL IsConnected() ;

private:
    CString * m_pstrDSN ;
    CString * m_pstrUserLogin ;
    CString * m_pstrUserPwd ;

    BOOL m_bConnected ;
};

//---------------------------------------------------------------------------

class CVarDBParam
{
public:
    CVarDBParam();
    virtual ~CVarDBParam() ;
    BOOL Connect() ;
    void Disconnect() ;
    CString GetRequestSource() ;

    CString         m_strDSN ;
    CString         m_strUserLogin ;
    CString         m_strUserPwd ;

    CString         m_strTableName ;
    CString         m_strColumnName ;
    CString         m_strTableOwner ;

    CString         m_strReqSource ;
    long            m_lReqType ;        // Table or Request
    long            m_lReqSelType ;     // AllRows or NumberMax
    long            m_lMaxRowNumber ;

    CVarDBSession   m_VarDBSession ;
};

//---------------------------------------------------------------------------

class CVarFile
{
public:
    CVarFile();
    virtual ~CVarFile() ;

    CString m_strFilePath ;
    CString m_strFVRPath ;

    void SetName(CString _strValue);
protected:
    CString DeleteQuote(CString _strValue);

};

//---------------------------------------------------------------------------

class CVarGeneral
{
public:
    CString  m_strVarName ;
    long     m_lVarScope ;
    long     m_lSourceInfo ;
    long     m_lVarOrder ;
    long     m_lVarType ;
	long	 m_lSeed;

    CVarGeneral();
    virtual CVarGeneral::~CVarGeneral() ;
};

//---------------------------------------------------------------------------

class CVarValueList
{
public:
    bool            m_bGenerateFile;
    CStringArray    m_Values;

    CVarValueList();
    virtual ~CVarValueList();
    void Add(CString _strValue);

protected:
    CString DeleteQuote(CString _strValue);

};

//---------------------------------------------------------------------------

class CVarInfo
{
public:
    CVarGeneral     m_VarGeneral ;
    CVarValueList   m_VarValueList ;
    CVarFile        m_VarFile ;

    CVarInfo() ;
    virtual ~CVarInfo() ;

};


//---------------------------------------------------------------------------

enum ResponseType
{
    RT_HEADER,
    RT_BODY
};

class CVarResponse
{
public:
    long            m_lConId ;
    CString         m_strVarName ;
    CString         m_strIdentifier ;
    ResponseType    m_ResponseType ;

    CVarResponse() ;
    virtual ~CVarResponse() ;

};


#endif

