/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Table.cpp : implementation file
//
#include "StdAfx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__ ;
#endif

#include "Table.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
CDBTable::CDBTable(CDatabase* _pDB, BOOL _bAlwaysBinding)
        :   m_nRowSetSize(100),
        m_pDatabase(_pDB),
        m_bAlwaysBinding(_bAlwaysBinding),
        m_bFirstTimeI(TRUE),
        m_bFirstTimeU(TRUE),
        m_bFirstTimeD(TRUE) ,
        m_bIsOpen( FALSE )
{
    m_hstmtS    = SQL_NULL_HSTMT;
    m_hstmtI    = SQL_NULL_HSTMT;
    m_hstmtU    = SQL_NULL_HSTMT;
    m_hstmtD    = SQL_NULL_HSTMT;
    m_strSQL.Empty();
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
CDBTable::~CDBTable()
{
    if  ( m_hstmtS != SQL_NULL_HSTMT )
    {
        ASSERT ( m_pDatabase->IsOpen() );
        SQLFreeStmt(m_hstmtS,SQL_DROP);
    }
    if  ( m_hstmtI != SQL_NULL_HSTMT )
    {
        ASSERT ( m_pDatabase->IsOpen() );
        SQLFreeStmt(m_hstmtI,SQL_DROP );
    }
    if  ( m_hstmtU != SQL_NULL_HSTMT )
    {
        ASSERT ( m_pDatabase->IsOpen() );
        SQLFreeStmt(m_hstmtU,SQL_DROP);
    }
    if  ( m_hstmtD != SQL_NULL_HSTMT )
    {
        ASSERT ( m_pDatabase->IsOpen() );
        SQLFreeStmt(m_hstmtD,SQL_DROP);
    }
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
BOOL CDBTable::Open(BOOL _bIUD)
{
    ASSERT ( m_pDatabase->IsOpen() );

    BOOL bRet = TRUE;

    bRet = bRet && OpenS();

    if ( _bIUD )
    {
        bRet = bRet && OpenI();
        bRet = bRet && OpenU();
        bRet = bRet && OpenD();
    }

    m_bIsOpen = bRet;

    return bRet;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
BOOL CDBTable::OpenS()
{
    if ( m_hstmtS != SQL_NULL_HSTMT)
        return TRUE;

    RETCODE ReturnCode;

    ReturnCode = SQLAllocStmt(m_pDatabase->m_hdbc, &m_hstmtS);

    return ( ReturnCode != SQL_ERROR );
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
BOOL CDBTable::OpenI()
{
    if ( m_hstmtI != SQL_NULL_HSTMT)
        return TRUE;

    RETCODE ReturnCode;

    m_bFirstTimeI = TRUE;
    ReturnCode = SQLAllocStmt(m_pDatabase->m_hdbc, &m_hstmtI);

    return ( ReturnCode != SQL_ERROR );
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
BOOL CDBTable::OpenU()
{
    if ( m_hstmtU != SQL_NULL_HSTMT)
        return TRUE;

    RETCODE ReturnCode;

    m_bFirstTimeU = TRUE;
    ReturnCode = SQLAllocStmt(m_pDatabase->m_hdbc, &m_hstmtU);

    return ( ReturnCode != SQL_ERROR );
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
BOOL CDBTable::OpenD()
{
    if ( m_hstmtD != SQL_NULL_HSTMT)
        return TRUE;

    RETCODE ReturnCode;

    m_bFirstTimeD = TRUE;
    ReturnCode = SQLAllocStmt(m_pDatabase->m_hdbc, &m_hstmtD);

    return ( ReturnCode != SQL_ERROR );
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void CDBTable::Close()
{
    if  ( !IsOpen() )
        return;

    if ( m_hstmtS != SQL_NULL_HSTMT )
    {
        SQLFreeStmt(m_hstmtS,SQL_CLOSE);
        SQLFreeStmt(m_hstmtS,SQL_DROP );
        m_hstmtS = SQL_NULL_HSTMT ;
    }
    if ( m_hstmtI != SQL_NULL_HSTMT )
    {
        SQLFreeStmt(m_hstmtI,SQL_CLOSE);
        SQLFreeStmt(m_hstmtI,SQL_DROP );
        m_hstmtI = SQL_NULL_HSTMT ;
    }
    if ( m_hstmtU != SQL_NULL_HSTMT )
    {
        SQLFreeStmt(m_hstmtU,SQL_CLOSE);
        SQLFreeStmt(m_hstmtU,SQL_DROP );
        m_hstmtU = SQL_NULL_HSTMT ;
    }
    if ( m_hstmtD != SQL_NULL_HSTMT )
    {
        SQLFreeStmt(m_hstmtD,SQL_CLOSE);
        SQLFreeStmt(m_hstmtD,SQL_DROP );
        m_hstmtD = SQL_NULL_HSTMT ;
    }

    m_bIsOpen = FALSE;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
BOOL CDBTable::IsOpen()
{
    return m_bIsOpen;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void CDBTable::Free()
{
    if ( m_pDatabase->IsOpen() )
    {

        if  (m_hstmtS)
        {
            SQLFreeStmt(m_hstmtS,SQL_CLOSE);
        }
    }
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
BOOL CDBTable::DropHstmt(HSTMT* _phstmt)
{
    ASSERT( m_bIsOpen );
    ASSERT( *_phstmt != SQL_NULL_HSTMT );

    RETCODE ReturnCode = SQLFreeStmt(*_phstmt,SQL_DROP );
    *_phstmt = SQL_NULL_HSTMT;

    return (ReturnCode != SQL_ERROR);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
CDatabase*  CDBTable::GetDatabase()
{
    return m_pDatabase;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void CDBTable::SetSQL(CString _strSQL)
{
    m_strSQL = _strSQL;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
SQLCHAR *CDBTable::GetNullValue( SDWORD _ind, SQLCHAR *_buffer )
{
    if ( _ind == SQL_NULL_DATA )
        return (SQLCHAR *)"";
    else
        return _buffer;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
SQLINTEGER CDBTable::GetNullValue( SDWORD _ind, SQLINTEGER _value)
{
    if ( _ind == SQL_NULL_DATA )
        return ID_DATA_NULL;
    else
        return _value;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
SQLDOUBLE CDBTable::GetNullValue( SDWORD _ind, SQLDOUBLE _value)
{
    if ( _ind == SQL_NULL_DATA )
        return ID_DATA_NULL;
    else
        return _value;
}

