/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include <afxdllx.h>
#include "PluginTypes.h"
#include "resource.h"       // main symbols
#include "ScriptMonDlg.h"
#include "TestFile.h"
#include "HttpInfo.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


static AFX_EXTENSION_MODULE CHttpMonDLL = { NULL, NULL };

CMultiDocTemplate*      pTInfoDocTemplate    = NULL;
CMultiDocTemplate*      pVUsersDocTemplate    = NULL;
CTestFile* m_pTestFile = NULL;
CStringArray m_arrTasks;
int m_nTaskGroup = -1;
CHTTPInfo* m_pHttpInfo = NULL;

HINSTANCE m_hInstance = NULL;
extern "C" int APIENTRY
    DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
	m_hInstance = hInstance;
    // Remove this if you use lpReserved
    UNREFERENCED_PARAMETER(lpReserved);

    if (dwReason == DLL_PROCESS_ATTACH)
    {
        TRACE0("CHttpMonDLL.DLL Initializing!\n");

        // Extension DLL one-time initialization
        if (!AfxInitExtensionModule(CHttpMonDLL, hInstance))
            return 0;

        // Insert this DLL into the resource chain
        // NOTE: If this Extension DLL is being implicitly linked to by
        //  an MFC Regular DLL (such as an ActiveX Control)
        //  instead of an MFC application, then you will want to
        //  remove this line from DllMain and put it in a separate
        //  function exported from this Extension DLL.  The Regular DLL
        //  that uses this Extension DLL should then explicitly call that
        //  function to initialize this Extension DLL.  Otherwise,
        //  the CDynLinkLibrary object will not be attached to the
        //  Regular DLL's resource chain, and serious problems will
        //  result.
	
		CyrtoolsInitialize();
    }
    else if (dwReason == DLL_PROCESS_DETACH)
    {
        TRACE0("CHttpMonDLL.DLL Terminating!\n");

        // Terminate the library before destructors are called
        AfxTermExtensionModule(CHttpMonDLL);
		CyrtoolsUninitialize();
    }

    return 1;   // ok
}

extern "C" AFX_EXT_API
BOOL OpenSTAVerify()
{
	return true;
}

extern "C" AFX_EXT_API
void GetTitle(CString& strTitle)
{
	strTitle = "HTTP Monitor Plugin";
}

extern "C" AFX_EXT_API
int GetType()
{
	return TEST_MONITOR_PLUGIN;
}

extern "C" AFX_EXT_API
void GetPackage(CString& strPackage)
{
	strPackage = "HTTP";
}

extern "C" AFX_EXT_API
BOOL IsSupportedExtension(CString strExt)
{
	if (strExt == "HTP")
		return true;
    return false;
}

extern "C" AFX_EXT_API
void GetAvailableScreens(CTestFile* pTestFile, int nTaskGroup, CStringArray& arrScreens)
{
	m_nTaskGroup = nTaskGroup;
	m_arrTasks.RemoveAll();
	m_pTestFile = pTestFile;

	for (int i =0; i<pTestFile->GetNumTasks(nTaskGroup); i++)
	{
		CString answer;
		answer.Format("%d %s",i+1
			,pTestFile->GetTaskName(nTaskGroup, i));
		arrScreens.Add(answer);
		m_arrTasks.Add(answer);
	}
}

extern "C" AFX_EXT_API
CWnd* ViewScreen(CString strManagerReg, CString strTestRunPath, CStringArray& arrScreens, CWnd* pParent)
{
	// Set the resource handle to be this dll while opening a document
	if (arrScreens.GetSize() != 1)
		return NULL;

	if (m_pHttpInfo == NULL)
	{
		m_pHttpInfo = new CHTTPInfo();
	}
	// Get the current Resource handle
	HINSTANCE hInst = AfxGetResourceHandle();

	CString strTaskName = arrScreens.GetAt(0);
	//remove up to ','
	char* token;
	token = strtok( strTaskName.GetBuffer(0), " ");
	int nTask=atoi(token);
	nTask--;

	CString answer;

	if(nTask>=0 && nTask<m_pTestFile->GetNumTasks(m_nTaskGroup))
	{
		answer.Format("%d %s",nTask+1,m_pTestFile->GetTaskName(m_nTaskGroup, nTask));
	//	AfxMessageBox(answer);
	}
	else
	{
		answer = "error";
	}
	
	AfxSetResourceHandle(m_hInstance);
	CScriptMonDlg* pMon = new CScriptMonDlg();
	pMon->InitMonitor(answer, m_nTaskGroup, nTask, strManagerReg);
	pMon->Create(IDD_SCRIPTMON, pParent);
	pMon->ShowWindow(SW_SHOW);
	// Restore the resource handle
	AfxSetResourceHandle(hInst);
	return pMon;
}

//AfxMessageBox(strTaskName);

/*	for (int nTaskID = 0; nTaskID<m_pTestFile->GetNumTasks(m_nTaskGroup); nTaskID++)
	{
		CString sTask = m_pTestFile->GetTaskName(m_nTaskGroup, nTaskID);

		if (sTask == strTaskName)
		{
			break;
		}
	}

  */