/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// HTTPGraphs.h: interface for the CHTTPGraphs class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HTTPGRAPHS_H__30804214_E70F_11D4_94C9_00C04F89E392__INCLUDED_)
#define AFX_HTTPGRAPHS_H__30804214_E70F_11D4_94C9_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "OstaDefs.h" 

class CHttpStatsOcx;
class CTimerList;
class CGraphWnd;

struct Timer
{
	DWORD dwTimeStamp;
	double dValue;
	DWORD dwActiveUsers;
	CString strTestName;
	CString strThreadName;
	CString strScriptName;
	CString strTimerName;
	bool bInFilter;

	Timer()
	{
		bInFilter = true;
	}
};

typedef CList <Timer*,Timer*> TimerList;

#pragma pack (1)
typedef struct
{
    DWORD dwTimeStamp;
    DWORD dwIP;     //IP Address one byte per field
    DWORD wPID;     //Process Id
    WORD wUser;     //User Number
    DWORD dwResponseTime;
    WORD wResponseCode;
    DWORD dwReplySize;
    int nURLNdx;
}
HttpData;
#pragma pack()

typedef CMap <CString ,LPCTSTR,int,int> URLMap;
typedef CArray<HttpData*,HttpData*> HttpArray;

typedef struct
{
	DWORD dwTimeStamp;
	DWORD dwCount;
}VUData;

typedef CList <VUData*,VUData*> VUList;

class CHTTPGraphs  
{
public:
	CHTTPGraphs();
	virtual ~CHTTPGraphs();

	void LoadRun(const CString& strRunFolder);
	CWnd *DisplayGraph(const CString& strName,CWnd *pParent,const CRect& rcBounds);
	void GetAvailableGraphs(GraphInfoArray& arGraphs);

private:
	CHttpStatsOcx*	LoadHTTPDataList(CWnd *pParent,const CRect& rcBounds);
	CTimerList*		LoadTimerList	(CWnd* pParent,const CRect& rcBounds);
	BOOL			LoadTimerGraph	(const CString& strName,CGraphWnd *pGraph);
	CGraphWnd*		LoadTimerGraph(const CString& strName,const CRect& rcBounds,CWnd *pParent);
	CGraphWnd*		LoadHttpGraph(const CString& strName,const CRect& rcBounds,CWnd *pParent);
	CGraphWnd*		LoadVUGraph(const CString& strName,const CRect& rcBounds,CWnd *pParent);

	void LoadTimers();
	void LoadVUData();
	void LoadHttp();
	BOOL LoadCachedHttpData();
	void CacheHttpData();

	static int SortHttpData( const void *arg1, const void *arg2 );
	static int CompareTimerStart( const void *arg1, const void *arg2 );

private:
	CString m_strLoadedRun;
	DWORD m_dwTestStartTime;

	HttpArray m_arHttp;
	URLMap m_mapURLs;
	TimerList m_listTimers;
	VUList m_listVUs;

	const CString m_strDataList;
	const CString m_strBytesPerSecVTime;
	const CString m_strRespTimeVNumResps;
	const CString m_strErrorsVUsers;
	const CString m_strErrorsVTime;
	const CString m_strRespsVTime;
	const CString m_strRespTimeVTime;
	const CString m_strUsersVTime;
	const CString m_strTimerList;
	const CString m_strTimersVUsers;
	const CString m_strTimersVTime;
};

#endif // !defined(AFX_HTTPGRAPHS_H__30804214_E70F_11D4_94C9_00C04F89E392__INCLUDED_)
