/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SubroutineDocument.cpp : implementation file
//

#include "stdafx.h"
#include "Modeller.h"
#include "SubroutineDocument.h"
#include "SubroutineView.h"
#include "Constantes.h"
#include "CaptureSaveDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CModellerApp theApp ;
/////////////////////////////////////////////////////////////////////////////
// CSubroutineDocument
//----------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CSubroutineDocument, CBasedDocument)

//----------------------------------------------------------------------------

CSubroutineDocument::CSubroutineDocument():CBasedDocument()
{}

//----------------------------------------------------------------------------


BOOL CSubroutineDocument::OnNewDocument()
{
    if (!CBasedDocument::OnNewDocument())
        return FALSE;
    return TRUE;
}

//----------------------------------------------------------------------------

CSubroutineDocument::~CSubroutineDocument()
{}

//----------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CSubroutineDocument, CBasedDocument)
//{{AFX_MSG_MAP(CSubroutineDocument)
ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSubroutineDocument diagnostics
//----------------------------------------------------------------------------

#ifdef _DEBUG
void CSubroutineDocument::AssertValid() const
{
    CBasedDocument::AssertValid();
}

//----------------------------------------------------------------------------

void CSubroutineDocument::Dump(CDumpContext& dc) const
{
    CBasedDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSubroutineDocument serialization
//----------------------------------------------------------------------------

void CSubroutineDocument::Serialize(CArchive& ar)
{
    if (ar.IsStoring())
    {
        // TODO: add storing code here
    }

    else
    {
        // TODO: add loading code here
    }

}

//----------------------------------------------------------------------------

void CSubroutineDocument::OnFileSaveAs()
{
    CCaptureSaveDlg dlg(CCaptureSaveDlg::enTemplateMode) ;
    if(dlg.DoModal() == IDOK)
    {
        CWaitCursor wc;

        CString strSCLFile = dlg.m_strScriptName;
		
        if(strSCLFile.Right(strlen(STR_SCL_EXTENSION)) != STR_SCL_EXTENSION)
            strSCLFile += STR_SCL_EXTENSION;

        int nSlashPos = strSCLFile.ReverseFind('\\');
		CString strSCLName = strSCLFile.Mid(nSlashPos + 1);

		//Get the location of the existing all file and fvr files
        CString strWorkingFolder = GetSCLDocInfo()->GetWorkingFolder();

        CString strOldSclFile = strWorkingFolder + GetSCLDocInfo()->GetSCLFileName();
        CString strNewSclFile = strWorkingFolder + strSCLName;
        CFile::Rename(strOldSclFile,strNewSclFile);

        //Setup new file information
		GetSCLDocInfo()->SetOriginalPath(strSCLFile);
        GetSCLDocInfo()->SetFileInfo(strSCLName) ;
        SetTitle(strSCLName);

        DoSave(NULL);

        CSubroutineView *pView = GetSubroutineView();
        pView->SetModified(FALSE);
    }


    //  CBasedDocument::OnFileSaveAs();
}

//----------------------------------------------------------------------------


BOOL CSubroutineDocument::RetrieveFromRepository()
{
    CWaitCursor wc;

    CFile file ;
    CString strScript;
    CString strSCLFile = GetSCLDocInfo()->GetOriginalPath();

    if( file.Open( strSCLFile, CFile::modeRead|CFile::shareDenyWrite) )
    {
        int nCount = file.GetLength() ;

        file.Read( strScript.GetBufferSetLength(nCount) , nCount ) ;

        file.Close() ;

        CSubroutineView *pView = GetSubroutineView();
        if(pView)
        {
            pView->SetText(strScript);
            pView->SetModified(FALSE);
        }

        //Copy file to working folder
        CString strWorkingPath = GetSCLDocInfo()->GetWorkingFolder();
        CopyFile(strSCLFile,strWorkingPath + GetSCLDocInfo()->GetSCLFileName(),false);
        return true;
    }
    return false;
}

//----------------------------------------------------------------------------

CSubroutineView * CSubroutineDocument::GetSubroutineView()
{
    POSITION pos = GetFirstViewPosition();
    if(pos)
        return (CSubroutineView*)GetNextView(pos);

    return NULL;
}

//----------------------------------------------------------------------------

BOOL CSubroutineDocument::DoSave(LPCTSTR lpszPathName, BOOL bReplace)
{
    CWaitCursor wc;

    CString strScript;
    CSubroutineView *pView = GetSubroutineView();
    if(pView)
        pView->GetText(strScript);

    CString strSCLFile = GetSCLDocInfo()->GetFullSCLFileName();

    try
    {
        CFile f(strSCLFile,CFile::modeCreate|CFile::modeWrite);

        f.Write(strScript.GetBuffer(0),strScript.GetLength());

        f.Close();

        if(SaveToRepository())
        {
            CSubroutineView *pView = GetSubroutineView();
            if(pView)
                pView->SetModified(FALSE);

			return true;
        }
    }
    catch(...)
    {}

    return false;
}

//----------------------------------------------------------------------------

BOOL CSubroutineDocument::SaveToRepository()
{
    CWaitCursor wc;

    MoveFileToRepositoryStorage();

    return TRUE;
}

//----------------------------------------------------------------------------

BOOL CSubroutineDocument::MoveFileToRepositoryStorage()
{
    CString strSCLFromFullPath = GetSCLDocInfo()->GetFullSCLFileName();
    CString strSCLToFullPath = GetSCLDocInfo()->GetOriginalPath();
    return CopyFile(  strSCLFromFullPath , strSCLToFullPath, FALSE ) ;
}

//----------------------------------------------------------------------------

BOOL CSubroutineDocument::SaveModified()
{
    CSubroutineView *pView = GetSubroutineView();
    if(pView)
        SetModifiedFlag(pView->IsModified());

    return CBasedDocument::SaveModified();
}

//----------------------------------------------------------------------------

void CSubroutineDocument::OnCloseDocument()
{
    theApp.GetRepositoryAPI()->SaveProjectTemplateFiles(GetSCLDocInfo()->GetSCLFileName() , true  ) ;
    DeleteWorkingDirectory();

    theApp.UpdateWindowText();
    CBasedDocument::OnCloseDocument();
}

//----------------------------------------------------------------------------


void CSubroutineDocument::DeleteWorkingDirectory()
{
    CString strWorkingDir = GetSCLDocInfo()->GetWorkingFolder();

    CFileFind *pFileFind = new CFileFind;
    if(pFileFind->FindFile(strWorkingDir + "*"))
    {
        BOOL bQuit = false;
        do
        {
            bQuit = !pFileFind->FindNextFile();

            if(pFileFind->IsDots())
                continue;

            if(pFileFind->IsDirectory())
                continue;

            DeleteFile(pFileFind->GetFilePath());

        }
        while(!bQuit);
    }
    delete pFileFind;
    pFileFind = NULL;

    RemoveDirectory(strWorkingDir);
}

void CSubroutineDocument::SetErrorLine(int nLine)
{
	CSubroutineView *pView = GetSubroutineView();
	pView->SetErrorLine(nLine);	
}