/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SubroutineView.cpp : implementation file
//

#include "stdafx.h"
#include "Modeller.h"
#include "SubroutineView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CModellerApp theApp ;

/////////////////////////////////////////////////////////////////////////////
// CSubroutineView
//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CSubroutineView, CView)

//---------------------------------------------------------------------------

CSubroutineView::CSubroutineView():
        m_Editor( theApp.GetRegistryPath() )
{}

//---------------------------------------------------------------------------


CSubroutineView::~CSubroutineView()
{}

//---------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CSubroutineView, CView)
//{{AFX_MSG_MAP(CSubroutineView)
ON_COMMAND(IDM_TOGGLEBOOKMARK, OnTogglebookmark)
ON_UPDATE_COMMAND_UI(IDM_TOGGLEBOOKMARK, OnUpdateTogglebookmark)
ON_WM_SIZE()
ON_WM_CREATE()
ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
ON_COMMAND(ID_EDIT_CUT, OnEditCut)
ON_COMMAND(ID_EDIT_FIND, OnEditFind)
ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
ON_COMMAND(ID_EDIT_REPLACE, OnEditReplace)
ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
ON_UPDATE_COMMAND_UI(ID_EDIT_FIND, OnUpdateEditFind)
ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
ON_UPDATE_COMMAND_UI(ID_EDIT_REPLACE, OnUpdateEditReplace)
ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSubroutineView drawing
//---------------------------------------------------------------------------

void CSubroutineView::OnDraw(CDC* pDC)
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CSubroutineView diagnostics
//---------------------------------------------------------------------------


#ifdef _DEBUG
void CSubroutineView::AssertValid() const
{
    CView::AssertValid();
}

//---------------------------------------------------------------------------

void CSubroutineView::Dump(CDumpContext& dc) const
{
    CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSubroutineView message handlers
//---------------------------------------------------------------------------

COSEditCtrl* CSubroutineView::GetEditCtrl()
{
    return &m_Editor ;
}

//---------------------------------------------------------------------------

void CSubroutineView::OnTogglebookmark()
{
    //$$$OS m_Editor.OnTogglebookmark();
}

//---------------------------------------------------------------------------


void CSubroutineView::OnUpdateTogglebookmark(CCmdUI* pCmdUI)
{
    //$$$OS m_Editor.OnUpdateBookmarkset( pCmdUI ) ;
}

//---------------------------------------------------------------------------


void CSubroutineView::SetText(const CString& strText)
{
    m_Editor.SetWindowText(strText);
}

//---------------------------------------------------------------------------

void CSubroutineView::GetText(CString& strText)
{
    strText = m_Editor.GetWindowText();
}

//---------------------------------------------------------------------------

BOOL CSubroutineView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
    if(m_Editor.OnCmdMsg(nID,nCode,pExtra,pHandlerInfo))
        return TRUE;

    return CView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

//---------------------------------------------------------------------------

BOOL CSubroutineView::IsModified()
{
    return m_Editor.IsModified();
}

//---------------------------------------------------------------------------

void CSubroutineView::SetModified(BOOL bModified)
{
    m_Editor.SetModified(bModified);
}

//---------------------------------------------------------------------------

void CSubroutineView::OnSize(UINT nType, int cx, int cy)
{
    CView::OnSize(nType, cx, cy);

    if ( m_Editor.m_hWnd )
        m_Editor.SetWindowPos( NULL, -1, -1, cx, cy, SWP_NOZORDER | SWP_NOMOVE );

}

//---------------------------------------------------------------------------

int CSubroutineView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CView::OnCreate(lpCreateStruct) == -1)
        return -1;

    if ( m_Editor.Create( WS_CHILD | WS_VISIBLE | WS_VSCROLL, CRect( 0, 0, 100, 100 ), this, 100 ) == -1 )
        return -1;

    return 0;
}

//---------------------------------------------------------------------------

void CSubroutineView::OnEditCopy()
{
    m_Editor.Copy();

}

//---------------------------------------------------------------------------

void CSubroutineView::OnEditCut()
{
    m_Editor.Cut();
}

//---------------------------------------------------------------------------

void CSubroutineView::OnEditFind()
{
    m_Editor.SendMessage(CMM_EXECUTECMD, CMD_FIND, 0 );
}

//---------------------------------------------------------------------------

void CSubroutineView::OnEditPaste()
{
    m_Editor.Paste();
}

//---------------------------------------------------------------------------

void CSubroutineView::OnEditRedo()
{
    m_Editor.Redo();
}

//---------------------------------------------------------------------------

void CSubroutineView::OnEditReplace()
{
    ::SendMessage( m_Editor.m_hWnd, CMM_EXECUTECMD, CMD_FINDREPLACE, 0 );
}

//---------------------------------------------------------------------------

void CSubroutineView::OnEditUndo()
{
    m_Editor.Undo();

}

//---------------------------------------------------------------------------

void CSubroutineView::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCopy());
}

//---------------------------------------------------------------------------

void CSubroutineView::OnUpdateEditCut(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCut());
}

//---------------------------------------------------------------------------

void CSubroutineView::OnUpdateEditFind(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSubroutineView::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanPaste());

}

//---------------------------------------------------------------------------

void CSubroutineView::OnUpdateEditRedo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanRedo());
}

//---------------------------------------------------------------------------

void CSubroutineView::OnUpdateEditReplace(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSubroutineView::OnUpdateEditUndo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanUndo());
}

//---------------------------------------------------------------------------

void CSubroutineView::SetErrorLine(int nLine)
{
	((CMDIChildWnd*)GetParentFrame())->MDIActivate();

	m_Editor.GoToLine( nLine ) ;
    m_Editor.SelectLine(nLine,TRUE) ;
    m_Editor.SetFocus() ;
}
