/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarFilePPage.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "VarFilePPage.h"
#include "htmlhelp.h"
#include "Help\VarFilePPage.inl"
#include <ConstantesLib.h>
#include "VarPSheet.h"
#include "RegFunctions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVarFilePPage property page
//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CVarFilePPage, CPropertyPage)

//---------------------------------------------------------------------------

CVarFilePPage::CVarFilePPage(CVarFile * _pVarFile, BOOL _bCreateMode) : CPropertyPage(CVarFilePPage::IDD)
{
    //{{AFX_DATA_INIT(CVarFilePPage)
	m_bCopyFileBtn = FALSE;
	m_strCopyFile = _T("");
	m_strDataFile = _T("");
	//}}AFX_DATA_INIT

    m_pVarFile      = _pVarFile ;
    m_bCreateMode   = _bCreateMode ;	
	m_bCreatedFile = false;
}

//---------------------------------------------------------------------------

CVarFilePPage::~CVarFilePPage()
{}

//---------------------------------------------------------------------------
void CVarFilePPage::DoDataExchange(CDataExchange* pDX)
{
    CPropertyPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CVarFilePPage)
	DDX_Control(pDX, IDC_BROWSE, m_ctlBrowse);
	DDX_Control(pDX, ID_EDITFILE_BUTTON, m_ctlEditFile);
	DDX_Control(pDX, IDC_COPY_FILE, m_ctlCopyFile);
	DDX_Control(pDX, IDC_COPY_FILE_BTN, m_ctlCopyFileBtn);
	DDX_Control(pDX, IDC_COPY_FRAME, m_ctlCopyFrame);
	DDX_Check(pDX, IDC_COPY_FILE_BTN, m_bCopyFileBtn);
	DDX_Text(pDX, IDC_COPY_FILE, m_strCopyFile);
	DDX_Text(pDX, ID_VAR_FILEPATH_EDIT, m_strDataFile);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CVarFilePPage, CPropertyPage)
	//{{AFX_MSG_MAP(CVarFilePPage)
	ON_BN_CLICKED(ID_EDITFILE_BUTTON, OnEditFileButton)
	ON_WM_CTLCOLOR()
	ON_WM_HELPINFO()
	ON_BN_CLICKED(IDC_COPY_FILE_BTN, OnCopyFileBtn)
	ON_EN_CHANGE(ID_VAR_FILEPATH_EDIT, OnChangeVarFilepathEdit)
	ON_BN_CLICKED(IDC_BROWSE, OnBrowse)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVarFilePPage message handlers
//---------------------------------------------------------------------------

BOOL CVarFilePPage::OnInitDialog()
{
    CPropertyPage::OnInitDialog() ;

    Values2Ctrl();

	m_ctlEditFile.EnableWindow( !m_strDataFile.IsEmpty() );

	m_bCopyFileBtn = FALSE;
	OnCopyFileBtn();

	UpdateData(FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


void CVarFilePPage::OnEditFileButton()
{
	UpdateData();
	
	CString strTargetFile = GetRepositoryPath() + strDataFolder + "\\" + m_strDataFile;
	if(m_strDataFile.Right(strlen(STR_FVR_EXTENSION)).CompareNoCase(STR_FVR_EXTENSION)!=0)
		strTargetFile += STR_FVR_EXTENSION;

	CFileStatus fs;
	if(m_bCopyFileBtn)
	{
		if(!CFile::GetStatus(m_strCopyFile,fs))
		{
			AfxMessageBox("Cannot find the file to copy.\nPlease check you have entered the filename and path correctly");
			return;
		}

		if(m_bCreateMode && CFile::GetStatus(strTargetFile,fs))
		{
			CString strMsg;
			strMsg.Format("The file \"%s\" already exists. Do you want to overwrite it?",m_strDataFile);
			if(AfxMessageBox(strMsg,MB_YESNO)==IDNO)
				return;
		}

		CopyFile(m_strCopyFile,strTargetFile,false);
	}
	else
	{
		if(!CFile::GetStatus(strTargetFile,fs))
		{
			CFile f;
			f.Open(strTargetFile,CFile::modeCreate);
		}
	}

	m_bCreatedFile = true;

    ShellExecute(m_hWnd,"open","notepad",strTargetFile,NULL,SW_SHOWNORMAL) ;
}

//---------------------------------------------------------------------------

void CVarFilePPage::Values2Ctrl()
{
	m_strDataFile = m_pVarFile->m_strFilePath;
	
	if(m_strDataFile.Right(strlen(STR_FVR_EXTENSION)).CompareNoCase(STR_FVR_EXTENSION)==0)
	{
		int nDotPos = m_strDataFile.ReverseFind('.');
		m_strDataFile = m_strDataFile.Left(nDotPos);
	}
	UpdateData(false);
}

//---------------------------------------------------------------------------

void CVarFilePPage::Ctrl2Values()
{
	UpdateData();
	if(m_strDataFile.Right(strlen(STR_FVR_EXTENSION)).CompareNoCase(STR_FVR_EXTENSION)==0)
	{
		int nDotPos = m_strDataFile.ReverseFind('.');
		m_strDataFile = m_strDataFile.Left(nDotPos);
	}

	m_pVarFile->m_strFilePath = m_strDataFile;
}


//---------------------------------------------------------------------------

LRESULT CVarFilePPage::OnWizardBack()
{
	if(((CVarPSheet*)GetParent())->m_Page1.m_pVarGeneral->m_lSourceInfo == VAR_SOURCE_LIST)
		return IDD_VAR_VALLIST_PPAGE ;
	else
		return IDD_VAR_GENERAL_PPAGE ;
}

//---------------------------------------------------------------------------

LRESULT CVarFilePPage::OnWizardNext()
{
    return CPropertyPage::OnWizardNext() ;
}

//---------------------------------------------------------------------------

HBRUSH CVarFilePPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

BOOL CVarFilePPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("OPENSTA_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

BOOL CVarFilePPage::OnWizardFinish() 
{
	if(!ValidateData())
		return false;
	return CPropertyPage::OnWizardFinish();
}


BOOL CVarFilePPage::OnSetActive() 
{
	((CPropertySheet*)GetParent())->SetWizardButtons(PSWIZB_BACK|PSWIZB_FINISH);
	
	return CPropertyPage::OnSetActive();
}

void CVarFilePPage::OnCopyFileBtn() 
{
	UpdateData();
	m_ctlCopyFrame.EnableWindow(m_bCopyFileBtn);
	m_ctlCopyFile.EnableWindow(m_bCopyFileBtn);
	m_ctlBrowse.EnableWindow(m_bCopyFileBtn);
}

void CVarFilePPage::OnChangeVarFilepathEdit() 
{
	UpdateData();
	m_ctlEditFile.EnableWindow( !m_strDataFile.IsEmpty() );
}

void CVarFilePPage::OnBrowse() 
{
	UpdateData();

    DWORD dwFlags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST ; 
    CFileDialog FileDlg(TRUE, NULL, m_strCopyFile, dwFlags, NULL, this) ;
    if (FileDlg.DoModal() == IDOK)
	{
		m_strCopyFile = FileDlg.GetPathName();
		if(m_strDataFile.IsEmpty())
			m_strDataFile = FileDlg.GetFileTitle();
		UpdateData(FALSE);

		OnChangeVarFilepathEdit();
	}
}

void CVarFilePPage::OnOK() 
{
	if(!ValidateData())
		return;
	
	CPropertyPage::OnOK();
}

BOOL CVarFilePPage::ValidateData()
{
	UpdateData();

	if(m_strDataFile.IsEmpty())
	{
		AfxMessageBox("Please enter a name for the data file");
		return false;
	}

	CString strTargetFile = GetRepositoryPath() + strDataFolder + "\\" + m_strDataFile;
	if(m_strDataFile.Right(strlen(STR_FVR_EXTENSION)).CompareNoCase(STR_FVR_EXTENSION)!=0)
		strTargetFile += STR_FVR_EXTENSION;

	CFileStatus fs;
	if(m_bCopyFileBtn)
	{
		if(!CFile::GetStatus(m_strCopyFile,fs))
		{
			AfxMessageBox("Cannot find the file to copy.\nPlease check you have entered the filename and path correctly");
			return false;
		}

		if(!m_bCreatedFile && m_bCreateMode && CFile::GetStatus(strTargetFile,fs))
		{
			CString strMsg;
			strMsg.Format("The file \"%s\" already exists. Do you want to overwrite it?",m_strDataFile);
			if(AfxMessageBox(strMsg,MB_YESNO)==IDNO)
				return false;
		}

		CopyFile(m_strCopyFile,strTargetFile,false);
	}
	else
	{
		if(!CFile::GetStatus(strTargetFile,fs))
		{
			CFile f;
			f.Open(strTargetFile,CFile::modeCreate);
		}
	}

	Ctrl2Values();
	return true;
}
