/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SclParser.cpp: implementation of the CSclParser class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "SclParser.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSclParser::CSclParser():
	m_nDefinitionsLine(-1)
{

}

CSclParser::~CSclParser()
{
	Reset();
}

void CSclParser::Reset()
{
	POSITION pos = m_listScriptItems.GetHeadPosition();
	while(pos)
		delete m_listScriptItems.GetNext(pos);
	m_listScriptItems.RemoveAll();
}

void CSclParser::Parse(const char* szScript)
{
	Reset();

	int nMultiLineStart=-1;
	bool bNeedMoreData=false;
	bool bInQuotes=false;
	register int nNumLines=0;
	char *pszCopy = strdup(szScript);
	register char *pPos=pszCopy;
	register char *pPrevPos = pPos;

	while (*pPos)
	{
		if(*pPos == '"')
			bInQuotes = !bInQuotes;
		
		if(*pPos == '\r')
			*pPos = ' ';
		
		//Handle comments by converting them to spaces
		if(*pPos == '!' && !bInQuotes)
		{
			char *pComment = pPos;
			while(*pPos && (*pPos != '\n') && (*pPos != '\r'))pPos++;
			*pPos = NULL;
			pPos++;
			
			CCommentInfo *pCommentInf = new CCommentInfo(pComment+1,nNumLines,nNumLines);
			m_listScriptItems.AddTail(pCommentInf);
			memset(pComment,' ',pPos-pComment);
			if(*pPos == '\n')
			{
				*pPos = ' ';
				nNumLines++;
			}
		}
		else
		if(*pPos == '\n')
		{
			*pPos = NULL;
			nNumLines++;

			if(!bNeedMoreData)
			{
				pPrevPos = StripWhiteSpace(pPrevPos);
				ProcessLine(pPrevPos,(nMultiLineStart == -1 ? nNumLines : nMultiLineStart) ,nNumLines);
				pPrevPos = pPos+1;
				nMultiLineStart = -1;
			}
			else
			{
				bNeedMoreData = false;
				*pPos = ' ';
			}
		}
		else 
		if(!bInQuotes && *pPos == '&')
		{
			if(nMultiLineStart == -1)
				nMultiLineStart = nNumLines+1;
			*pPos = ' ';
			bNeedMoreData = true;
		}

		pPos++;
	}
	free(pszCopy);
}

char *CSclParser::StripWhiteSpace(char *szLine)
{
	while(*szLine == '\t' || *szLine == ' ')
		szLine++;
	return szLine;
}

BOOL CSclParser::ProcessVariable(char *szLine,int nStartLine,int nEndLine)
{
	//Get the variable type
	CVariableInfo::VariableType nVarType= CVariableInfo::enCharacter;
	if(strnicmp(szLine, "CHARACTER",9)==0)
	{
		szLine+=9;
		nVarType = CVariableInfo::enCharacter;
	}
	else
	if(strnicmp(szLine,"INTEGER",7)==0)
	{
		szLine+=7;
		nVarType = CVariableInfo::enInteger;
	}
	else
		return false;

	szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
	
	//Parse optional size field
	int nSize = 1;
	if(*szLine == '*')
	{
		szLine = StripWhiteSpace(++szLine);	//Move to end of spaces and tabs etc
		nSize = atoi(szLine);
		while(isdigit(*szLine))
			szLine++;
		szLine = StripWhiteSpace(++szLine);	//Move to end of spaces and tabs etc
	}
	
	//Expect the next field to be the name
	char *szName = szLine;
	while(*szLine != ',' && *szLine!=' '&& *szLine!='(' && *szLine!='[') szLine++;
	
	if(*szLine == ' ')
	{
		*szLine = NULL;
		szLine = StripWhiteSpace(++szLine);	//Move to end of spaces and tabs etc
	}

	//The name can be followed by all sorts of optional things
	//eg scope, random, initialisation data , file ....

	if(*szLine == '[')	//Found an array,This variable was not created by the modeller!
		return false;
	
	//Check if this is a variable initialisation field
	char *szVariableInit = NULL;
	if(*szLine == '(')
	{
		*szLine = NULL;
		szLine++;
		szVariableInit = szLine;
		while(*szLine && (*szLine != ')')) szLine++;
		*szLine=NULL;
	}
	
	if(*szLine != ',')
	{
		*szLine = NULL;
		szLine++;
		szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
	}
	
	//Check for optional fields
	bool bRandom = false;
	bool bRepeatable = false;
	int nSeed = 0;
	CVariableInfo::VariableScope nScope = CVariableInfo::enLocal;
	char *szFile=NULL;
	while(*szLine == ',')
	{
		*szLine = NULL;
		szLine++;
		szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
		if(strnicmp(szLine,"LOCAL",5)==0)
		{
			szLine += 5;
			nScope = CVariableInfo::enLocal;
		}
		else
		if(strnicmp(szLine,"SCRIPT",6)==0)
		{
			szLine += 6;
			nScope = CVariableInfo::enScript;
		}
		else
		if(strnicmp(szLine,"THREAD",6)==0)
		{
			szLine += 6;
			nScope = CVariableInfo::enThread;
		}
		else
		if(strnicmp(szLine,"GLOBAL",6)==0)
		{
			szLine += 6;
			nScope = CVariableInfo::enGlobal;
		}
		else
		if(strnicmp(szLine,"REPEATABLE",10)==0)
		{
			szLine += 10;
			szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
			if(strnicmp(szLine,"RANDOM",6)!=0)
				return false;
			szLine += 6;
						
			bRepeatable = true;
			bRandom = true;
		}
		else
		if(strnicmp(szLine,"RANDOM",6)==0)
		{
			szLine += 6;
			bRandom = true;
		}
		else
		if(strnicmp(szLine,"SEED",4)==0)
		{
			szLine += 4;
			szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
			if(*szLine != '=')
				return false;
			szLine++;
			szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
			nSeed = atoi(szLine);
			while(*szLine && isdigit(*szLine)) szLine++;
		}
		else
		if(strnicmp(szLine,"FILE",4)==0)
		{
			szLine += 4;
			szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
			if(*szLine != '=')
				return false;
			szLine++;
			szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
		
			if(*szLine != '"')
				return false;
			szLine++;
			szFile = szLine;
				
			while(*szLine && (*szLine != '"')) szLine++;
			*szLine = NULL;
		}

		szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
	}

	CVariableInfo *pVariable = new CVariableInfo(nVarType,szName,nSize,szVariableInit,nScope,
										szFile,bRandom,nStartLine,nEndLine,bRepeatable,nSeed);
	AddScriptItem(pVariable);

	return true;
}

BOOL CSclParser::ProcessTimer(char *szLine,int nStartLine,int nEndLine)
{
	BOOL bStartTimer=true;
	if(strnicmp(szLine,"START",5)==0)
	{
		szLine += 5;
		szLine = StripWhiteSpace(szLine);
		bStartTimer=true;
	}
	else
	if(strnicmp(szLine,"END",3)==0)
	{
		szLine += 3;
		szLine = StripWhiteSpace(szLine);
		bStartTimer=false;
	}
	else
		return false;

	if(strnicmp(szLine,"TIMER",5)!=0)
		return false;

	szLine += 5;
	szLine = StripWhiteSpace(szLine);

	char *szName = szLine;
	while(*szLine && *szLine!=' ') szLine++;
	*szLine = NULL;

	CTimerInfo *pTimer = new CTimerInfo(szName,bStartTimer,nStartLine,nEndLine);
	AddScriptItem(pTimer);

	return true;
}

BOOL CSclParser::ProcessRequest(char *szLine,int nStartLine,int nEndLine)
{
	//Parse optional primary field
	bool bPrimary = false;
	if(strnicmp(szLine,"PRIMARY",7)==0)
	{
		szLine += 7; // move to end of primary keyword
		szLine = StripWhiteSpace(szLine);	//remove spaces and tabs etc
		bPrimary = true;
	}

	//Find out the request type (we're just interested in if the keyword exists NOT the type for the time being)
//	enum {enGet,enPost,enHead,enNone} RequestType = enNone;
	if(strnicmp(szLine,"GET",3)==0)
	{
		szLine += 3; // move to end of get keyword
//		RequestType = enGet;
	}
	else
	if(strnicmp(szLine,"POST",4)==0)
	{
		szLine += 4; // move to end of post keyword
//		RequestType = enPost;
	}
	else
	if(strnicmp(szLine,"HEAD",4)==0)
	{
		szLine += 4; // move to end of head keyword
//		RequestType = enHead;
	}
	else
		return false;

	szLine = StripWhiteSpace(szLine);	//remove spaces and tabs etc

	//expect URL or URI next
	if(strnicmp(szLine,"URL",3)!=0 && strnicmp(szLine,"URI",3)!=0)
		return false;
	else
		szLine += 3;
	
	szLine = StripWhiteSpace(szLine);	//remove spaces and tabs etc
	
	//Expect to have a quoted string containing the url 
	char *szUrl = ++szLine;
	//Get the url string
	while(*szLine != '"')	szLine++;
	*szLine = ' ';
	
	char *szLastPos=szLine;
	szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
	while(*szLine == '"')
	{
		*szLine= ' ';
		*szLine++;
		//The line has been continued so move everything back
		while(*szLine != '"')
		{
			*szLastPos = *szLine;
			*szLine++;
			*szLastPos++;
		}
		*szLine = ' ';
		szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
	}
	*szLastPos = NULL;
	
	//Expect to have the ON keyword
	if(strnicmp(szLine,"ON",2)==0)
		szLine+=2;
	else
		return false;
	
	szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
		
	//Then the connection id
	int nConId = atoi(szLine);
//	while(isdigit(*szLine))
//		szLine++;

//	szLine = StripWhiteSpace(szLine);	//Move to end of spaces and tabs etc
	
	//Ignore everything else for the time being

	//Add data to info class
	CRequestInfo *pUrl = new CRequestInfo(bPrimary, szUrl,nConId,nStartLine,nEndLine);
	AddScriptItem(pUrl);

	return true;
}

void CSclParser::ProcessLine(char *szLine,int nStartLine , int nEndLine)
{
	if(strnicmp(szLine,"DEFINITIONS",11)==0)
	{
		SetDefinitionsLine(nStartLine);
		return;
	}

	if(ProcessRequest(szLine,nStartLine,nEndLine))
		return;
	
	if(ProcessVariable(szLine,nStartLine,nEndLine))
		return;

	if(ProcessTimer(szLine,nStartLine,nEndLine))
		return;
}

#ifdef _DEBUG
void CSclParser::Dump() const 
{
	POSITION pos = m_listScriptItems.GetHeadPosition();
	while(pos)
	{
		m_listScriptItems.GetNext(pos)->Dump();
	}
}
#endif

int CSclParser::GetPrimaryUrls(UrlArray& arUrls) const 
{
	arUrls.RemoveAll();

	POSITION pos = m_listScriptItems.GetHeadPosition();
	while(pos)
	{
		CScriptItemInfo *pItem = m_listScriptItems.GetNext(pos);
		if((pItem->GetItemType() == CScriptItemInfo::enRequest) && ((CRequestInfo*)pItem)->IsPrimary())
		{
			arUrls.Add((CRequestInfo*)pItem);
		}
	}
	return arUrls.GetSize();
}

int CSclParser::GetVariableInsertLine() const 
{
	int nInsertLine=m_nDefinitionsLine;
	POSITION pos = m_listScriptItems.GetHeadPosition();
	while(pos)
	{
		CScriptItemInfo *pItem = m_listScriptItems.GetNext(pos);
		if(pItem->GetItemType() == CScriptItemInfo::enVariable)
		{
			int nLine = pItem->GetLastLine();
			if(nInsertLine < nLine)
				nInsertLine = nLine;
		}
	}
	return (nInsertLine + 1);
}

const CScriptItemInfo* CSclParser::GetItemOnLine(int nLine) const 
{
	POSITION pos = m_listScriptItems.GetHeadPosition();
	while(pos)
	{
		CScriptItemInfo *pItem = m_listScriptItems.GetNext(pos);
		if(nLine >= pItem->GetStartLine() && nLine<=pItem->GetLastLine())
		{
			return pItem;
		}
	}
	return NULL;
}

POSITION CSclParser::GetFirstRequestPosition()
{
	POSITION pos = m_listScriptItems.GetHeadPosition();
	POSITION posPrev;
	while(pos)
	{
		posPrev = pos;
		CScriptItemInfo *pItem = m_listScriptItems.GetNext(pos);
		if(pItem->GetItemType() == CScriptItemInfo::enRequest)
			return posPrev;
	}
	return NULL;
}

CRequestInfo* CSclParser::GetNextRequest(POSITION& pos)
{
	while(pos)
	{
		CScriptItemInfo *pItem = m_listScriptItems.GetNext(pos);
		if(pItem->GetItemType() == CScriptItemInfo::enRequest)
		{
			POSITION posPrev;
			while(pos)
			{
				posPrev = pos;
				CScriptItemInfo *pItemNext = m_listScriptItems.GetNext(pos);
				if(pItemNext->GetItemType() == CScriptItemInfo::enRequest)
				{
					pos = posPrev;
					break;
				}
			}

			return (CRequestInfo*)pItem;
		}
	}
	return NULL;
}

POSITION CSclParser::GetFirstItemPosition()
{
	return m_listScriptItems.GetHeadPosition();
}

CScriptItemInfo* CSclParser::GetNextItem(POSITION& pos)
{
	return m_listScriptItems.GetNext(pos);
}
