/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#include <iostream>
using namespace std;
#include "SNMPMain.h"
#include <OSTAORB\OSTAORB.h>
#include "TestExecuter_i.h"
#include "OnExecuterStop.hpp"
#include <mutex.hpp>
#include <ctime.hpp>
#include "process.h"
#include "ipmutex.hpp"
#include "OstaTraceDef.h"

CORBAServer* m_pServer = NULL;
TestExecuterImpl* TE;
string m_strRunString;
string m_strTestName;
string m_strReposHost;
ofstream* m_pLogFile;
string m_strLogFile;
string m_strSignal;
int m_nLogLevel = 0;
void usage();
void run();
void InitLogging();
void CloseLogging();

int main(int argc, char* argv[])
{
	bool bDebug = false;
	m_strReposHost = "";
	m_strTestName = "";
	m_strRunString = "";
	m_strSignal = "";

    for(int i=1;i<argc;i++)
    {
        if(strcmp(argv[i],"-v")==0)             // Node to run on
        {
            if(i+1<argc)m_nLogLevel=atoi(argv[++i]);
        }
        else if(strcmp(argv[i],"-d")==0)             // Node to run on
        {
            bDebug = true;
        }
        else if(strcmp(argv[i],"-r")==0)             // Node to run on
        {
            if(i+1<argc)m_strReposHost=argv[++i];
        }
        else if(strcmp(argv[i],"-n")==0)             // Node to run on
        {
            if(i+1<argc)m_strTestName=argv[++i];
        }
        else if(strcmp(argv[i],"-t")==0)             // Node to run on
        {
            if(i+1<argc)m_strRunString=argv[++i];
        }
        else if(strcmp(argv[i],"-s")==0)             // Node to run on
        {
            if(i+1<argc)m_strSignal=argv[++i];
        }
        else
        {
            usage();
            return 1;
        }
	}
	
	CyrtoolsInitialize();
	InitLogging();
	// Initialise CORBA
    try
    {
		cout<<"Initailizing CORBA Services..."<<endl;
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_LOW, "Connecting to CORBA Server");
        m_pServer=new CORBAServer(0,NULL);
    }
    catch(...)
    {
        cout<<"Caught Exception on startup, is Nameservice Running?"<<endl;
		LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, "Connecting to CORBA Server");
		CloseLogging();
        return 1;
    }

	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_LOW, "Starting execution");
	run();
	CyrtoolsUninitialize();

	return 0;
}

void usage()
{
	cout <<"Invalid command line"<<endl;
}

void run()
{
	try
	{
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_LOW, "Creating TestExecuter Interface");
		TE = new TestExecuterImpl(m_pServer, m_strRunString, m_strTestName, m_strReposHost);
		LOG_MESSAGE(TRCCAT_RUNNING, TRCLEV_LOW, "Registering log file");
		TE->m_pControl->SetResultsFile(m_strLogFile.c_str());
		LOG_MESSAGE(TRCCAT_RUNNING, TRCLEV_LOW, "Blocking until complete");

		CIPSignal condition;
		if(!condition.attach(m_strSignal.c_str())
			||!condition.isValid() ||!condition.signal())
		{
			LOG_MESSAGE(TRCCAT_RUNNING, TRCLEV_LOW, string("Error Sending Signal"+TE->m_registered_name));
			return;
		}

		onExecuterStop(TE->m_pControl, TE->m_registered_name);


		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_LOW, "Execution complete");
		LOG_MESSAGE(TRCCAT_SHUTDOWN, TRCLEV_LOW, "Store log file and shutdown.");
		TE->Shutdown();
		delete TE;	
		delete m_pServer;
		CloseLogging();
	}
	catch(...)
	{
	}
}

void LOG_MESSAGE(const char* category, int nLevel, string strMsg)
{
    // Only log messages at selected trace level and below.
	if (nLevel > m_nLogLevel) return;

	CTTime time;
	time.now();

	char strTmp[256];
    sprintf(strTmp, "%s,%s,%d,%s\r\n", time.Format("%H:%M:%S %d-%m-%Y").c_str(),category, nLevel, strMsg.c_str());

	string msg = strTmp;
    m_pLogFile->write( msg.c_str(), msg.length() );
	m_pLogFile->flush();
}

void InitLogging()
{
	char strTmp[256];
    sprintf(strTmp, "TExecuter_smp_%d.log", _getpid());
	m_strLogFile = strTmp;

	string strLocalPath = getenv("OPENSTA_HOME");
	strLocalPath += "Engines\\";
	strLocalPath += m_strLogFile;

    m_pLogFile = new ofstream( strLocalPath.c_str(), std::ios::binary|std::ios::app|std::ios::out);
    if( !m_pLogFile->good() )
    {
		return;
	}
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_LOW, "TExecuter_smp started.");

	char strMsg[64];
    sprintf(strMsg, "Trace Level = %d", m_nLogLevel);
	LOG_MESSAGE(TRCCAT_STARTUP, TRCLEV_NONE, strMsg);
}

void CloseLogging()
{
	m_pLogFile->flush();
	delete m_pLogFile;
}
