/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MonitorGraph.cpp : implementation file
//

#include "stdafx.h"
#include "MonitorGraph.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
static int DEFAULT_REFRESH_RATE = 500;
const int VtChSeriesType2dLine = 6;
#define IDT_POLL_DATA 999
extern HINSTANCE m_hInstance;

/////////////////////////////////////////////////////////////////////////////
// CMonitorGraph dialog

extern int currentTaskGroup;

CMonitorGraph::CMonitorGraph(CWnd* pParent /*=NULL*/)
	: CDialog(CMonitorGraph::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMonitorGraph)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_pParent = pParent;
	m_bDataFileOpen = false;
	m_bHeadersLoaded = false;
	m_nFilePos = 0;
	m_nRollingChecked=0;

	m_colPlotBk = RGB(0xff,0xff,0xff);
    m_strLabelFont = "Verdana";
    m_colGrid = RGB(0xf0,0xca,0xa6);
    m_colLabel = RGB(0,0,0);
    m_colAxisText = RGB(0,0,0);

	m_nTaskGroup=currentTaskGroup;

	LoadGraphStyles();
}


void CMonitorGraph::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMonitorGraph)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMonitorGraph, CDialog)
	//{{AFX_MSG_MAP(CMonitorGraph)
	ON_WM_TIMER()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_ROLLING_GRAPH, ToggleRollingGraphs)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMonitorGraph message handlers

void CMonitorGraph::InitMonitor(CString strRunFolder, CStringArray& arrDesc)
{
	m_strRunFolder = strRunFolder;
	for (int i = 0; i<arrDesc.GetSize(); i++)
	{
		m_arrDescriptions.Add(arrDesc.GetAt(i));
	}
}

BOOL CMonitorGraph::OnInitDialog() 
{
	CDialog::OnInitDialog();
	SetWindowText(m_strDesc);
	SetIcon(LoadIcon( m_hInstance, MAKEINTRESOURCE(IDI_MONITOR)), FALSE);

	SetTimer(IDT_POLL_DATA,DEFAULT_REFRESH_RATE,NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

CXChartCtrl* CMonitorGraph::CreateDisplay(CString strTitle)
{
    CXChartCtrl *pChart = new CXChartCtrl;

    if(!pChart)
        return NULL;

	CRect rcGraph;
	GetClientRect(rcGraph);

    pChart->Create(NULL,WS_CHILD|WS_VISIBLE,rcGraph,this,0);

    pChart->SetPlotBkColor(m_colPlotBk);

    pChart->SetLabelColor(m_colLabel);

    pChart->SetAxisTextColor(m_colAxisText);

    pChart->SetGridColor(m_colGrid);
    pChart->SetLabelFont(m_strLabelFont);

    pChart->AddDataPoint(1,0.0);
    pChart->SetSeriesStyle(1,VtChSeriesType2dLine);
    pChart->EnableZoom(FALSE);

	//Setup data colors
	COleSafeArray saData;
	saData.CreateOneDim(VT_I4,m_arDefLineColours.GetSize(),m_arDefLineColours.GetData());
    pChart->SetDataColors(saData);

    pChart->SetYAxisLabel(strTitle);
    return pChart;
}

void CMonitorGraph::OnTimer(UINT nIDEvent) 
{
	if (!m_bHeadersLoaded)
	{
		LoadHeaders();
		return;
	}

    try
    {
		if (!m_bDataFileOpen)
		{
			CFileStatus fs;
			CString StatPath = 	m_strRunFolder + "\\SNMPStat.txt";
			if (!CFile::GetStatus(StatPath, fs))
				return;
			m_bDataFileOpen = m_dataFile.Open(StatPath, CFile::modeRead | CFile::typeText | CFile::shareDenyNone );
			if (!m_bDataFileOpen)
				return;
		}
		m_dataFile.Seek(m_nFilePos, CFile::begin);

        CString strData;
		while(m_dataFile.ReadString(strData))
		{
            CString taskGroup, index, start, stop, value;
			GetNextField(',', strData, taskGroup);
			GetNextField(',', strData, index);
			GetNextField(',', strData, start);
			GetNextField(',', strData, stop);
			GetNextField(',', strData, value);
            double dVal = atof(value);
			int iTaskGroup=atoi(taskGroup);
			if((iTaskGroup==0)&&(taskGroup[0]!='0'))
			{
				//non integer response
				iTaskGroup=-1;
			}
			for (int i =0; i<m_arrGraphs.GetSize(); i++)
			{
				if ((atoi(index) == m_arrGraphs.GetAt(i).nDataIndex)&&(m_nTaskGroup==iTaskGroup))
				{
						//	CString asd;
						//	asd.Format("plotting a point for currently selected taskgroup%d, screen title= %s, value =%d",m_nTaskGroup,m_arrGraphs.GetAt(i).strDesc,(int)dVal);
						//	AfxMessageBox(asd);
				
					m_arrGraphs.GetAt(i).pChart->AddDataPoint(1,dVal);
					m_arrGraphs.GetAt(i).pChart->Invalidate();
				}
			}
        }

		m_nFilePos = m_dataFile.GetPosition();
    }
    catch(...)
    {
        GetParent()->PostMessage(WM_CLOSE);
    }		
	CDialog::OnTimer(nIDEvent);
}

int CMonitorGraph::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr == "")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep, 0);

    if (nPos == -1)
    {
        field = inStr;
        inStr = "";
        return 0;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr = inStr.Right(len - nPos - 1);
        return 0;
    }
}

void CMonitorGraph::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);

	if (m_arrGraphs.GetSize() == 0)
		return;

	int nHeight = cy/m_arrGraphs.GetSize();
	for (int i = 0; i<m_arrGraphs.GetSize(); i++)
	{
		GRAPH_INFO inf = m_arrGraphs.GetAt(i);
		if(inf.pChart && inf.pChart->m_hWnd)
		{
			inf.pChart->MoveWindow(0,i*nHeight,cx, nHeight);			
		}
	}
}

BOOL CMonitorGraph::OnEraseBkgnd(CDC *pDC)
{
	return true;
}


void CMonitorGraph::LoadHeaders()
{
	try
	{
		// Find the data index of this description
		CString HeaderPath = m_strRunFolder + "\\SNMPHeader.txt";
		CStdioFile f(HeaderPath, CFile::modeRead | CFile::typeText );

		CString headerInfo;
		while(f.ReadString(headerInfo))
		{
			GRAPH_INFO inf;

			CString tgid = strtok(headerInfo.GetBuffer(0),",");	
			if(atoi(tgid)==m_nTaskGroup)
			{
				//AfxMessageBox("loading snmp header");
		        CString strIndex = strtok(NULL,",");
			    inf.nDataIndex = atoi(strIndex);
			    inf.strDesc = strtok(NULL,",");
		        inf.strDesc.TrimRight(",");
				inf.pChart = NULL;

				for (int i = 0; i<m_arrDescriptions.GetSize(); i++)
				{
					if (inf.strDesc == m_arrDescriptions.GetAt(i))
					{
						inf.pChart = CreateDisplay(inf.strDesc);
						m_arrGraphs.Add(inf);
					}
				}	

				m_bHeadersLoaded = true;
				CRect rc;
				GetClientRect(rc);
				OnSize(SIZE_RESTORED,rc.Width(), rc.Height());
			}
		}
	}
	catch(...)
	{
	}

}

void CMonitorGraph::LoadGraphStyles()
{
	CWinApp *pApp = AfxGetApp();

    m_colPlotBk = pApp->GetProfileInt("GraphStyles","BGCol",(int)m_colPlotBk);
    m_strLabelFont = pApp->GetProfileString("GraphStyles","LabelFont", m_strLabelFont);
    m_colGrid = pApp->GetProfileInt("GraphStyles","GridCol",(int)m_colGrid);
    m_colLabel = pApp->GetProfileInt("GraphStyles","LabelCol",(int)m_colLabel);
    m_colAxisText = pApp->GetProfileInt("GraphStyles","ATCol",(int)m_colAxisText);

	CString strKey;
	int nNumLineColors = m_arDefLineColours.GetSize();
	for(int nCol=0;nCol<nNumLineColors;nCol++)
	{
		strKey.Format("LineCol%d",nCol);
		m_arDefLineColours[nCol] = pApp->GetProfileInt("GraphStyles",strKey,(int)m_arDefLineColours[nCol]);
	}
}

void CMonitorGraph::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	//AfxMessageBox("context menu loaded");
	//This block was added by the Pop-up Menu component
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }
	AfxSetResourceHandle(m_hInstance);
    CMenu menu;	
	menu.LoadMenu(IDR_ROLLING_GRAPHS);

	menu.CheckMenuItem(ID_ROLLING_GRAPH,MF_BYCOMMAND|(m_nRollingChecked ? MF_CHECKED:MF_UNCHECKED));
	
    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);
	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);

	AfxSetResourceHandle(AfxGetInstanceHandle());
}

void CMonitorGraph::ToggleRollingGraphs()
{
	if(m_nRollingChecked==0)
	{
		m_nRollingChecked=1;
		m_arrGraphs[0].pChart->SetRollingGraph(20);
	}
	else
	{
		m_nRollingChecked=0;
		KillTimer(IDT_POLL_DATA);
		delete m_arrGraphs[0].pChart;
		m_nFilePos=0;
		m_arrGraphs[0].pChart=CreateDisplay(m_arrGraphs[0].strDesc);
		SetTimer(IDT_POLL_DATA,DEFAULT_REFRESH_RATE,NULL);
	}

}
