/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
/*===========================================================================*/
/*									     */
/*			     **** C O M M O N ****   			     */
/*									     */
/*===========================================================================*/
/*									     */
/*			Module Name: TRM_SUPPORT_UNIX.C			     */
/*			============					     */
/*									     */
/* Description:								     */
/* ============								     */
/*									     */
/*	This module handles/provides terminal support routines for UNIX.     */
/*									     */
/*									     */
/*===========================================================================*/


#include <psl_macros.h>			/* PSL Standard Macros */
#ifndef VMS

/*
 *  Add macros which say whether this platform has colour support.
 */
#if defined(HPV1020) || defined(SUN) || defined(NCR) || defined (AIXV41)
#   define CURSES_COLOUR_SUPPORT 1
#elif defined(HP) || defined(IBM) || defined(OSF) || defined (WIN32) 
/* Colour not available on HP/IBM 3.2.5/OSF/WIN32 */
#else
#error Platform unsupported - \
do not know whether colour support is available			     
#endif

/*
 *  Add macros which say whether this platform has support for alternate
 *  character sets.
 */
#if defined(HPV1020) || defined(SUN) || defined(NCR) || defined(AIXV41) || defined(WIN32)/*42X015*/
#   define ALT_CHRSET_SUPPORT 1
#elif defined(HP) || defined(IBM) || defined(OSF)
/*
 *  Alternate character sets are not supported on HP and IBM 3.2.5
 *  On OSF they appear to be partially supported but don't
 *  actually work!
 *  This may be acertained by looking for acs_string in curses
 */
#else
#error Platform unsupported - \
do not know whether alternate chrset support is available
#endif

/*
 *  Add a macro which is used to determine if this platform supports scrolling
 *  This may be ascertained by looking for the function has_il() (this was
 *  true until OSF came along). On OSF, it has scroll() but none of the other
 *  functions - so in effect it does not support scrolling as far as we are
 *  concerned.
 */
#if defined(HPV1020) || defined(SUN) || defined(NCR) || defined(AIXV41) || defined(WIN32) /*42X015*/
#   define SCROLLING_SUPPORT 1                          
#elif defined(HP) || defined(IBM) || defined(OSF)		
/*  has_il() does not exist on the HP or IBM 3.2.5 */
/*  has_il() does exist in OSF but it is missing [w]scrl() */
#else
#error Platform unsupported - \
do not know whether this platform supports scrolling properly
#endif
/*
 *  Add a macro which is used to determine whether this platform has problems
 *  outputting a character in the last character position on the terminal.
 */
#if defined(SUN) || defined(IBM) || defined(HP) || defined(NCR) || defined(OSF) || defined(WIN32)
#define LAST_CHARACTER_BUG 1
#else
#error Platform unsupported - \
assuming this platform does not have LAST_CHARACTER_BUG               
#endif

/*
 *  Add a macro to say whether this platform supports hline()/vline()
 */
#if defined(HPV1020) || defined(AIXV41) || defined(SUN) || defined(NCR) || defined(WIN32) /*42X015*/
#   define LINE_SUPPORT 1
#elif defined(HP) || defined(IBM) || defined(OSF)
/*
 *  The function hline() and vline() and their 'w' additions are not
 *  supported on the HP/IBM 3.2.5/OSF. These functions allow us to draw
 *  horizontal and vertical lines.
 */
#else
#error Platform unsupported - \
assuming this platform does not have LINE support
#endif

/*
 *  Add a macro to say whether this platform has problems recognising LF and BS
 *  if defined as a function key.
 */
#ifdef WIN32
/* this seems to be OK in the public domain version of NT curses */
#else
#   define LF_BS_AS_FNKEY_NOT_SEEN 1
#endif


/*
 *  Add a macro to say whether this platform supports curs_set()
 */
#if defined(HPV1020) || defined(SUN) || defined(NCR) || defined(AIXV41) || defined(WIN32) /*42X015*/
#   define CURSOR_SET_SUPPORT 1
#elif defined(HP) || defined(IBM) || defined(OSF)
/*
 *  The function curs_set() sets the cursor to off, block, blinking etc
 *  The HP, IBM 3.2.5 and OSF do not have this function. On these platforms we
 *  have to use putp(cursor_type).
 */
#else
#error Platform unsupported - \
assuming this platform does not have CURSOR_SET support
#endif

/*
 *  Add some macros specific to WIN32
 */
#ifdef WIN32
#   define WIN32_TRMTYP "NTConsole"
#endif

/*
 *  Add macro used to decide whether confirmation of the PSL hard key is
 *  required. This causes "ctrl/x" to be displayed at the bottem left of
 *  the screen when ctrl/x is hit.
 */
#   define HARDKEY_CONFIRMATION 1

/*
 *  Macro to define whether this platform supports terminal resizing.
 */
#if defined(SUN) || defined(IBM) || defined(HP) || defined(NCR)
/*  We support terminal resizing but would really prefer to remove it */
#   define RESIZE_SUPPORT 1
#elif defined(OSF) || defined(WIN32)				    
/*  We do not support terminal resizing on OSF and WIN32 */
#else
#error Platform unsupported - \
do not know whether this platform supports terminal resizing
#endif


/*
 *  Macro to define whether we should disable ctrl/z from putting the process
 *  in the background - UNIX only
 */
#ifdef UNIX
#   define     DISABLE_SUSPEND_KEY 1
#endif

/*
 *  If we need to set the TERMINFO environment variable to tell curses
 *  where to find the terminfo database then say so here.
 */
#ifdef UNIX
#   define TERMINFOENV_REQUIRED 1
#elif defined(WIN32)
#else
#error Platform unsupported - \
Dont know whether we need to support the TERMINFO environment variable
#endif

/*
 *  If we need to pick up the TERM environment variable to see what terminal
 *  we are running on then say so here.
 */
#ifdef UNIX
#   define TERMENV_REQUIRED 1
#elif defined(WIN32)
#else
#error Platform unsupported - \
Dont know whether we need to support the TERM environment variable
#endif

/*
 *  Define useful Include structures
 */
#include <curses.h>
#include <stdarg.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#ifdef UNIX
#include <term.h>
#include <termios.h>
#include <unistd.h>
#include <sys/ioctl.h>						      
#include <signal.h>						      
#endif /* UNIX */

/*
 *  Include bits necessary to handle input timeouts.
 */
#ifdef OSF
#include <setjmp.h>
	jmp_buf osf_alarm;
	Boolean osf_tmr_active;
static  char tgetbuff[20];
#   define TGETSTRARG &tgetbuff
#else
#   define TGETSTRARG NULL
#endif

/*
 *  Define Function Prototypes for all external routines used by the module.
 */
#include <psl_rtndef.h>			/* Standard PSL Rtns */
#include <trm_rtndef.h>			/* Terminal Rtns */
#include <scr_rtndef.h>			/* Screen Image Rtns */
#include "trm_intrtndef.h"		/* Internal Terminal Rtns */

/*
 *  The IBM is missing the function prototype for tgetstr().
 */
#ifdef IBM
    char *tgetstr(const char *id, char **area);
#endif /* IBM */

/*
 *  Define Function Prototypes for all internal routines.
 */
#ifdef UNIX
static unsigned long    CHECK_FNKEYS(struct tcb *tcb_ptr);	      
static unsigned long	TRM__TIMEOUT(struct tcb *tcb_ptr,	      
				     long timeout_val);		      
static unsigned long	REMOVE_ENVVAR(struct tcb *tcb_ptr,	      
				      const char *envvar);	      
#endif /* UNIX */
static 		Boolean GOOD_TCOL(char *col_ptr);	

static		void 	TRM__ATTRSET(unsigned long trm_attr);

static		short	CONVERT_FKEY(struct tcb *tcbp,
                                     int localchar);

static unsigned long	PSLCHR_INIT(struct tcb *tcb_ptr);



#ifdef OSF				
static void SIGALRM_HANDLER (int signo);			      
#endif /*OSF*/

/*
 *  Function prototypes for specific platforms only.
 */
#ifndef ALT_CHRSET_SUPPORT                                
static		void	SETUP_HP_ACS_CHARS(void);		      
#endif

/*
 *  Function prototypes for colour support
 */
#ifdef CURSES_COLOUR_SUPPORT
static		void	TRM__COLPR_INIT(struct tcb *tcb_ptr);
#endif /* CURSES_COLOUR_SUPPORT */

/*
 *  Define PSL Structures and macros.
 */
#include <tcbdef.h>			/* Terminal Control Block */
#include <fkydef.h>			/* Function Keys */
#include <shrmsg.h>			/* Message Statuses */
#include <trmdef.h>			/* I/O Subfunctions */
#include <trmtyp.h>			/* Terminal Types */
#include <pslchr.chrset>		/* Character set params */
#include <pslchr.chr>			/* PSL character definitions */
#include <pslchr_prmdef.h>		/* PSL character parameters */
#include <pslerr_prmdef.h>		/* Error trapping symbols */

#ifndef ALT_CHRSET_SUPPORT                                            
/*
 *  Some platforms do not define the ACS_* values. These define the graphics
 *  characters (upper left corner etc) and are assigned from the terminfo
 *  database. In order to ensure that all platforms can run the same code we
 *  must imitate these macros. In the future, if HP/IBM 3.2.5 upgrade their
 *  curses support then these definitions may be removed.
 */
/*
 *  Define a table to contain the translations for the graphics characters.
 */
static unsigned long	acs_map[256];
/*
 *  Define the key characters for the translations in the acsc terminfo
 *  string. The acsc string contains pairs of characters. The first of each
 *  pair represents a particular graphic and the second is the character in the
 *  alternative character set which produces this graphic.
 */
static char acsckey[] = "lmkjutvwqxnos`afg~,+.-hi0";
/*
 *  Define the default translations in the normal character set in case the
 *  terminal does not support a particular graphic or the database does not
 *  contain the string of translations.
 */
static char acscdef[] = "++++++++-|+-_+:'#o<>v^###";
/*
 *  Define the number of translations.
 */
#define ACSC_LENGTH 25
/*
 *  Curses on the sun defines a number of macros for the graphics characters.
 *  We set them up in our code for the HP as they are missing for HPUX 9.0.
 *  Note that we explicitly undefine them first as on OSF the values are
 *  defined even though we don't want to use them.
 */
#undef	ACS_ULCORNER						      
#define	ACS_ULCORNER	acs_map['l']	/* Upper left corner */
#undef	ACS_LLCORNER						      
#define	ACS_LLCORNER	acs_map['m']	/* Lower left corner */
#undef	ACS_URCORNER						      
#define	ACS_URCORNER	acs_map['k']	/* Upper right corner */
#undef	ACS_LRCORNER						      
#define	ACS_LRCORNER	acs_map['j']	/* Lower right corner */
#undef	ACS_RTEE						      
#define	ACS_RTEE	acs_map['u']	/* Right tee */
#undef	ACS_LTEE						      
#define	ACS_LTEE	acs_map['t']	/* Left tee */
#undef	ACS_BTEE						      
#define	ACS_BTEE	acs_map['v']	/* Bottom tee */
#undef	ACS_TTEE						      
#define	ACS_TTEE	acs_map['w']	/* Top tee */
#undef	ACS_HLINE						      
#define	ACS_HLINE	acs_map['q']	/* Horizontal line */
#undef	ACS_VLINE						      
#define	ACS_VLINE	acs_map['x']	/* Vertical line */
#undef	ACS_PLUS						      
#define	ACS_PLUS	acs_map['n']	/* Plus */
#undef	ACS_S1							      
#define	ACS_S1		(acs_map['o'])	/* scan line 1 */
#undef	ACS_S9							      
#define	ACS_S9		(acs_map['s'])	/* scan line 9 */
#undef	ACS_DIAMOND						      
#define	ACS_DIAMOND	(acs_map['`'])	/* diamond */
#undef	ACS_CKBOARD						      
#define	ACS_CKBOARD	(acs_map['a'])	/* checker board (stipple) */
#undef	ACS_DEGREE						      
#define	ACS_DEGREE	(acs_map['f'])	/* degree symbol */
#undef	ACS_PLMINUS						      
#define	ACS_PLMINUS	(acs_map['g'])	/* plus/minus */
#undef	ACS_BULLET						      
#define	ACS_BULLET	(acs_map['~'])	/* bullet */
#undef	ACS_LARROW						      
#define	ACS_LARROW	(acs_map[','])	/* arrow pointing left */
#undef	ACS_RARROW						      
#define	ACS_RARROW	(acs_map['+'])	/* arrow pointing right */
#undef	ACS_DARROW						      
#define	ACS_DARROW	(acs_map['.'])	/* arrow pointing down */
#undef	ACS_UARROW						      
#define	ACS_UARROW	(acs_map['-'])	/* arrow pointing up */
#undef	ACS_BOARD						      
#define	ACS_BOARD	(acs_map['h'])	/* board of squares */
#undef	ACS_LANTERN						      
#define	ACS_LANTERN	(acs_map['i'])	/* lantern symbol */
#undef	ACS_BLOCK						      
#define	ACS_BLOCK	(acs_map['0'])	/* solid square block */
#endif /* No alternate character set support */

/*
 *  Define a table for translation of keys to PSL function keys on the sun.
 *  This table is a CURSES key -> PSL function key translation. Users can
 *  change the key required for a particular PSL function by changing the
 *  function key table in the TERMINFO database for a particular terminal.
 *
 *  e.g. 
 *      The following table links the curses key "KEY_REDO" to the PSL function
 *      DFK_EDIT. In the TERMINFO database entry for a vt220 "KEY_REDO" will
 *      be linked to CTRL/a. If the user prefers CTRL/v for DFK_EDIT he simply
 *      changes the TERMINFO entry, recompiles it with tic and then changes
 *      the vt220.legend file which defines the legends on the keys used for
 *      PSL functions to read "<edit>=<CTRL/V>".
 */
#if defined(HPV1020) || defined(SUN) || defined(NCR) || defined(AIXV41) || defined(WIN32) /*42X015*/
static	 int	    fkytab[2][30][2]=                                 
{
/*
 *  There are in fact two tables. The first is the official set that we
 *  currently document and is referred to above. The second set is a hard-wired
 *  set which may be used on terminals without ANY or ALL the necessary
 *  function keys (this set is useful if you are running a terminal emulator
 *  which may not have the same keys as the terminal being emulated - added
 *  so we could login to other offices machines over the modem using an xterm).
 */
    {
        {KEY_COPY, DFK_DEF},
        {KEY_BACKSPACE, DFK_DELETE},
        {KEY_REDO, DFK_EDIT},
        {KEY_ENTER, DFK_ENTER},
        {KEY_CANCEL, DFK_ERASE},
        {KEY_EXIT, DFK_EXIT},
        {KEY_OPEN, DFK_EXPAND},
        {KEY_SHELP, DFK_FKEYH},
        {KEY_DOWN, DFK_DOWN},
        {KEY_LEFT, DFK_LEFT},
        {KEY_UP, DFK_UP},
        {KEY_RIGHT, DFK_RIGHT},
        {KEY_HELP, DFK_HELP},
        {KEY_IC, DFK_INSERT},
        {KEY_MOVE, DFK_MOVE},
        {KEY_NPAGE, DFK_NXTPAG},
        {KEY_NEXT, DFK_NXTSEQ},
        {KEY_OPTIONS, DFK_OPTION},
        {KEY_PPAGE, DFK_PRVPAG},
        {KEY_PREVIOUS, DFK_PRVSEQ},
        {KEY_PRINT, DFK_PRINT},
        {KEY_END, DFK_QUIT},
        {KEY_REFRESH, DFK_REFRSH},
        {KEY_SELECT, DFK_SELECT},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0}
    },
/*
 *  This second set of translations are the hard-wired keys that must be used
 *  after the PSL_HARD_KEY to obtain the PSL functions.
 */
    {
        {'g', DFK_DEF},		
        {'b', DFK_DELETE},
        {'a', DFK_EDIT},
        {'e', DFK_ENTER},
        {'s', DFK_ERASE},
        {'x', DFK_EXIT},
        {'j', DFK_EXPAND},
        {'k', DFK_FKEYH},
        {'d', DFK_DOWN},
        {'l', DFK_LEFT},
        {'u', DFK_UP},
        {'r', DFK_RIGHT},
        {'h', DFK_HELP},
        {'i', DFK_INSERT},
        {'m', DFK_MOVE},
        {14, DFK_NXTPAG},               /* CTRL/n */
        {'n', DFK_NXTSEQ},
        {'o', DFK_OPTION},
        {16, DFK_PRVPAG},               /* CTRL/p */
        {'p', DFK_PRVSEQ},
        {'f', DFK_PRINT},
        {'q', DFK_QUIT},
        {'w', DFK_REFRSH},
        {'c', DFK_SELECT},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0}
    }
};
#elif !defined(AIXV41) && (defined(HP) || defined(IBM))		      

/*
 *  Define key symbols for HP/IBM which are not defined in curses.h
 */
#define KEY_F30		KEY_F0 + 30
#define KEY_F31		KEY_F0 + 31
#define KEY_F32		KEY_F0 + 32
#define KEY_F33		KEY_F0 + 33
#define KEY_F34		KEY_F0 + 34
#define KEY_F35		KEY_F0 + 35
#define KEY_F36		KEY_F0 + 36
#define KEY_F37		KEY_F0 + 37
#define KEY_F38		KEY_F0 + 38
#define KEY_F39		KEY_F0 + 39
#define KEY_F40		KEY_F0 + 40
#define KEY_F41		KEY_F0 + 41
#define KEY_F42		KEY_F0 + 42
#define KEY_F43		KEY_F0 + 43
#define KEY_F44		KEY_F0 + 44
#define KEY_F45		KEY_F0 + 45

/*
 * +31X149
 *
 *  Define a table for translation of keys to PSL function keys on the HP.
 *  It is desirable for the SUN and HP to use the same translation table but
 *  unfortunately the HP curses implementation is lacking a number of keys
 *  we use on the SUN. We cannot make the SUN use the same table as the HP
 *  because the SUNs version of curses contains unresolved bugs which prevent
 *  KEY_F<n> values from being translated properly.
 *
 *  See comments above for the SUN table.
 */
static	 int	    fkytab[2][30][2]=                                 
{
    {
        {KEY_F30, DFK_DEF},		
        {KEY_BACKSPACE, DFK_DELETE},
        {KEY_F31, DFK_EDIT},
        {KEY_F32, DFK_ENTER},
        {KEY_F33, DFK_ERASE},
        {KEY_F34, DFK_EXIT},
        {KEY_F35, DFK_EXPAND},
        {KEY_F36, DFK_FKEYH},
        {KEY_DOWN, DFK_DOWN},
        {KEY_LEFT, DFK_LEFT},
        {KEY_UP, DFK_UP},
        {KEY_RIGHT, DFK_RIGHT},
        {KEY_F37, DFK_HELP},
        {KEY_IC, DFK_INSERT},
        {KEY_F38, DFK_MOVE},
        {KEY_NPAGE, DFK_NXTPAG},
        {KEY_F39, DFK_NXTSEQ},
        {KEY_F40, DFK_OPTION},
        {KEY_PPAGE, DFK_PRVPAG},
        {KEY_F41, DFK_PRVSEQ},
        {KEY_F42, DFK_PRINT},
        {KEY_F43, DFK_QUIT},
        {KEY_F44, DFK_REFRSH},
        {KEY_F45, DFK_SELECT},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0}
    },
    {
        {'g', DFK_DEF},		
        {'b', DFK_DELETE},
        {'a', DFK_EDIT},
        {'e', DFK_ENTER},
        {'s', DFK_ERASE},
        {'x', DFK_EXIT},
        {'j', DFK_EXPAND},
        {'k', DFK_FKEYH},
        {'d', DFK_DOWN},
        {'l', DFK_LEFT},
        {'u', DFK_UP},
        {'r', DFK_RIGHT},
        {'h', DFK_HELP},
        {'i', DFK_INSERT},
        {'m', DFK_MOVE},
        {14, DFK_NXTPAG},               /* CTRL/n */
        {'n', DFK_NXTSEQ},
        {'o', DFK_OPTION},
        {16, DFK_PRVPAG},               /* CTRL/p */
        {'p', DFK_PRVSEQ},
        {'f', DFK_PRINT},
        {'q', DFK_QUIT},
        {'w', DFK_REFRSH},
        {'c', DFK_SELECT},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0}
    }
      
};
#elif defined(OSF)
/*
 *  Define key symbols for OSF which are not defined in curses.h
 *  Note that keys above 36 do not (currently) work on OSF.
 */
#define KEY_F21		KEY_F0 + 21
#define KEY_F22		KEY_F0 + 22
#define KEY_F23		KEY_F0 + 23
#define KEY_F24		KEY_F0 + 24
#define KEY_F25		KEY_F0 + 25
#define KEY_F26		KEY_F0 + 26
#define KEY_F27		KEY_F0 + 27
#define KEY_F28		KEY_F0 + 28
#define KEY_F29		KEY_F0 + 29
#define KEY_F30		KEY_F0 + 30
#define KEY_F31		KEY_F0 + 31
#define KEY_F32		KEY_F0 + 32
#define KEY_F33		KEY_F0 + 33
#define KEY_F34		KEY_F0 + 34
#define KEY_F35		KEY_F0 + 35
#define KEY_F36		KEY_F0 + 36

/*
 *  Define a table for translation of keys to PSL function keys in OSF.
 *  It is desirable for all platforms to use the same translation table but
 *  unfortunately the HP/OSF curses implementation is lacking a number of keys
 *  we use on the SUN. We cannot make the SUN use the same table as the HP
 *  because the SUNs version of curses contains unresolved bugs which prevent
 *  KEY_F<n> values from being translated properly.
 *
 *  See comments above for the SUN table.
 */
static	 int	    fkytab[2][30][2]=
{
    {
        {KEY_F30, DFK_DEF},		
        {KEY_BACKSPACE, DFK_DELETE},
        {KEY_F31, DFK_EDIT},
        {KEY_F32, DFK_ENTER},
        {KEY_F33, DFK_ERASE},
        {KEY_F34, DFK_EXIT},
        {KEY_F35, DFK_EXPAND},
        {KEY_F36, DFK_FKEYH},
        {KEY_DOWN, DFK_DOWN},
        {KEY_LEFT, DFK_LEFT},
        {KEY_UP, DFK_UP},
        {KEY_RIGHT, DFK_RIGHT},
        {KEY_F21, DFK_HELP},
        {KEY_IC, DFK_INSERT},
        {KEY_F22, DFK_MOVE},
        {KEY_NPAGE, DFK_NXTPAG},
        {KEY_F23, DFK_NXTSEQ},
        {KEY_F24, DFK_OPTION},
        {KEY_PPAGE, DFK_PRVPAG},
        {KEY_F25, DFK_PRVSEQ},
        {KEY_F26, DFK_PRINT},
        {KEY_F27, DFK_QUIT},
        {KEY_F28, DFK_REFRSH},
        {KEY_F29, DFK_SELECT},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0}
    },
    {
        {'g', DFK_DEF},		
        {'b', DFK_DELETE},
        {'a', DFK_EDIT},
        {'e', DFK_ENTER},
        {'s', DFK_ERASE},
        {'x', DFK_EXIT},
        {'j', DFK_EXPAND},
        {'k', DFK_FKEYH},
        {'d', DFK_DOWN},
        {'l', DFK_LEFT},
        {'u', DFK_UP},
        {'r', DFK_RIGHT},
        {'h', DFK_HELP},
        {'i', DFK_INSERT},
        {'m', DFK_MOVE},
        {14, DFK_NXTPAG},               /* CTRL/n */
        {'n', DFK_NXTSEQ},
        {'o', DFK_OPTION},
        {16, DFK_PRVPAG},               /* CTRL/p */
        {'p', DFK_PRVSEQ},
        {'f', DFK_PRINT},
        {'q', DFK_QUIT},
        {'w', DFK_REFRSH},
        {'c', DFK_SELECT},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0},
        {0,0}
    }
};
#else
#error Platform unsupported - \
No function keys set up for platform	                              
#endif

static	 unsigned   long fkymax = 24;	/* # of function keys in above array */
#define   PSL_HARD_KEY 24               /* PSL hard key is CTRL/x */  
#define   PSL_HARD_KEY_TEXT "ctrl/x "   /* PSL hard key text */       
#define	  PSL_HARD_KEY_BLANK "       "	/* string to blank above */   

/*
 *  Macro to make getting colours from colour pairs easier. This is complex
 *  because although the number of colour pairs defined is 64, only 63 are
 *  available for use. Therefore any colour pair number generated is reduced
 *  to less than 64 if it is greater.
 */
#ifdef CURSES_COLOUR_SUPPORT
#define	COLPR_NUM(fcol, bcol) (((fcol)*(bcol))<64) ? ((fcol)+(((bcol)-1)*8)):63
#endif

/*
 *  Static curses structure to hold the state of CTRL/Z processing
 */
#ifdef DISABLE_SUSPEND_KEY
static struct termios orgterm;
#endif

/*
 *  Value of TERMINFO environment variable BEFORE curses is entered.
 */
#ifdef TERMINFOENV_REQUIRED
static	 char	    terminfo_database[256];
#endif

/*
 *  Value of the TERM/PSLTERM environment variable BEFORE curses is entered.
 *  Note
 *  [1] If TRM_INIT is changed to allow the caller to pass a terminal name
 *      string (rather than type) then the following char array should be
 *      updated.
 *  [2] This string may be used by TRM_GETTYP. As TRM_GETTYP may be called
 *      independently of TRM_INIT (which sets terminal_name) then we must
 *      ensure terminal_name is initially set to the null string. This
 *      enables TRM_GETTYP to determine whether to use this string or go and
 *      get PSLTERM/TERM itself.
 */
#ifdef TERMENV_REQUIRED
static	 char	    terminal_name[256]="\0";			      
#endif

/*
 *  Allow us to get at the environ variable.
 */
#ifdef UNIX
extern char **environ;
#endif /* UNIX */

/*
 *  Pointer to screen returned from newterm()
 */
static	 SCREEN	    *screen_ptr;	/* Reference screen to output on */


/************************************************************************/
/*									*/
/*	TRM_BELL - Ring the Terminal Bell				*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_BELL(tcb_ptr);					*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*									*/
/************************************************************************/
unsigned long	TRM_BELL(struct tcb *tcb_ptr)
{
/*
 *  Trace this routine call if required.
 */
    TRM__TRC_BELL(tcb_ptr);
/*
 *  Call the curses routine to perform the bell action for this particular
 *  terminal, if one is not available it will flash the screen. The return
 *  status is ignored as it will always be OK.
 */
    (void)beep();

/*
 *  Exit here with success
 */
    return PSL_NORMAL;
}


/************************************************************************/
/*									*/
/*	TRM_CLEAR - Clear some of the Screen				*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_CLEAR(tcb_ptr, clr_flag, line, column, depth,	*/
/*			    width)					*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   clr_flag = Clear Flags as defined by TRMDEF (Longword)	*/
/*	   line     = Start Line Number (Longword)			*/
/*	   col      = Start Column Number (Longword)			*/
/*	   depth    = Depth (Longword)					*/
/*	   width    = Width (Longword)					*/
/*									*/
/************************************************************************/
unsigned long	TRM_CLEAR(struct tcb *tcb_ptr,
			  unsigned long clear_flags,
			  long line,				     
			  long column,				     
			  long depth,				     
			  long width)				     
{
/*
 *  Local Data
 */
	 short	    tmplin;		/* Line Number */
	 short	    tmpcol;		/* Column Number */
unsigned long	    iret;		/* PSL Routine Status */
	 int	    rtnsts;		/* Curses Routine Status */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 Boolean    param_error;	/* Parameter problem? */
/*
 *  Assume the parameters are OK initially
 */
    iret = PSL_NORMAL;
    param_error = PFALSE;
/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_CLEAR");

/*
 *  Trace this routine if required
 */
    TRM__TRC_CLEAR(tcb_ptr, clear_flags, line, column, depth, width);

/*
 *  If line or column values are negative then adjust them
 */
    if (line < 0) line = tcb_ptr->tcb_maxlinw + 1 + line;
					/* Reset line number */

    if (column < 0) column = tcb_ptr->tcb_maxcolw + 1 + column;
					/* Reset column number */

/*
 *  Check the parameters - these depend on the clear function so we use a
 *  general routine to check the parameters faking those that are not needed
 *  for the clear function.
 */
    switch (clear_flags)
    {
	case TCLR_SCN_ALL:		/* Clear Screen All */
	{
	    line = 1;			/* Fake line */
	    column = 1;			/* Fake column */
	    depth = 1;			/* Fake depth */
	    width = 1;			/* Fake width */
	    break;
	}
	case TCLR_SCN_RST:		/* Clear Rest of Screen */
	case TCLR_LIN_RST:		/* Clear Rest of Line */
	{
	    depth = 1;			/* Fake depth */
	    width = 1;			/* Fake width */
	    break;
	}
	case TCLR_LIN_ALL:		/* Clear Line All */
	{
	    column = 1;			/* Fake column */
	    depth = 1;			/* Fake depth */
	    width = 1;			/* Fake width */
	    break;
	}
	case TCLR_RCT:			/* Clear Rectangle */
	case TCLR_BOX:			/* Clear Box */
	{
	    break;			/* No parameters to fake */
	}
	default:			/* Invalid clear flag */
	{
	    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			       "Invalid value %u for clear_flags",
			       clear_flags);
	    param_error = PTRUE;
	}
    }

/*
 *  If start line is zero or
 *     start line is still negative after adjusting or
 *     depth of area extends off the screen then error.
 */
    if ((line < 0) ||			/* Line negative after adjusting */
	(line == 0) ||
	((line + depth - 1) > tcb_ptr->tcb_maxlinw))		      
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid Line Number %ld for Depth %ld",
			   line,
			   depth);
	param_error = PTRUE;
    }

/*
 *  If start column is zero or
 *     start column is still negative after adjusting or
 *     width of area extends off screen then error
 */
    if ((column < 0) ||			/* Column negative after adjusting*/
	(column == 0) ||
	((column + width - 1) > tcb_ptr->tcb_maxcolw))		    
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid Column Number %ld for Width %ld",
			   column,
			   width);
	param_error = PTRUE;
    }

/*
 *  If there was a parameter problem; report it and exit.
 */
    if (param_error)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_CLEAR", iret, 0);
	goto end;
    }

/*
 *  Now adjust line and column number for curses
 */
    line -= 1;
    column -= 1;

/*
 *  Switch on the Clear Type and do appropriate function
 */
    switch (clear_flags)		/* Case on Clear Flags */
    {
/*
 *	Clear the whole of the screen
 */
	case TCLR_SCN_ALL:		/* Clear Screen All */
	{
/*
 *	Enable alternate character set - we do this here to ensure that
 *	it will be output when a screen refresh is done
 */
#ifdef UNIX
	    (void)putp(reset_1string);	/* Enable alternate character set */
#endif
	    iret = PSL_NORMAL;		/* Assume success */
	    rtnsts = clear();		/* Clear the whole screen */
	    if (rtnsts != OK)
	    {
	    	iret = PSL_CURSES;
	    	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from clear() in TRM_CLEAR");
	    	TRM__SIGNAL_ERR(tcb_ptr, "clear()", iret, 0);
		goto end;
	    }
	    break;			/* and carry on */
	}
/*
 *	Clear the rest of the screen
 */
	case TCLR_SCN_RST:		/* Clear Rest of Screen */
	{
	    iret = PSL_NORMAL;		/* Assume success */
	    rtnsts = move(line, column); /* Move to specified Position */
	    if (rtnsts == ERR)		/* If an Error */
	    {
	    	iret = PSL_CURSES;
	    	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from move(%d, %d) in TRM_CLEAR",
		    line, column);
	    	TRM__SIGNAL_ERR(tcb_ptr, "move()", iret, 0);
		goto end;
	    }
	    iret = PSL_NORMAL;		/* Assume success */
	    rtnsts = clrtobot();	/* And clear rest of Screen */
	    if ((rtnsts != OK) &&	/* If an Error */
		(rtnsts < 0))
	    {
	    	iret = PSL_CURSES;
	    	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from clrtobot() in TRM_CLEAR");
	    	TRM__SIGNAL_ERR(tcb_ptr, "clrtobot()", iret, 0);
		goto end;
	    }
	    break;			/* and carry on */
	}
/*
 *	Clear the whole line
 */
	case TCLR_LIN_ALL:		/* Clear Line All */
	{
	    iret = PSL_NORMAL;		/* Assume success */
	    rtnsts = move(line, 0);	/* Move to start of Specified Line */
	    if (rtnsts == ERR)		/* If an Error */
	    {
	    	iret = PSL_CURSES;
	    	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from move(%d, %d) in TRM_CLEAR",
		    line, column);
	    	TRM__SIGNAL_ERR(tcb_ptr, "move()", iret, 0);
		goto end;
	    }
	    iret = PSL_NORMAL;		/* Assume success */
	    rtnsts = clrtoeol();	/* And clear rest of Line */
	    if ((rtnsts != OK) &&	/* If an Error */
		(rtnsts < 0))
	    {
	    	iret = PSL_CURSES;
	    	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from clrtoeol() in TRM_CLEAR");
	    	TRM__SIGNAL_ERR(tcb_ptr, "clrtoeol()", iret, 0);
		goto end;
	    }
	    break;			/* and carry on */
	}
/*
 *	Clear the rest of the line
 */
	case TCLR_LIN_RST:		/* Clear Rest of Line */
	{
	    iret = PSL_NORMAL;		/* Assume success */
	    rtnsts = move(line, column);/* Move to specified Position */
	    if (rtnsts == ERR)		/* If an Error */
	    {
	    	iret = PSL_CURSES;
	    	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from move(%d, %d) in TRM_CLEAR",
		    line, column);
	    	TRM__SIGNAL_ERR(tcb_ptr, "move()", iret, 0);
		goto end;
	    }
	    iret = PSL_NORMAL;		/* Assume success */
	    rtnsts = clrtoeol();	/* And clear rest of Line */
	    if ((rtnsts != OK) &&	/* If an Error */
		(rtnsts < 0))
	    {
	    	iret = PSL_CURSES;
	    	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from clrtoeol() in TRM_CLEAR");
	    	TRM__SIGNAL_ERR(tcb_ptr, "clrtoeol()", iret, 0);
		goto end;
	    }
	    break;			/* and carry on */
	}

/*
 *	Clear a rectangle
 */
	case TCLR_RCT:			/* Clear Rectangle */
	{
	    for	(tmplin = line;
		 tmplin < (line + depth);			     
		 tmplin++)
	    {				/* For each line in rectangle */
		for (tmpcol = column;
		     tmpcol < (column + width);			      
		     tmpcol++)
		{			/* For each column*/
		    iret = PSL_NORMAL;	/* Assume success */
		    if ((tmplin != 23) && (tmpcol != COLS))
		    {
                        rtnsts = mvaddch(tmplin, tmpcol, ' ');
					/* Insert a Space */
                        if (rtnsts == ERR)	/* If an Error */
                        {
                            iret = PSL_CURSES;
                            PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from mvaddch(%d, %d, ' ') in TRM_CLEAR",
                                               tmplin, tmpcol);
                            TRM__SIGNAL_ERR(tcb_ptr, "clrtoeol()", iret, 0);
                            goto end;
                        }
		    }
		}
	    }
	    break;			/* and carry on */
	}

/*
 *	Clear a box
 */
	case TCLR_BOX:			/* Clear Box */
	{
	    for	(tmplin = line;
		 tmplin < (line + depth);			      
		 tmplin++)
	    {				/* For each line in rectangle */
		for (tmpcol = column;
		     tmpcol < (column + width);			     
		     tmpcol++)
		{			/* For each column*/
		    if ((tmplin == line) ||
					/* If on start line */
			(tmplin == (line + depth - 1)) ||
					/* or on bottom line */
			(tmpcol == column) ||
					/* or on start column */
			(tmpcol == (column + width - 1)))
					/* or on right column */
		    {
			iret = PSL_NORMAL;
					/* Assume success */
			rtnsts = mvaddch(tmplin, tmpcol, ' ');
					/* Insert a Space */
			if (rtnsts == ERR)
			{		/* If an Error */
	    		    iret = PSL_CURSES;
	    		    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    		"Unexpected Error from mvaddch() in TRM_CLEAR");
	    		    TRM__SIGNAL_ERR(tcb_ptr, "mvaddch()", iret, 0);
			    goto end;
			}
		    }
		}
	    }
	    break;			/* and carry on */
	}
	default:			/* Owt Else */
	{
	    iret = PSL_BADPARAM;	/* Bad Parameter */
	    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			       "Invalid value %u for clear_flags",
			       clear_flags);
	    TRM__SIGNAL_ERR(tcb_ptr, "mvaddch()", iret, 0);
	    goto end;
	}
    }

/*
 *  Successful completion so return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

/*
 * END: - We branch here with an error
 *	Exit here with status.
 */
end:
    return iret;			/* Return Status to Caller */
}


/************************************************************************/
/*									*/
/*	TRM_CURSOR - Set Cursor Type					*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_CURSOR(tcb_ptr, type)				*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   type	 = Cursor Type (Byte)					*/
/*									*/
/************************************************************************/
unsigned long	TRM_CURSOR(struct tcb *tcb_ptr,
			   unsigned char cursor_type)
{
/*
 *  Local variables
 */
unsigned long	    iret;		/* Return value variable */
unsigned long	    diag_save;		/* Diagnostic stack context */

    iret = PSL_NORMAL;			/* Assume successful routine */

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_CURSOR");

/*
 *  Trace this routine if required
 */
    TRM__TRC_CURSOR(tcb_ptr, cursor_type);

/*
 *  Perform action to get cursor in particular mode depending on cursor_type
 *  passed
 */
    switch (cursor_type)
    {
	case TCUR_NONE:			/* Invisible cursor */
	{
#ifdef CURSOR_SET_SUPPORT                                           
	    curs_set(0);
#else								    
	    (void)putp(cursor_invisible);	
#endif								    
	    break;
	}
	case TCUR_STATBLK:		/* Static block cursor */
	{
#ifdef CURSOR_SET_SUPPORT              
	    curs_set(1);
#else								   
	    (void)putp(cursor_normal);	
#endif								   
	    break;
	}
	case TCUR_BLNKBLK:		/* Blinking block cursor */
	{
#ifdef CURSOR_SET_SUPPORT             
	    curs_set(1);
#else								  
	    (void)putp(cursor_normal);	
#endif								  
	    break;
	}
	case TCUR_STATUND:		/* Static underscore cursor */
	{
#ifdef CURSOR_SET_SUPPORT           
            curs_set(1);
#else								
	    (void)putp(cursor_normal);	
#endif								
	    break;
	}
	case TCUR_BLNKUND:		/* Blinking underscore cursor */
	{
#ifdef CURSOR_SET_SUPPORT            
	    curs_set(1);
#else								 
	    (void)putp(cursor_normal);	
#endif								 
	    break;
	}

	default:			/* Invalid cursor type parameter */
	{
	    iret = PSL_BADPARAM;	/* Bad Parameter */
	    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			       "Invalid value %u for cursor_type",
			       cursor_type);
	    TRM__SIGNAL_ERR(tcb_ptr, "TRM_CURSOR", iret, 0);
	    goto end;
	}
    }
/*
 *  Successful completion so return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

/*
 * END: - drop to here if we have any problems along the way.
 *  Exit here with status.
 */
end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_DRWBOX - Draw a Box						*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_DRWBOX(tcb_ptr, line, col, depth, width,		*/
/*			     char, colchr, colbck)			*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   line     = Line Number (Longword)				*/
/*	   col      = Column Number (Longword)				*/
/*	   depth    = Depth (Longword)					*/
/*	   width    = Width (Longword)					*/
/*	   char	    = Attributes (Longword):				*/
/*	   colchr   = Character Colour					*/
/*	   colbck   = Character Background Colour			*/
/*									*/
/*      This routine is currently only used from TRM_PANIC.             */
/*                                                                      */
/************************************************************************/
unsigned long	TRM_DRWBOX(struct tcb *tcb_ptr,
			   long line,				     
			   long column,				     
			   long depth,				     
			   long width,				     
			   unsigned long trm_attr,
			   char char_colour,			 
			   char cell_colour)			 
{
/*
 *  Local Data
 */
unsigned long	    iret;		/* PSL Routine Status */
	 int	    rtnsts;		/* Curses Routine Status */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 Boolean    param_error;	/* Parameter problem? */
	 WINDOW	    *subwin_ptr;	/* Sub window ptr */

/*
 *  Assume the parameters are OK initially
 */
    iret = PSL_NORMAL;
    param_error = PFALSE;

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_DRWBOX");

/*
 *  Trace this routine if required
 */
    TRM__TRC_DRWBOX(tcb_ptr, line, column, depth, width,
		    trm_attr, char_colour, cell_colour);

/*
 *  If line or column values are negative then adjust them
 */
    if (line < 0) line = tcb_ptr->tcb_maxlinw + 1 + line;
					/* Reset line number */

    if (column < 0) column = tcb_ptr->tcb_maxcolw + 1 + column;
					/* Reset column number */

/*
 *  Now test the box will fit on the screen.
 *
 *  If start line is zero or
 *     start line is still negative after adjusting or
 *     depth of area extends off the screen then error.
 */
    if ((line < 0) ||			/* Line negative after adjusting */
	(line == 0) ||
	((line + depth - 1) > tcb_ptr->tcb_maxlinw))		    
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid Line Number %ld for Depth %ld",
			   line,
			   depth);
	param_error = PTRUE;
    }

/*
 *  If start column is zero or
 *     start column is still negative after adjusting or
 *     width of area extends off screen then error
 */
    if ((column < 0) ||			/* Column negative after adjusting*/
	(column == 0) ||
	((column + width- 1) > tcb_ptr->tcb_maxcolw))		      
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid Column Number %ld for Width %ld",
			   column,
			   width);
	param_error = PTRUE;
	goto end;
    }

/*
 *  Check the character and cell colours are valid
 */
    if (((tcb_ptr->tcb_trmchal & TCCH_COL) != 0) &&
	((!GOOD_TCOL(&char_colour)) ||
	 (!GOOD_TCOL(&cell_colour))))
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid colour value specified");
	param_error=PTRUE;
    }

/*
 *  If there was a parameter problem; report it and exit.
 */
    if (param_error)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_DRWBOX", iret, 0);
	goto end;
    }

/*
 *  Now adjust line and column number for curses
 */
    line -= 1;
    column -= 1;

/*
 *  Call routine to set up the attributes as required.
 */
    TRM__ATTRSET(trm_attr);

/*
 *  Set the foreground and background colour to as we wish
 */
#ifdef CURSES_COLOUR_SUPPORT
    if ((tcb_ptr->tcb_trmchal & TCCH_COL) != 0)
    {
	(void)attron(COLOR_PAIR(COLPR_NUM(char_colour,cell_colour)));
    }
#endif								    

/*
 *  Create a curses sub-window within the screen and draw a box around it.
 */
    subwin_ptr = subwin(stdscr, depth, width, line, column);
    if (subwin_ptr == NULL)		/* Error if NULL pointer returned */
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "subwin()", iret, 0);
	goto end;
    }

    (void)box(subwin_ptr, ACS_VLINE, ACS_HLINE);		    
					/* Use default graphics segments */

    rtnsts = touchwin(stdscr);		/* Show window needs updating */
    if (rtnsts == ERR)			/* If error */
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "touchwin()", iret, 0);
	goto end;
    }

    rtnsts = delwin(subwin_ptr);	/* Delete sub-window */
    if (rtnsts == ERR)			/* If error */
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "delwin()", iret, 0);
	goto end;
    }

/*
 *  Reset the attributes
 *  Switch off all attributes except the character set off
 */
    (void)attroff(A_STANDOUT | A_UNDERLINE | A_REVERSE |
		  A_BLINK | A_DIM | A_BOLD); /* Switch off all attributes */

/*
 *  Successful completion so return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

/*
 * END: - drop to here if we have any problems along the way.
 *  Exit here with status.
 */
end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_DRWLIN - Draw a Line					*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_DRWLIN(tcb_ptr, line, col, length, direct,	*/
/*			     char, colchr, colbck)			*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   line     = Line Number (Longword)				*/
/*	   col      = Column Number (Longword)				*/
/*	   length   = Length (Longword)					*/
/*	   direct   = Direction						*/
/*	   char	    = Attributes (Longword)				*/
/*	   colchr   = Character Colour					*/
/*	   colbck   = Character Background Colour			*/
/*									*/
/*      This routine is not currently used in UNIX but as it has been   */
/*      written there seems little point in removing it.                */
/*									*/
/************************************************************************/
unsigned long TRM_DRWLIN (struct tcb *tcb_ptr,
			  long line,				     
			  long column,				     
			  long length,				     
			  unsigned char direction,
			  unsigned long trm_attr,
			  char char_colour,			     
			  char cell_colour)			     
{
/*
 *  Local Data
 */
unsigned long	    iret;		/* PSL Routine Status */
	 int	    rtnsts;		/* Curses Routine Status */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 Boolean    param_error;	/* Parameter problem? */
#ifndef LINE_SUPPORT                                 
	 short	    isub;		/* Loop variable */	     
#endif
/*
 *  Assume the parameters are OK initially
 */
    iret = PSL_NORMAL;
    param_error = PFALSE;

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_DRWLIN");

/*
 *  Trace this routine if required
 */
    TRM__TRC_DRWLIN(tcb_ptr, line, column, length, direction,
		    trm_attr, char_colour, cell_colour);

/*
 *  If line or column values are negative then adjust them
 */
    if (line < 0) line = tcb_ptr->tcb_maxlinw + 1 + line;
					/* Reset line number */

    if (column < 0) column = tcb_ptr->tcb_maxcolw + 1 + column;
					/* Reset column number */

/*
 *  Have we got a valid direction
 */
    if ((direction != TGPH_HORIZ) || (direction != TGPH_HORIZ))
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid value for direction %d",
			   direction);
	param_error = PTRUE;
    }

/*
 *  Now test the line will fit on the screen.
 *
 *  If start line is zero or
 *     start line is still negative after adjusting or
 *     length of vertical line extends off the screen then error.
 */
    if ((line < 0) ||			/* Line negative after adjusting */
	(line == 0) ||
	(((line + length - 1) > tcb_ptr->tcb_maxlinw) &&	     
	 (direction == TGPH_VERT)))
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid Line Number %ld for length %ld",
			   line,
			   length);
	param_error = PTRUE;
    }

/*
 *  If start column is zero or
 *     start column is still negative after adjusting or
 *     length of horizontal line extends off screen then error
 */
    if ((column < 0) ||			/* Column negative after adjusting*/
	(column == 0) ||
	(((column + length - 1) > tcb_ptr->tcb_maxcolw) &&	     
	 (direction == TGPH_HORIZ)))
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid Column Number %ld for length %ld",
			   column,
			   length);
	param_error = PTRUE;
	goto end;
    }

/*
 *  Check the character and cell colours are valid
 */
    if (((tcb_ptr->tcb_trmchal & TCCH_COL) != 0) &&
	((!GOOD_TCOL(&char_colour)) ||
	 (!GOOD_TCOL(&cell_colour))))
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid colour value specified");
	param_error=PTRUE;
    }

/*
 *  If there was a parameter problem; report it and exit.
 */
    if (param_error)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_DRWLIN", iret, 0);
	goto end;
    }

/*
 *  Now adjust line and column number for curses
 */
    line -= 1;
    column -= 1;

/*
 *  Move to the cursor position the caller wants the line to start
 */
    rtnsts = move(line, column);
    if (rtnsts == ERR)
    {
	iret = PSL_CURSES;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Error from curses move(%ld,%ld)",
			   line,
			   column);
	TRM__SIGNAL_ERR(tcb_ptr, "move()", iret, 0);
	goto end;
    }

/*
 *  Call routine to set up the attributes as required.
 */
    TRM__ATTRSET(trm_attr);

/*
 *  Set the foreground and background colour to as we wish
 */
#ifdef CURSES_COLOUR_SUPPORT
    if ((tcb_ptr->tcb_trmchal & TCCH_COL) != 0)
    {
	(void)attron(COLOR_PAIR(COLPR_NUM(char_colour,cell_colour)));
    }
#endif								   

/*
 *  Draw either a horizontal or vertical line as caller specified, the return
 *  status's are ignored as they are not useful.
 */
    if (direction == TGPH_HORIZ)	/* Horizontal line */
    {
#ifdef LINE_SUPPORT                                          
/*
 *	On the sun/ncr we can draw a horizontal line with one call to hline().
 */
	(void)hline(ACS_HLINE, length);

/*
 *	On the hp a horizontal line must be drawn manually.
 */

#else /* LINE_SUPPORT not defined */                    

	for (isub = 0; isub < length; isub++)			    
	{
	    rtnsts = mvaddch(line, (column + isub), ACS_HLINE);
					/* Insert horizontal line chr */
	    if (rtnsts == ERR)		/* If an Error */
	    {
		iret = PSL_CURSES;
		PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    "Unexpected Error from mvaddch(%d, %d, ACS_HLINE) in TRM_DRWLIN\n",
		    line, (column + isub));
		TRM__SIGNAL_ERR(tcb_ptr, "mvaddch()", iret, 0);
		goto end;
	    }
	}
#endif


    }
    else				/* Vertical line */
    {
/*
 *	On the sun we can draw a vertical line with one call to vline().
 */
#ifdef LINE_SUPPORT                                      

	(void)vline(ACS_VLINE, length);
/*

 *	On the hp a vertical line must be drawn manually.
 */

#else /* LINE_SUPPORT not defined */                     

	for (isub = 0; isub < length; isub++)			     
	{
	    rtnsts = mvaddch((line + isub), column, ACS_VLINE);
					/* Insert horizontal line chr */
	    if (rtnsts == ERR)		/* If an Error */
	    {
		iret = PSL_CURSES;
		PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    "Unexpected Error from mvaddch(%d, %d, ACS_VLINE) in TRM_DRWLIN\n",
		    (line + isub), column);
		TRM__SIGNAL_ERR(tcb_ptr, "mvaddch()", iret, 0);
		goto end;
	    }
	}
#endif
    }

    rtnsts = touchwin(stdscr);		/* Show window needs updating */
    if (rtnsts == ERR)			/* If error */
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "touchwin()", iret, 0);
	goto end;
    }

/*
 *  Reset the attributes
 *  Switch off all attributes except the character set off
 */
    (void)attroff(A_STANDOUT | A_UNDERLINE | A_REVERSE |
		  A_BLINK | A_DIM | A_BOLD); /* Switch off all attributes */

/*
 *  Successful completion so return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

/*
 * END: - drop to here if we have any problems along the way.
 *  Exit here with status.
 */
end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_FINISH - Tidy up the Terminal				*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_FINISH(tcb_ptr)					*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*									*/
/************************************************************************/
unsigned long	TRM_FINISH(struct tcb *tcb_ptr)

{
/*
 *  Local data
 */
unsigned long	    iret;		/* Return value of function */
struct	 diagstk    *diag_ptr;		/* Diagnostic pointer */
unsigned long	    diag_save;		/* Saved diagnostic context */
	 jmp_buf    jmp_env;		/* setjmp environment */
	 int	    rtnsts;		/* Curses Routine Status */
unsigned long	    err_ctx;		/* Error context */

/*
 *  Assume success
 */
    iret = PSL_NORMAL;

/*
 *  Get diagnostic pointer from the TCB and add a diagnostic message to the
 *  stack to say what we're doing.
 */
    diag_ptr = tcb_ptr->tcb_errstkps;	/* Set up diagnostic pointer */
    tcb_ptr->tcb_errstkps = NULL;	/* Clear in TCB to avoid confusion */
    diag_save = PSL_ADD_STATDIAG(diag_ptr, "During TRM_FINISH");

/*
 *  Trace the call if the flag requests this
 */
    TRM__TRC_FINISH(tcb_ptr);

/*
 *  Clear the screen as it will not always be done by the curses endwin.
 */
    (void)wclear(stdscr);
    rtnsts = wrefresh(stdscr);
#ifndef HPV1020                                                     
    if (rtnsts == ERR)
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "refresh()", iret, 0);
	goto end;
    }
#endif                                                     
    
/*
 *  Ensure the cursor is visible
 */
    (void)TRM_CURSOR(tcb_ptr, TCUR_BLNKBLK);

/*
 *  Ensure the terminal is in it's normal character set
 */
    (void)TRM_RESET_CHRSET(tcb_ptr, CURSES_CHRSET);

/*
 *  Call the curses routine that tidies everything up
 */
    rtnsts = endwin();				/* Reset terminal to normal */
    if (rtnsts == ERR)
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "endwin()", iret, 0);
	goto end;
    }

/*
 *  Delete memory allocated by curses for the screen
 */
#if defined(SUN) || defined(NCR)      				    
    delscreen(screen_ptr);
#endif								   

/*
 *  Restore the state of CTRL/Z
 *
 *  Taken from SUNSOLVE 2.1 Systems & Resolutions Data Base (SRDB) 2323
 *  (see TRM_INIT)
 */
#ifdef DISABLE_SUSPEND_KEY
    (void) tcsetattr(0, TCSANOW, &orgterm);		
#endif

/*
 *  Flag that we want to trap signalled errors
 */
    if (PSL_SETJMP(jmp_env) != 0)
    {					/* Set trap and resignal after fired */
	iret = PSL_RESIGNAL_ERR(diag_ptr);
	goto end;			/* Exit with status if we return */
    }

    err_ctx = PSL_TRAP_ERRORS(
		    diag_ptr,		/* Trap errors via this diag ptr */
		    0,			/* All classes */
		    0,			/* All numbers */
		    PSLERRACT_JUMP,	/* perform jump */
		    JMP_ADDR(jmp_env),	/* with above environment */
		    NULL);		/* No error handling routine */

/*
 *  Deallocate any screen images that may have been used.
 */
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_scrimgps);
					/* Live screen image ptr */
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_scrwrkps);
					/* Work screen image ptr */
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_errimgps),
					/* Error line screen image ptr */
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_msgimgps);
					/* Message line screen image ptr */
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_brdimgps);
					/* Broadcast line screen image ptr */
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_ahlimgps);
					/* Autohelp line screen image ptr */

/*
 *  Remove our error trap
 */
    PSL_FLUSH_ERRORS(diag_ptr, err_ctx);

/*
 *  Clear the structure type name.
 */
    (void)memcpy(tcb_ptr->tcb_strtypc4, "    ", 4);
					/* Clear Type, used for validation */
/*
 *  Remove the TCB address from the Cardfile.
 */
    PSL_CARDFILE_REMOVE ("TCB --> ");	/* Remove TCB pointer */

/*
 *
 *  On the HP curses sometimes leaves the terminal in alternate character set
 *  so we reset it here.
 */
#ifndef ALT_CHRSET_SUPPORT                                            
	(void)putp(exit_alt_charset_mode);
#endif

/*
 *  Remove diagnostic message here when we have done everything correctly
 */
    PSL_REMOVE_DIAGNOSTICS(diag_ptr, diag_save);

/*
 * END: - drop to here if we have any problems along the way.
 *  Exit here with status.
 */
end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_INIT - Initialize for Terminal				*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_INIT(tcb_ptr, trm_type, trm_flag, diag_ptr)	*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   trm_typ  = Terminal Type.  Passed by Value.			*/
/*		    = 0 if TCB is to be set up for listing purposes	*/
/*	   trm_flag = Options Flag. Passed by Value.			*/
/*		    = TFLG_ASZ if Terminal is to be 'auto-sized'	*/
/*		    = TFLG_OSC to use "OLD" screen images		*/
/*		    = TFLG_CHAN if channel already assigned to terminal	*/
/*	   diag_ptr = Pointer to Diagnostics Stack.  Passed by Value.	*/
/*									*/
/*									*/
/************************************************************************/
unsigned long	TRM_INIT(struct tcb *tcb_ptr,
			 unsigned char trmtyp,
			 unsigned long trmflags,
			 struct diagstk *diag_ptr)
{
volatile unsigned long  iret = PSL_NORMAL;      /* Return value of function */
                                                     
unsigned long	        diag_save;              /* Saved diagnostic context */
jmp_buf                 jmp_env;		/* setjmp environment */
char	                *trmtyp_txt;            /* String for the trm type*/
int	                rtnsts;                 /* Curses Routine Status */
char	                buffer[512];            /* Useful character buffer */
unsigned long	        err_ctx;		/* Error context */
char	                *chr_ptr;		/* Useful character pointer */
#ifdef TERMINFOENV_REQUIRED
static	 char	        newterminfoenv[512];    /* New terminfo envvar */
char	                pslterminfo[512];	/* PSL terminfo location */
#endif
#if defined(TERMINFOENV_REQUIRED) || defined(TERMENV_REQUIRED)
char	                trmdir[2];		/* Trm terminfo sub-dir */
#endif
#ifdef DISABLE_SUSPEND_KEY
struct termios          anewterm;		/* New termios structure */
#endif

/*
 *  Add diagnostic to the stack to say what we're doing.
 */
    diag_save = PSL_ADD_STATDIAG(diag_ptr, "During TRM_INIT");

/*
 *  Set up the diagnostics pointer and the structure type in the TCB before they
 *  may be required.
 */
    (void)memcpy(tcb_ptr->tcb_strtypc4, "TCB ", 4);
					/* Type used for validation */
    tcb_ptr->tcb_errstkps = diag_ptr;	/* The diagnostic pointer */

/*
 *  Store the TCB address in the Cardfile.
 */
    PSL_CARDFILE_ADD (tcb_ptr, "TCB --> ");
					/* Add TCB pointer */

/*
 *  If we want to trace then we will set the tracing flag in the TCB and trace
 *  the routine call
 */
    TRM__TRC_INIT(tcb_ptr, trmtyp, trmflags, diag_ptr);

    tcb_ptr->tcb_mscflgb |= TCMS_TRC;

/*
 *  The options flag may only validly contain TFLG_ASZ in UNIX.
 */
    if ((trmflags & (~TFLG_ASZ)) != 0)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_INIT", iret, "Invalid options flag");
	goto end;
    }

/*
 *  We set the terminal type up to be the specific CURSES type then the
 *  character sets will be simply standard curses or alternate curses.
 */
    tcb_ptr->tcb_trmtypb = TRTY_CURSES;			

/*
 *  We need to set the TERMINFO environment variable to pick up our terminal
 *  database. Note that because the user may want to exit curses for the
 *  equivalent of a spawn we must save the current value of TERMINFO if it
 *  exists.
 */
#ifdef TERMINFOENV_REQUIRED
    chr_ptr = getenv("TERMINFO");
    if (chr_ptr == NULL)                        /* TERMINFO not defined */
    {
	terminfo_database[0] = '\0';            /* ... nothing to save */
    }
    else                                        /* TERMINFO defined */
    {
	(void)strcpy(terminfo_database, chr_ptr); /* ... save it */
    }

/*
 *  We need to set the environment variable 'TERMINFO' to ensure the terminal
 *  definitions picked up are our own. Obtain the location of our terminfo
 *  database.
 */

    PSL_ADDROOT ("/lib/terminfo", pslterminfo,	
                 sizeof (pslterminfo));			
    
/*
 *  Now set TERMINFO to point to our database.
 */
    (void)strcpy(newterminfoenv, "TERMINFO=");		
    (void)strcat(newterminfoenv, pslterminfo);		
    rtnsts = putenv(newterminfoenv);				
    if (rtnsts != 0)
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "putenv()", iret, NULL);
	goto end;
    }
#endif /* TERMINFOENV_REQUIRED */

/*
 *+ 31X133
 *
 *  We set the terminal type string to the value of the environment variable
 *  PSLTERM (if defined) else TERM.
 *
 *  N.B.
 *
 *  [1] for UNIX the terminal type passed in the parameter list is irrelevant.
 *  [2] we do not care at this stage if TERM or PSLTERM is not defined as it
 *      will fall out as PSL_MISSTRM in the next section of code.
 */
#ifdef TERMENV_REQUIRED
    trmtyp_txt = getenv("PSLTERM");		
    if (trmtyp_txt == NULL)				
    {								    
        trmtyp_txt = getenv("TERM");
    }								    

    if (trmtyp_txt == NULL)
    {
	iret = PSL_MISSTRM;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, trmtyp_txt);
	PSL_SIGNAL_ERROR(tcb_ptr->tcb_errstkps, PSLERRCLS_TRM, iret);
	goto end;
    }
#endif /* TERMENV_REQUIRED */
    
/*
 *  Ensure the terminal in the users TERM environment variable is in the
 *  PSL terminfo database. We do this because curses will search our database
 *  first and then the standard one. We do not want to continue if this
 *  terminal is not in our database.
 *
 *  The name of this terminal entry is:
 *  /usr/psl_root/common_root/terminfo/<l>/<terminal>
 *  where <l> is the first letter of the terminal.
 */
#if defined(TERMINFOENV_REQUIRED) && defined(TERMENV_REQUIRED)
    (void)strcpy(buffer, pslterminfo);          /* Start with DB location */
    (void)strcat(buffer, "/");
    trmdir[0] = trmtyp_txt[0];                  /* Get first letter of trm */
    trmdir[1] = '\0';
    (void)strcat(buffer, trmdir);               /* Add first letter of trm */
    (void)strcat(buffer, "/");
    (void)strcat(buffer, trmtyp_txt);           /* Add terminal name */
    rtnsts = access(buffer, F_OK);              /* Does it exist? */
    if (rtnsts != 0)
    {
	iret = PSL_MISSTRM;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, trmtyp_txt);
	PSL_SIGNAL_ERROR(tcb_ptr->tcb_errstkps, PSLERRCLS_TRM, iret);
	goto end;
    }

/*
 *  Now that we know the terminal exists in our TERMINFO database we can
 *  save its name for TRM_GETTYP.
 */
    if (strlen(trmtyp_txt) < sizeof(terminal_name))
    {
	(void)strcpy(terminal_name, trmtyp_txt);
    }
    else
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Terminal name too big for buffer");
	PSL_SIGNAL_ERROR(tcb_ptr->tcb_errstkps, PSLERRCLS_TRM, PSL_BADPARAM);
	goto end;
    }
#endif /* TERMINFOENV_REQUIRED && TERMENV_REQUIRED */
    
/*
 *  Disable CTRL/Z and save the previous state so we can restore it in
 *  TRM_FINISH.
 *
 *  Taken from SUNSOLVE 2.1 Systems & Resolutions Data Base (SRDB) 2323
 */
#ifdef DISABLE_SUSPEND_KEY
    (void) tcgetattr(0, &orgterm);				
    anewterm = orgterm;
    anewterm.c_cc[VSUSP] = _POSIX_VDISABLE;		
    (void) tcsetattr(0, TCSANOW, &anewterm);	
#endif
    
/*
 *  Here we call the curses routine newterm to initialize the curses structures
 *  and prepare the terminal, we use this as opposed to initscr because we are
 *  interested in the error status, and may wish to specify a terminal type.
 */
#ifdef UNIX
    screen_ptr = newterm(trmtyp_txt,	/* String of terminal type */
			 stdout,	/* Output file always stdout */
			 stdin);	/* Input file always stdin */
    if (screen_ptr == NULL)		/* If no screen returned, error */
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_INIT", iret, "Invalid terminal type");
	goto end;
    }

#elif defined(WIN32)                                            
    trmtyp_txt = WIN32_TRMTYP;                                  
    initscr();
#endif

/*
 *  If the this terminal is too small then throw it out.
 */
    if ((COLS < 80) || (LINES < 24))
    {
	iret = PSL_TRMTOOSMALL;					     
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret);	
	PSL_SIGNAL_ERROR(tcb_ptr->tcb_errstkps, PSLERRCLS_TRM, iret); 
	goto end;
    }

#ifdef UNIX
/*
 *  Check that all the necessary function keys are defined.
 */
    iret = CHECK_FNKEYS(tcb_ptr);
    if (!(psl_test_status(iret)))
    {
	endwin();
	PSL_SIGNAL_ERROR(tcb_ptr->tcb_errstkps, PSLERRCLS_TRM, iret);
	goto end;
    }
	
/*
 *  On the HP ensure that the correct graphics character-sets are initialized.
 */
#ifndef ALT_CHRSET_SUPPORT                                            
    (void)putp(reset_1string);
#endif
#endif /* UNIX */

/*
 *  Set up various features of curses to be how we desire them. If return
 *  values are ignored then they will always be OK.
 */
    rtnsts = cbreak();			/* Pass a character at a time */
    if (rtnsts == ERR)
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "cbreak()", iret, NULL);
	goto end;
    }

/*
 *  Do not flush I/O queues if an interrupt key is received.
 *  This prevents the situation where curses loses track of the terminal status.
 */
    rtnsts = intrflush(stdscr, FALSE);		/* Don't flush interrupts */
    if (rtnsts == ERR)
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "intrflush()", iret, NULL);
	goto end;
    }

/*
 *  Enable keypad input.
 */
    rtnsts = keypad(stdscr, TRUE);
    if (rtnsts == ERR)
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "keypad()", iret, NULL);
	goto end;
    }

#ifdef CURSES_COLOUR_SUPPORT
    if (has_colors())			/* If colour supported */
    {
	rtnsts = start_color();		/* Make colour capabilities available */
	if (rtnsts == ERR)
	{
	    iret = PSL_CURSES;
	    TRM__SIGNAL_ERR(tcb_ptr, "start_color()", iret, NULL);
	    goto end;
	}
    }
#endif /* CURSES_COLOUR_SUPPORT */                           

/*
 *  refresh() should not affect cursor position.
 */
    (void)leaveok(stdscr, FALSE);

/*
 *  Do not translate newline into carriage return and linefeed on output
 *  and return to newline on input.
 */
    (void)nonl();

/*
 *  Disable scrolling on the right or bottom of the terminal.
 */
    (void)scrollok(stdscr, FALSE);

/*
 *  Initialize curses character sets etc.
 */
    iret = PSLCHR_INIT(tcb_ptr);
    if (!psl_test_status(iret)) goto end;
    tcb_ptr->tcb_chrsetb = CURSES_CHRSET;/* The initial character set */
/*
 *  The bits that are useful for UNIX can now be set up in the TCB.
 */
    tcb_ptr->tcb_trmflgw = 0;		/* Initialize routines flag */
    tcb_ptr->tcb_maxlinw = LINES;	/* Number lines in terminal */
    tcb_ptr->tcb_maxcolw = 80;		/* Number of columns in terminal */
    tcb_ptr->tcb_curlinb = 0;		/* The current line number */
    tcb_ptr->tcb_curcolb = 0;		/* The current column number */
    tcb_ptr->tcb_inplinb = 0;		/* The input line number */
    tcb_ptr->tcb_inpcolb = 0;		/* The input column number */
    tcb_ptr->tcb_fkyflgb &= ~(TCFK_RUN | TCFK_EDI);
					/* Show use development mode keys */
    tcb_ptr->tcb_scncolb = TCOL_BLACK;	/* Assume black screen */
    tcb_ptr->tcb_colchrb = TCOL_WHITE;	/* Assume white character */
    tcb_ptr->tcb_colbckb = TCOL_BLACK;	/* ... with black background */

/*
 *  Set up default help, error and broadcast lines, colours, etc.
 */
    tcb_ptr->tcb_brdlinw = tcb_ptr->tcb_maxlinw;
					/* Broadcast on the bottom line */
    tcb_ptr->tcb_brdattw = TDEF_BRD_TRMATTW;	/* Default attributes */
    tcb_ptr->tcb_brdchrb = TDEF_BRD_COLCHRB;	/* Default character colour */
    tcb_ptr->tcb_brdbckb = TDEF_BRD_COLBCKB;	/* Default background colour */
    tcb_ptr->tcb_brckadl = 0;					  
						/* No broadcast check routine */

    tcb_ptr->tcb_msglinw = tcb_ptr->tcb_maxlinw;
					/* Help on the bottom line */
    tcb_ptr->tcb_hlpattw = TDEF_HLP_TRMATTW;	/* Default attributes */
    tcb_ptr->tcb_hlpchrb = TDEF_HLP_COLCHRB;	/* Default character colour */
    tcb_ptr->tcb_hlpbckb = TDEF_HLP_COLBCKB;	/* Default background colour */

    tcb_ptr->tcb_errlinw = tcb_ptr->tcb_maxlinw;
					/* Errors on the bottom line */
    tcb_ptr->tcb_errattw = TDEF_ERR_TRMATTW;	/* Default attributes */
    tcb_ptr->tcb_errchrb = TDEF_ERR_COLCHRB;	/* Default character colour */
    tcb_ptr->tcb_errbckb = TDEF_ERR_COLBCKB;	/* Default background colour */

/*
 *  Flag that we want to trap signalled errors for calls to SCR routines
 */
    if (PSL_SETJMP(jmp_env) != 0)	/* Set trap and resignal after fired */
    {
        iret = PSL_RESIGNAL_ERR(diag_ptr);
	goto end;
    }

    err_ctx = PSL_TRAP_ERRORS(
		    diag_ptr,		/* Trap errors via this diag ptr */
		    0,			/* All classes */
		    0,			/* All numbers */
		    PSLERRACT_JUMP,	/* perform jump */
		    JMP_ADDR(jmp_env),	/* with above environment */
		    NULL);		/* No error handling routine */

/*
 *  Screen images that may be needed, allocate and if needed connect.
 */
/*
 *  Live screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_scrimgps),
		       tcb_ptr->tcb_maxlinw,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Connect live image
 */
    (void)SCR_CONNECT(tcb_ptr->tcb_scrimgps, tcb_ptr, 0, 0);

/*
 *  Work screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_scrwrkps),
		       tcb_ptr->tcb_maxlinw,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Connect work image
 */
    (void)SCR_CONNECT(tcb_ptr->tcb_scrwrkps, tcb_ptr, 0, 0);
/*
 *  Error line screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_errimgps),
		       1,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Message line screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_msgimgps),
		       1,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Broadcast line screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_brdimgps),
		       1,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Autohelp line screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_ahlimgps),
		       1,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Initialize the screen image list headers
 */
    tcb_ptr->tcb_scrlstps2[0] = NULL;
    tcb_ptr->tcb_scrlstps2[1] = NULL;

/*
 *  Remove our error trap
 */
    PSL_FLUSH_ERRORS(diag_ptr, err_ctx);

/*
 *  Work out and set up the terminal characteristics
 */
    tcb_ptr->tcb_trmchal = 0;		/* Initialize characteristics */
#ifdef CURSES_COLOUR_SUPPORT
    if (has_colors())			/* If colour supported */
    {
	tcb_ptr->tcb_trmchal |= TCCH_COL;	/* set flag */
	TRM__COLPR_INIT(tcb_ptr);	/* Initialize colour pairs */
    }
#endif /* CURSES_COLOUR_SUPPORT */   

#ifdef RESIZE_SUPPORT						      
/*
 *  Does the terminal have a 132 column mode.
 */
    if ((init_1string[0] != '\0') && (init_2string[0] != '\0'))	 
    {
	tcb_ptr->tcb_trmchal |= TCCH_132;
    }
#endif								      

/*
 *  Does the terminal have attributes.
 */
#ifdef UNIX
    if (tgetstr("me", TGETSTRARG))
    {
	tcb_ptr->tcb_trmchal |= TCCH_ATT;
    }
#endif
    
/*
 *  HP does not support scrolling.
 */
#ifdef SCROLLING_SUPPORT
    if (has_il() != ERR)		/* If insert line */
    {
	tcb_ptr->tcb_trmchal |= TCCH_SCR;
					/* Set scroll flag */
    }
#endif /* SCROLLING_SUPPORT */

    tcb_ptr->tcb_trmattb = 0;		/* Initialize attributes */

    tcb_ptr->tcb_trmstsb = 0;		/* Initialize terminal status */

/*
 *  Miscellaneous flags and values that are probably not useful under UNIX
 *  but we set up to be some sort of meaningful value.
 */
    tcb_ptr->tcb_chanw = 0;		/* No channel to terminal */
    tcb_ptr->tcb_outchrw = 0;		/* Chars in output buffer */
    tcb_ptr->tcb_gphtbll = NULL;	/* No graphics table */	     
    tcb_ptr->tcb_scnaddl = NULL;	/* No screen image buffer */ 
    tcb_ptr->tcb_fntnumb = 0;		/* Fount number */

/*
 *  Remove diagnostic message here when we have done everything correctly
 */
    PSL_REMOVE_DIAGNOSTICS(diag_ptr, diag_save);

/*
 * END:
 *  Drop to here if we have a problem, return to caller with the error status.
 */
end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_INPUT - Solicit Input Text from User			*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_INPUT(tcb_ptr, inpbuf_ptr, maxchars, inplen_ptr,	*/
/*			    inpfkey_ptr, flags, line, column, fill,	*/
/*			    timeout, pmtdsc, trmatt, chrcol, bckcol)	*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr     = Pointer to Terminal Control Block		*/
/*	   inpbuf_ptr  = Pointer to Input Buffer			*/
/*	   bufsiz      = Size of Input Buffer				*/
/*	   maxchars    = Maximum Characters Allowed			*/
/*	   inplen_ptr  = Pointer to returned Input Character Count	*/
/*	   inpfkey_ptr = Returned Input Function Key Number		*/
/*	   flags       = Input Attributes				*/
/*	   line        = Line Number					*/
/*	   column      = Column Number					*/
/*	   fill	       = Filler Character				*/
/*	   timeout     = Timeout in Seconds				*/
/*	   pmtbuf_ptr  = Prompt String (null-terminated)		*/
/*	   trmatt      = Terminal Attributes for Input			*/
/*	   chrcol      = Foreground Colour				*/
/*	   bckcol      = Background Colour				*/
/*									*/
/************************************************************************/
unsigned long	TRM_INPUT (struct tcb *tcb_ptr,
			   char *inpbuf_ptr,
			   long bufsiz,				   
			   long maxchars,			   
			   short *inplen_ptr,		
			   short *inpfky_ptr,
			   unsigned long flags,
			   long line,				   
			   long column,				   
			   char filler,
			   long timeout_val,		
			   char *pmtbuf_ptr,
			   unsigned long trm_attr,
			   char char_colour,		
			   char cell_colour)		
{
/*
 *  Local Data
 */
unsigned long	    iret = PSL_NORMAL;	/* PSL Routine Status */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 Boolean    param_error = PFALSE; /* Parameter problem? */
         long       isub;		/* Loop subscript */         
	 int	    rtnsts;		/* Curses Routine Status */
	 int	    inpchar;		/* Input character */	     
	 char	    *inpchr_ptr;	/* Pointer to next chr in buffer */
								      
	 long	    cursor_offset;	/* Cursor offset in field */  
	 long	    len;		/* Length */		      
/*
 *  Terminal characteristics used for timeouts and RUBOUT.
 */
#ifdef UNIX
	 struct	 termios    savtty;		/* Saved terminal characteristics */
								      
#endif
	 int	    rubout;		/* Rubout chr */	     
	 Boolean    insert;		/* Insert mode ? */	     
#ifdef OSF
struct   sigaction sigact;		/* Signal handler action */
#endif /*OSF*/

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_INPUT");

/*
 *  Trace this routine if required
 */
    TRM__TRC_INPUT(tcb_ptr, inpbuf_ptr, bufsiz, maxchars, inplen_ptr,
		   inpfky_ptr, flags, line, column, filler, timeout_val,
		   pmtbuf_ptr, trm_attr, char_colour, cell_colour);

/*
 *  Local variable initialization.
 */
    insert = PTRUE;			/* We start in insert mode */

/*
 *  Initialize Return Variables
 */
    *inpfky_ptr = 0;			/* No function key */

/*
 *  Convert line & column numbers to positive adjust to account for
 *  different TRM and curses origins.
 */
    if (line < 0) line = tcb_ptr->tcb_maxlinw + 1 + line;
					/* Reset line number */
    line -= 1;				/* Adjust for CURSES */
    if (column < 0) column = tcb_ptr->tcb_maxcolw + 1 + column;
					/* Reset column number */
    column -= 1;			/* Adjust for CURSES */

/*
 *  Validate the parameters
 *
 *  First check that if the user has specified IOS_RETDEL then IOS_ENFTRM
 *  must not be set and maxchars must be 1.
 */
    if (((flags & IOS_RETDEL) != 0) &&
	(((flags & IOS_ENFTRM) != 0) || (maxchars != 1)))
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid routine flags specified");
	param_error = PTRUE;
    }

/*
 *  Now check maxchars is within the legal limits
 */
    if ((maxchars == 0) ||
	(maxchars > TSIZ_MAXINP) ||
	((maxchars + TSIZ_FKYPAD) > bufsiz))			      
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid value %l for MaxChars", maxchars);
	param_error = PTRUE;
    }

/*
 *  If there was a parameter problem; report it and exit.
 */
    if (param_error)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_INPUT", iret, 0);
	goto end;
    }

/*
 *  If IOS_SWCURS was specified then turn the cursor on
 */
    if ((flags & IOS_SWCURS) != 0)
    {
	(void)TRM_CURSOR(tcb_ptr, TCUR_BLNKBLK);
    }

/*
 *  Refresh the screen unless asked not to.
 */
    if ((flags & IOS_NOREFR) == 0)
    {
	(void)SCR_REFRESH(tcb_ptr, 0);
    }

/*
 *  If we have a prompt buffer and 'With No Echo' is not specified then
 *  output the prompt now.
 */
    if ((pmtbuf_ptr != NULL) && ((flags & IOS_NOECHO) == 0))
    {
	iret = TRM_OUTPUT(tcb_ptr,
			  pmtbuf_ptr,
			  0,		/* Null terminated */
			  line + 1,	/* Different TRM and curses origins */
			  column + 1,	/* Different TRM and curses origins */
			  trm_attr,
			  char_colour,
			  cell_colour);
	if (!(psl_test_status(iret))) goto end;
    }
/*
 *  If we are echoing input then invalidate the part of the live screen
 *  image that the input will corrupt so that we can guarantee that SCR_REFRESH
 *  will sort out anything we do.
 */
    if ((flags & IOS_NOECHO) == 0)				      
    {
	(void)SCR_INVALIDATE(tcb_ptr, line + 1, column + 1, 1, maxchars);
    }

/*
 *  If we have a filler character, then output it
 */
    if (filler != 0)			/* If we have a filler */
    {
	rtnsts = move(line, column);	/* Move to specified Position */
	if (rtnsts == ERR)		/* If an Error */
	{
	    iret = PSL_CURSES;
	    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from move(%d, %d) in TRM_INPUT",
		    line, column);
	    TRM__SIGNAL_ERR(tcb_ptr, "move()", iret, 0);
	    goto end;
	}

/*
 *	Loop around
 */
	for (isub = *inplen_ptr;
	     isub < maxchars;
	     isub++)
	{
	    rtnsts = addch(filler);	/* Insert the filler character */
	    if (rtnsts == ERR)		/* If an Error */
	    {
		iret = PSL_CURSES;
		PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from addch() in TRM_INPUT\n");
		TRM__SIGNAL_ERR(tcb_ptr, "addch()", iret, 0);
		goto end;
	    }
	}
    }

    if ((flags & IOS_PRGTY) != 0)	/* If purge type-ahead */
    {
	flushinp();			/* Flush the input */
    }

/*
 *  We do our own echoing so turn curses echoing off now.
 */
    (void)noecho();

#ifdef UNIX
/*
 *  Save the current terminal characteristics.
 *  Note that we alter the terminal characteristics to implement
 *  the timeout functionality and need to restore them when TRM_INPUT
 *  completes.
 */
    rtnsts = tcgetattr(0, &savtty);
    if (rtnsts != 0)
    {
	iret = PSL_UNIXERR;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    "Unexpected Error from tcgetattr()\n");
	TRM__SIGNAL_ERR(tcb_ptr, "tcgetattr()", iret, 0);
	goto end;
    }
#endif

/*
 *  Obtain the rubout key.
 */
    rubout = (int)erasechar();                           

#ifdef UNIX
/*
 *  Set up the timeout for this input.
 */
    iret = TRM__TIMEOUT(tcb_ptr, timeout_val);			 
#endif

/*
 *  If editing input
 *  If the input buffer contains some text then the input position
 *  is moved relative to this.
 */
    if ((flags & IOS_EDIT) != 0)	/* Editing input */
    {
	/*EMPTY*/;
    }
    else				/* Not editing input */
    {
	*inplen_ptr = 0;		/* No input yet */
    }
    cursor_offset = *inplen_ptr;	/* Offset in input buffer */

/*
 *  Reposition the Cursor
 */
    iret = TRM_POSCUR(tcb_ptr, line + 1, column + cursor_offset + 1);
    if (!(psl_test_status(iret))) goto end;

/*
 *  Now try to do the input
 */
    inpchr_ptr = inpbuf_ptr + *inplen_ptr;
					/* Initialize buffer pointer */
/*
 *  For OSF, timeouts are handled by an alarm.  This is somewhat messy as
 *  it requires the use of setjmp.
 */
#ifdef OSF
    if (setjmp(osf_alarm) != 0) /* If got here from signal handler...*/
    {
	inpchar = ERR;		/* Feign a timeout error */
	goto osf_timeout;       /* And continue as such */
    } 
/* 
 *  Set up the SIGALRM signal so that it when it's delivered, our handler 
 *  routine will be invoked.
 */
    sigact.sa_handler = SIGALRM_HANDLER;
    sigact.sa_flags = 0;
    (void) sigemptyset (&sigact.sa_mask);
    if (sigaction (SIGALRM, &sigact, NULL) < 0)
    {
	iret = PSL_UNIXERR;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    "Unexpected Error from sigaction()\n");
	TRM__SIGNAL_ERR(tcb_ptr, "sigaction()", iret, 0);
	goto end;
    }
#endif /*OSF*/

/*
 *  GET_IN : This label is brnached to when we need to obtain another input
 *           character - this occurs when the we process the input instead of
 *           passing it back to the caller.
 */
get_in:
    for (;;)                            /* Loop indefinitely */
    {
	(void) refresh();		/* Refresh to see new cursor posn */
					/* or editing */
#ifdef OSF          
	osf_tmr_active = PTRUE;		/* Mark timer as active		*/
	alarm (timeout_val);	        /* And request an alarm signal  */
					/* at the desired timeout	*/
#endif /*OSF*/      

	inpchar = getch();		/* Get the next character */
#ifdef OSF			
	osf_tmr_active = PFALSE;        /* If we got one, set flag to	*/
					/* ignore alarm signal when delivered */
osf_timeout:	
#endif /*OSF*/

	if (inpchar == ERR)		/* If a timeout */
	{
/*
 *	    getch timed out OR the controlling terminal has disappeared.
 *	    In the controlling terminal disappears then closedown
 *	    curses and terminate this process as we can nolonger communicate
 *	    with the user. This has the added advantage that if a user is
 *	    driving a utility which uses TRM_INPUT with V-TEST and detaches
 *	    the test terminal, then the current utility will exit.
 */
#ifdef UNIX
	    if (!isatty(0))		/* If no controlling terminal */
	    {
		endwin();		/* Closedown curses */
		exit(0);		/* Terminate process */
	    }
#endif

	    iret = PSL_TIMEOUT;		/* Return Timeout status */
	    goto end_success;
	}

/*
 *	curses fails to recognize LF and BS as function keys when defined
 *	for cursor keys (as for WYSE60), so check for them here
 */
#ifdef LF_BS_AS_FNKEY_NOT_SEEN                      
	if (inpchar == 0x08) inpchar = KEY_LEFT;
	if (inpchar == 0x0A) inpchar = KEY_DOWN;
#endif

/*
 *	If we have a RUBOUT and the user wants it returned to him then do so.
 */
	if (inpchar == rubout)
	{
	    if ((flags & IOS_RETDEL) != 0)	      		     
	    {
		*inplen_ptr = 0;	/* Set input length to zero */
		*inpfky_ptr = DFK_RUBOUT;
		iret = PSL_NORMAL;
		goto end_success;
	    }
/*
 *	    If we have rubout AND
 *	       were not asked to return it AND
 *	       are not editing THEN
 *			we can only throw it away.
 */
	    else if ((flags & IOS_EDIT) == 0)
	    {
		goto get_in;
	    }
	}

/*
 *	If the <KEY_ENTER> key was hit then the input is terminated.
 */
#if defined(HPV1020) || defined(SUN) || defined(NCR) || defined(AIXV41)
	if (inpchar == KEY_ENTER)	/* Return/Enter */
#elif defined(HP) || defined(IBM) || defined(OSF)		      	if (inpchar == KEY_F32)		/* Return/Enter */
#elif defined(WIN32)		      		
	if ((inpchar == KEY_ENTER) || (inpchar == '\r'))	
					/* Return/Enter */ 
#else
#error Platform unsupported at line __LINE__                        
#error do not platform key for return/enter                         
#endif
	{
	    if (*inplen_ptr != 0)	/* Input exists */
	    {
		iret = PSL_NORMAL;	/* Return success */
		goto end_success;
	    }
	}

/*
 *	If the user specified editing then handle the editing functions
 */
	if ((flags & IOS_EDIT) != 0)	/* If we are editing */
	{
/*
 *	    If the user hit cursor down then move the cursor to the end of
 *	    line and get more input.
 */
	    if (inpchar == KEY_DOWN)
	    {

		cursor_offset = *inplen_ptr;
		inpchr_ptr = inpbuf_ptr + cursor_offset;
		iret = TRM_POSCUR(tcb_ptr, line + 1,
				  column + cursor_offset + 1);
		if (!(psl_test_status(iret))) goto end;
		goto get_in;
	    }
/*
 *	    If the user hit cursor up then move cursor to the start of
 *	    line and get more input.
 */
	    else if (inpchar == KEY_UP)
	    {
		cursor_offset = 0;
		inpchr_ptr = inpbuf_ptr;
		iret = TRM_POSCUR(tcb_ptr, line + 1,
				      column + cursor_offset + 1);
		if (!(psl_test_status(iret))) goto end;
		goto get_in;
	    }
/*
 *	    If the user hit cursor left move cursor left and get more input.
 */
	    else if (inpchar == KEY_LEFT)
	    {
		if (cursor_offset != 0)
		{
		    inpchr_ptr -= 1;
		    cursor_offset -= 1;
		    iret = TRM_POSCUR(tcb_ptr, line + 1,
				      column + cursor_offset + 1);
		    if (!(psl_test_status(iret))) goto end;
		}
		goto get_in;
	    }
/*
 *	    If the user hit cursor right move cursor right
 */
	    else if (inpchar == KEY_RIGHT)
	    {
		if (cursor_offset <  *inplen_ptr)
		{
		    inpchr_ptr += 1;
		    cursor_offset += 1;
		    iret = TRM_POSCUR(tcb_ptr, line + 1,
				      column + cursor_offset + 1);
		    if (!(psl_test_status(iret))) goto end;
		}
		goto get_in;
	    }
/*
 *	    If the user hit RUBOUT
 */
	    else if (inpchar == rubout)
	    {
/*
 *		If there is a character to delete then reduce our cursor
 *		offset in field, reduce the buffer length and reduce our
 *		pointer into the buffer.
 */
		if (cursor_offset != 0)
		{
		    cursor_offset -= 1;
		    (*inplen_ptr) -= 1;
		    inpchr_ptr -= 1;

/*
 *		    If we are in the middle of the input line due to
 *		    cursoring left then delete character to left by
 *		    moving buffer to right of cursor left.
 */
		    if (cursor_offset != *inplen_ptr)
		    {
			len = *inplen_ptr - cursor_offset;
			(void)memcpy(inpchr_ptr, (inpchr_ptr + 1), len);

			if ((flags & IOS_NOECHO) == 0)
			{
			    iret = TRM_OUTPUT(tcb_ptr,
					      inpbuf_ptr,
					      (long)*inplen_ptr,
					      line + 1,
					      column + 1,
					      trm_attr,
					      char_colour,
					      cell_colour);
			    if (!(psl_test_status(iret))) goto end;
			}
		    }
/*
 *		    Output a space over the deleted character
 */
		    if ((flags & IOS_NOECHO) == 0)
		    {
			iret = TRM_OUTPUT(tcb_ptr,
					  " ",
					  1,
					  line + 1,
					  column + *inplen_ptr + 1,
					  trm_attr,
					  char_colour,
					  cell_colour);
			if (!(psl_test_status(iret))) goto end;
		    }
/*
 *		    Move the cursor back one character
 */
		    iret = TRM_POSCUR(tcb_ptr, line + 1,
				      column + cursor_offset + 1);
		    if (!(psl_test_status(iret))) goto end;
		}			/* End of if chr to delete */
		goto get_in;
	    }				/* End of test on inpchar */
	    else if (CONVERT_FKEY(tcb_ptr, inpchar) == DFK_EDIT)  
	    {
		if (insert)
		{
		    insert = PFALSE;
		}
		else
		{
		    insert = PTRUE;
		}
		goto get_in;
	    }
/*
 *          If the user hits <QUIT> whilst editing an input then just return it.
 */
            else if (CONVERT_FKEY(tcb_ptr, inpchar) == DFK_QUIT)      
            {
		*inplen_ptr = 255;	/* Special case for <QUIT> */
		*inpfky_ptr = DFK_QUIT;
		iret = PSL_NORMAL;
		goto end_success;
            }
	}				/* End of if editing */

/*
 *	If the character input is a function key then decode it and return it
 *	to the user.
 */
	if ((inpchar > 255) ||		/* If a function key */
	    (inpchar < 32))		/* or a control character */
	{
	    *inplen_ptr = 0;		/* Set input length to zero */
	    *inpfky_ptr = CONVERT_FKEY(tcb_ptr, inpchar);         
					/* And return fkey number */
	    iret = PSL_NORMAL;		/* Return success */
	    goto end_success;
	}

/*
 *	If the maximum number of characters have been entered then beep at
 *	the user.
 */
	if (*inplen_ptr == maxchars)		/* If buffer already full */
	{
	    (void)beep();		/* Ring the bell */
	}
	else				/* Else buffer not full & normal key */
	{

/*
 *	    If the user wants characters converted to uppercase then do it now
 */
	    if ((flags & IOS_UPPER) != 0)
	    {
		inpchar = toupper(inpchar);
					/* Convert character to upper case */
	    }

/*
 *	    Append/Insert/Over-type the character to the buffer and echo it
 *	    ourselves if that is what is required.
 */
	    len = *inplen_ptr - cursor_offset;			  

/*	    Note that memmove does nothing if appending a character */

	    if (insert)
	    {
		(void)memmove(inpbuf_ptr + cursor_offset + 1,	    
			      inpbuf_ptr + cursor_offset, len);
	    }
	    *inpchr_ptr = (char)inpchar;    /* Store the character */

/*
 *	    Point to the next character and increase the input length.
 */
	    if ((insert) || (len == 0)) (*inplen_ptr) += 1;	     
	    inpchr_ptr += 1;		/* Point to next character */
	    cursor_offset += 1;		/* Increment the count */    
	    if ((flags & IOS_NOECHO) == 0)
	    {
		iret = TRM_OUTPUT(tcb_ptr,
			  inpbuf_ptr,
			  *inplen_ptr,
			  line + 1,
			  column + 1,
			  trm_attr,
			  char_colour,
			  cell_colour);
		if (!(psl_test_status(iret))) goto end;

/*
 *		Reposition the cursor
 */
		iret = TRM_POSCUR(tcb_ptr, line + 1,
				  column + cursor_offset + 1);
		if (!(psl_test_status(iret))) goto end;

		(void) refresh();	/* !! Refresh to see echoed character */
	    }
	
/*
 *	   If the buffer is now full and we are auto-terminating return to user.
 */
	    if ((*inplen_ptr == maxchars) &&	/* If buffer now full */
 		((flags & IOS_ENFTRM) == 0))
					/* And auto-termination */
	    {
		iret = PSL_NORMAL;	/* Return success */
		goto end_success;
	    }
	}
    }

/*
 * END_SUCCESS:
 *  We branch here if the input is terminated or times out.
 */
end_success:

/*
 *  We have fiddled with the terminal characteristics to set them back.
 */
#ifdef UNIX
    (void) tcsetattr(0, TCSANOW, &savtty);
#endif

/*
 *  If IOS_SWCURS was specified then turn the cursor off
 */
    if ((flags & IOS_SWCURS) != 0)
    {
	(void)TRM_CURSOR(tcb_ptr, TCUR_NONE);
    }

/*
 *  Return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

/*
 * END: - Drop to here if an error occurred.
 *  Exit here with status
 */
end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_NEWLIN - Output one or more Linefeeds			*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_NEWLIN(tcb_ptr, no_line)				*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   no_line  = Number of Linefeeds required (defaults to 1)	*/
/*									*/
/************************************************************************/
unsigned long	TRM_NEWLIN(struct tcb *tcb_ptr,
			   short num_lines)
{
/*
 *  Local Data
 */
unsigned long	    iret = PSL_NORMAL;	/* PSL Routine Status */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 int	    rtnsts;		/* Curses Routine Status */
	 short	    isub;		/* Loop variable */

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_NEWLIN");

/*
 *  Trace this routine if required
 */
    TRM__TRC_NEWLIN(tcb_ptr, num_lines);

/*
 *  Loop for the number of line specified
 */
    for (isub = 1;
         isub <= num_lines;
         isub++);
    {
	rtnsts = addch('\n');
	if (rtnsts == ERR)		/* If an Error */
	{
	    iret = PSL_CURSES;
	    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    		       "Unexpected Error from addch(NEWLIN)");
	    TRM__SIGNAL_ERR(tcb_ptr, "addch()", iret, 0);
	    goto end;
	}
    }

/*
 *  Return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

/*
 * END: - Drop to here if an error occurred.
 *  Exit here with status
 */
end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_OUTPUT - Output text in buffer to terminal			*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_OUTPUT(tcb_ptr, txt_buff, txt_len, line, col,	*/
/*			     char, colchr, colbck)			*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr	= Pointer to Terminal Control Block		*/
/*	   txtbuf_ptr	= Pointer to Text String			*/
/*	   txt_len	= Length of text string OR 0 for Null Terminated*/
/*	   line		= Line Number (Longword)			*/
/*	   col		= Column Number (Longword)			*/
/*	   char		= Attributes (Longword)				*/
/*	   colchr	= Character Colour				*/
/*	   colbck	= Character Background Colour			*/
/*									*/
/************************************************************************/
unsigned long	TRM_OUTPUT (struct tcb *tcb_ptr,
			    char *txtbuf_ptr,
			    long txt_len,			      
			    long line,				      
			    long column,			      
			    unsigned long trm_attr,
			    char char_colour,			  
			    char cell_colour)			  
{
/*
 *  Local Data
 */
unsigned long	    iret = PSL_NORMAL;	/* PSL Routine Status */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 int	    rtnsts;		/* Curses Routine Status */
	 char	    *outbuf_ptr;	/* Pointer to text to be output */
	 Boolean    param_error = PFALSE; /* Parameter problem? */
	 Boolean    dwd;		/* Double width specified */  
	 long	    buf_len;		/* Len of supplied buffer */
	 long	    isub;		/* Loop variable */	      
         Boolean    ignore_error=PFALSE;/* Ignore mvaddstr() errors */

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_OUTPUT");

/*
 *  Trace this routine if required
 */
    TRM__TRC_OUTPUT(tcb_ptr, txtbuf_ptr, txt_len, line, column, trm_attr,
		    char_colour, cell_colour);

/*
 *  Convert line & column numbers to positive and adjust for CURSES
 */
    if	(line < 0) line = tcb_ptr->tcb_maxlinw + 1 + line;
					/* Reset line number */
    if	(column < 0) column = tcb_ptr->tcb_maxcolw + 1 + column;
					/* Reset column number */

/*
 *  Check line and column number to be valid
 */
    if ((line > tcb_ptr->tcb_maxlinw) || (column > tcb_ptr->tcb_maxcolw))
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		"Invalid value for line %d or column %d)",
		line, column);
	param_error=PTRUE;
    }

/*
 *  Check the character and cell colours are valid
 */
    if (((tcb_ptr->tcb_trmchal & TCCH_COL) != 0) &&
	((!GOOD_TCOL(&char_colour)) ||
	 (!GOOD_TCOL(&cell_colour))))
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid colour value specified");
	param_error=PTRUE;
    }

/*
 *  If there was a parameter problem; report it and exit.
 */
    if (param_error)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_OUTPUT", iret, 0);
	goto end;
    }

/*
 *  Find out if we are performing a double width output.
 */
    if ((trm_attr & TCAT_DWD) != 0)
    {
	dwd = PTRUE;
    }
    else
    {
	dwd = PFALSE;
    }

/*
 *  Adjust line and column values for CURSES
 */
    line -= 1;				/* Adjust for CURSES */
    column -= 1;			/* Adjust for CURSES */

/*
 *  Call routine to set up the attributes as required.
 */
    TRM__ATTRSET(trm_attr);

/*
 *  Set the foreground and background colour to as we wish, if colour
 *  is being used.
 */
#ifdef CURSES_COLOUR_SUPPORT
    if ((tcb_ptr->tcb_trmchal & TCCH_COL) != 0)
    {
	(void)attron(COLOR_PAIR(COLPR_NUM(char_colour,cell_colour)));
    }
#endif								    

/*
 *  Malloc enough space for the message and the terminating NULL. Note that
 *  if 'txt_len' is not 0 then the string is not null terminated and so we
 *  need to add a NULL. Unfortunately, we do not know if the calling function
 *  allocated enough space for the NULL so we need to make a copy.
 *  Also, note that if the attribute is double width we need to display two
 *  characters as curses does not yet support double width. In this case we
 *  allocate twice as much space and double every character up.
 */
    if ((txt_len != 0) || dwd)
    {

/*
 *	Work out the size of the supplied buffer.
 */
	if (txt_len != 0)
	{
	    buf_len = txt_len;			/* Not null terminated buffer */
	}
	else
	{
	    buf_len = strlen(txtbuf_ptr);	/* Null terminated buffer */
	}

/*
 *	If double width allocate twice as much space.
 */
	if (dwd)
	{
            outbuf_ptr = malloc((2 * buf_len) + 1);
	}
	else
/*
 *	Not double width so only allocate the length of the passed buffer +
 *	one character for a trailing null.
 */
	{
            outbuf_ptr = malloc(buf_len + 1);
	}

/*
 *	Check we could get enough memory.
 */
	if (outbuf_ptr == NULL)
	{
	    iret = PSL_INSVIRMEM;
	    TRM__SIGNAL_ERR(tcb_ptr, "malloc", iret, 0);
	    goto end;
	}

/*
 *	If not double width then copy the supplied buffer across to our buffer
 *	and null terminate.
 */
	if (!dwd)
	{
	    (void)memcpy(outbuf_ptr, txtbuf_ptr, txt_len);
	    *(outbuf_ptr + txt_len) = '\0';
	}
	else
/*
 *	Double width. Loop through the supplied buffer doubling up each
 *	character in our buffer and null terminating.
 */
	{
	    for (isub = 0; isub < buf_len; isub++)
	    {
		outbuf_ptr[(2*isub)] = txtbuf_ptr[isub];
		outbuf_ptr[(2*isub) + 1] = txtbuf_ptr[isub];
	    }
	    outbuf_ptr[(2*isub)] = '\0';
	}
    }
    else
/*
 *  Supplied text is not double width and is already null terminated.
 */
    {
	outbuf_ptr = txtbuf_ptr;
    }

#ifdef LAST_CHARACTER_BUG					      
/*
 *==========================================================================
 *	N.B. The following if statement is a solution to the
 *	problem where printing in the last column of the last line causes
 *	an error from mvaddstr. We used to prevent this situation by
 *      truncating the output if a character was to be output in the bottom
 *      right hand corner of the screen. We now just ignore errors from
 *      mvaddstr().
 *      Note that IBM state that this problem may be overcome by changing the
 *      width of the terminal to 79 but quite obviously, this means we cannot
 *      address the last column directly.
 *==========================================================================
 */
    if ((line == (tcb_ptr->tcb_maxlinw - 1)) &&
	((column + strlen(outbuf_ptr)) == tcb_ptr->tcb_maxcolw))
    {
        ignore_error = PTRUE;                      
    }
#endif /* LAST_CHARACTER_BUG */					   

/*
 *  Move to the appropriate position and output the text.
 */
    rtnsts = mvaddstr(line, column, outbuf_ptr);
    if ((rtnsts == ERR) && (!ignore_error))     /* If an Error */    
    {
	iret = PSL_CURSES;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	"Unexpected Error from mvaddstr(%d, %d), length %d in TRM_OUTPUT",
		line, column, strlen(outbuf_ptr));
	TRM__SIGNAL_ERR(tcb_ptr, "mvaddstr()", iret, 0);
	goto end;
    }

/*
 *  If double height, output the text again on the line above. This is because
 *  the screen image routines only pass double height bottom.
 */
    if ((trm_attr & TCAT_DHT) != 0)
    {
        rtnsts = mvaddstr(line - 1, column, outbuf_ptr);
        if ((rtnsts == ERR) && (!ignore_error))                      
    	{
	    iret = PSL_CURSES;
	    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    "Unexpected Error from mvaddstr(%d, %d), length %d in TRM_OUTPUT",
		line, column, strlen(outbuf_ptr));
	    TRM__SIGNAL_ERR(tcb_ptr, "mvaddstr()", iret, 0);
	    goto end;
        }
    }

/*
 *  Reset the attributes
 *  Switch off all attributes except the character set off
 */
    (void)attroff(A_STANDOUT | A_UNDERLINE | A_REVERSE |
		  A_BLINK | A_DIM | A_BOLD); /* Switch off all attributes */

/*
 *  Return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

    if (txt_len != 0)
    {
	free (outbuf_ptr);
    }
/*
 * END: - Drop to here if an error occurred.
 *  Exit here with status
 */
end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_POSCUR - Position the Cursor				*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_POSCUR(tcb_ptr, line, col)			*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   line	    = Line Number (Word)				*/
/*	   col	    = Column Number (Word)				*/
/*									*/
/************************************************************************/
unsigned long	TRM_POSCUR(struct tcb *tcb_ptr,
			   short line,
			   short column)
{
/*
 *  Local Data
 */
unsigned long	    iret = PSL_NORMAL;	/* PSL Routine Status */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 int	    rtnsts;		/* Curses Routine Status */

/*
 *  Trace this routine if required
 */
    TRM__TRC_POSCUR(tcb_ptr, line, column);

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_POSCUR");

/*
 *  Convert line & column numbers to positive
 */
    if (line < 0) line = tcb_ptr->tcb_maxlinw + 1 + line;
					/* Reset line number */
    if (column < 0) column = tcb_ptr->tcb_maxcolw + 1 + column;
					/* Reset column number */

/*
 *  Check line and column number to be valid
 */
    if ((line > tcb_ptr->tcb_maxlinw) || (column > tcb_ptr->tcb_maxcolw))
    {
	iret = PSL_BADPARAM;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		"Invalid value for (line %d or column %d)",
		line, column);
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_POSCUR", iret, 0);
	goto end;
    }

/*
 *  Adjust line and column values for CURSES
 */
    line -= 1;				/* Adjust for CURSES */
    column -= 1;			/* Adjust for CURSES */

/*
 *  Position the Cursor
 */
    rtnsts = move(line, column);	/* Move to specified Position */
    if (rtnsts == ERR)			/* If an Error */
    {
	iret = PSL_CURSES;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		"Unexpected Error from move(%d, %d)",
		line, column);
	TRM__SIGNAL_ERR(tcb_ptr, "move()", iret, 0);
	goto end;
    }

/*
 *  Return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

/*
 * END: - Drop to here if an error occurred.
 *  Exit here with status
 */
end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_SET_SCROLL - Set up a Scroll Area				*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_SET_SCROLL(tcb_ptr, str_line, end_line)		*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   str_line = Start Line Number (Word)				*/
/*	   end_line = End Line Number (Word)				*/
/*									*/
/************************************************************************/
unsigned long	TRM_SET_SCROLL(struct tcb *tcb_ptr,
			       short start_line,
			       short end_line)
{
/*
 *  Exit here with success
 */
    return PSL_NORMAL;
}


/************************************************************************/
/*									*/
/*	TRM_SET_SIZE - Set screen width and/or depth                   	*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   TRM_SET_SIZE(tcb_ptr, width, depth)				*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   width    = Required width					*/
/*	   depth    = Required depth					*/
/*									*/
/*	In VMS:								*/
/*	[1] As we only support a set of terminals we know about we are	*/
/*	    in a position to say whether the terminal may be re-sized.	*/
/*	[2] Widths of 0, 80 and 132 are permissible.			*/
/*	[3] Depths other than zero (no change) are only supported for	*/
/*	    decterms.							*/
/*	In UNIX:							*/
/*	[1] We know very little about the terminal (it could be		*/
/*	    anything).							*/
/*	[2] We know the terminal must be at least 80 columns wide and	*/
/*	    24 lines deep or TRM_INIT would have thrown it out.		*/
/*	[3] We know that the terminal is only capable of being re-sized	*/
/*	    if the terminfo database contains an entry for init_1string	*/
/*	    and init_2string - we checked for this in TRM_INIT		*/
/*	[4] We never allow depth to be changed.				*/
/*									*/
/************************************************************************/
unsigned long	TRM_SET_SIZE(struct tcb *tcb_ptr,
			     long width,			     
			     long depth)			     
{
#ifdef UNIX
	 int	    rtnsts;		/* Curses Routine Status */
	 Boolean    param_error;	/* Parameter error? */
unsigned long	    iret;		/* PSL return status */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 jmp_buf    jmp_env;		/* setjmp environment */
volatile unsigned long err_ctx;		/* Error context */         
struct	 diagstk    *diag_ptr;		/* Diagnostic stack pointer */
struct	 winsize    win;					    

    diag_ptr = tcb_ptr->tcb_errstkps;
/*
 *  Add diagnostic for this routine
 */
    diag_save = PSL_ADD_STATDIAG(diag_ptr, "During TRM_SET_SIZE");

/*
 *  Trace this routine if required
 */
    TRM__TRC_SETSIZ(tcb_ptr, width, depth);

/*
 *  No parameter problems yet
 */
    param_error = PFALSE;
    iret = PSL_NORMAL;

/*
 *  N.B. We just ignore the depth argument at present as there does not appear
 *  to be a way of supporting it in curses.
 */

/*
 *  width must be 0, 80 or 132
 */
    if ((width != 0) && (width != 80) && (width != 132))
    {
	PSL_ADD_DIAGNOSTIC(diag_ptr, "Invalid value for width %d", width);
	param_error=PTRUE;
    }

/*
 *  If we had a parameter problem then signal it
 */
    if (param_error)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_SET_SIZE", iret, 0);
	goto end;
    }
/*
 *  If the terminal is already the required size then do nothing
 */
    if ((width == tcb_ptr->tcb_maxcolw) || (width == 0)) goto end;

/*
 *  If there is no entry in the terminfo database for init_1string or
 *  init_2string then this terminal may not be resized.
 */
    if ((init_1string[0] == '\0') || (init_2string[0] == '\0'))
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_SET_SIZE", PSL_BADPARAM,
		"Terminal may not be resized");
	goto end;
    }

/*
 *  Flag that we want to trap signalled errors for calls to SCR routines
 */
    if (PSL_SETJMP(jmp_env) != 0)	/* Set trap and resignal after fired */
    {
        iret = PSL_RESIGNAL_ERR(diag_ptr);
	goto end;
    }

    err_ctx = PSL_TRAP_ERRORS(
		    diag_ptr,		/* Trap errors via this diag ptr */
		    0,			/* All classes */
		    0,			/* All numbers */
		    PSLERRACT_JUMP,	/* perform jump */
		    JMP_ADDR(jmp_env),	/* with above environment */
		    NULL);		/* No error handling routine */

/*
 *  Now we must deallocate all those screen images the TRM routines have.
 *  Ensure that all of the images have been disconnected first
 *
 */
    (void)SCR_DISCONNECT(tcb_ptr->tcb_scrimgps, tcb_ptr);
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_scrimgps);
					/* Live screen image ptr */
    (void)SCR_DISCONNECT(tcb_ptr->tcb_scrwrkps, tcb_ptr);
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_scrwrkps);
					/* Work screen image ptr */
    (void)SCR_DISCONNECT(tcb_ptr->tcb_errimgps, tcb_ptr);
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_errimgps),
					/* Error line screen image ptr */
    (void)SCR_DISCONNECT(tcb_ptr->tcb_msgimgps, tcb_ptr);
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_msgimgps);
					/* Message line screen image ptr */
    (void)SCR_DISCONNECT(tcb_ptr->tcb_brdimgps, tcb_ptr);
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_brdimgps);
					/* Broadcast line screen image ptr */
    (void)SCR_DISCONNECT(tcb_ptr->tcb_ahlimgps, tcb_ptr);
    (void)SCR_DEALLOCATE(tcb_ptr->tcb_ahlimgps);
					/* Autohelp line screen image ptr */

/*
 *  Because we are going to change the size of the screen we need to invalidate
 *  the curses screen image. Curses provides clearok() to invalidate the screen
 *  but because the screen is refreshed a number of times in a resize
 *  operation it does not seem to work too well. A number of terminal refreshes
 *  occur:
 *  [1] When the terminal gets the escape sequence to change size it resizes
 *	but does not clear the screen.
 *  [2] When curses receives the SIGWINCH signal to resize it refreshes again.
 *  [3] Finally somebody calls SCR_REFRESH
 *  All this looks hideous to the user so we clear the screen NOW so that
 *  when the various refreshes take place the user does not see them.
 */
    (void)clear();
    (void)refresh();

/*
 *  Reset the maxcol value in the TCB and then reset the terminal
 */
    tcb_ptr->tcb_maxcolw = width;
    if (width == 80)
    {
	rtnsts = putp(init_2string);	/* Put terminal into 80 columns */
	tcb_ptr->tcb_trmstsb &= ~TCTS_W13;
/*
 *	Let curses know the change in size. We do this by changing the
 *	CURSES COLS value and setting the terminal size in the winsize
 *	structure.
 */
	COLS = 80;			/* Let curse know the change in size */
	win.ws_row = (unsigned short)LINES;
	win.ws_col = (unsigned short)80;

    }
    else if (width == 132)
    {
	rtnsts = putp(init_1string);	/* Put terminal into 132 columns */
	tcb_ptr->tcb_trmstsb |= TCTS_W13;
/*
 *	Let curses know the change in size. We do this by changing the
 *	CURSES COLS value and setting the terminal size in the winsize
 *	structure.
 */
	COLS = 132;			/* Let curse know the change in size */
	win.ws_row = (unsigned short)LINES;
	win.ws_col = (unsigned short)132;

    }
    else				/* Should never happen */
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr,
			"Internal error resizing terminal to width %d",
			width, NULL);
	goto end;
    }
    if (rtnsts == ERR)
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "putp(init_nstring)", iret, NULL);
	goto end;
    }

/*
 *  Set the new winsize values for curses to read when we raise the SIGWINCH
 *  signal.
 */
    rtnsts = ioctl(0, TIOCSWINSZ, &win);
    if (rtnsts == -1)
    {
	iret = PSL_UNIXERR;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    "Unexpected Error from ioctl() - setting window size\n");
	TRM__SIGNAL_ERR(tcb_ptr, "ioctl()", iret, 0);
	goto end;
    }
    rtnsts = raise(SIGWINCH);
    if (rtnsts != 0)
    {
	iret = PSL_UNIXERR;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    "Unexpected Error from raise(SIGWINCH)\n");
	TRM__SIGNAL_ERR(tcb_ptr, "raise()", iret, 0);
	goto end;
    }

/*
 *  It appears that some of curses may be multi-threaded in SunOS 5.3
 *  (& certainly is on the NCR) and this causes problems receiving the
 *  SIGWINCH in time. We therefore present an opportunity for this process
 *  to be context switched so that the SIGWINCH signal will be received
 *  before we continue.
 */
#ifdef SVR4							 
    sleep(1);
    sleep(1);
    sleep(1);
    sleep(1);
#ifdef NCR							 
    sleep(1);						
    sleep(1);						
    sleep(1);						
    sleep(1);						
#endif								 
#endif

/*
 *  Having successfully resized the terminal we must reallocate and reconnect
 *  the various screen images.
 *
 *  Live screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_scrimgps),
		       tcb_ptr->tcb_maxlinw,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Connect live image
 */
    (void)SCR_CONNECT(tcb_ptr->tcb_scrimgps, tcb_ptr, 0, 0);
/*
 *  Work screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_scrwrkps),
		       tcb_ptr->tcb_maxlinw,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Connect work image
 */
    (void)SCR_CONNECT(tcb_ptr->tcb_scrwrkps, tcb_ptr, 0, 0);
/*
 *  Error line screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_errimgps),
		       1,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Message line screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_msgimgps),
		       1,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Broadcast line screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_brdimgps),
		       1,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Autohelp line screen image
 */
    (void)SCR_ALLOCATE(&(tcb_ptr->tcb_ahlimgps),
		       1,
		       tcb_ptr->tcb_maxcolw,
		       diag_ptr);
/*
 *  Remove our error trap
 */
    PSL_FLUSH_ERRORS(diag_ptr, err_ctx);

/*
 *  Remove diagnostic message here when we have done everything correctly
 */
    PSL_REMOVE_DIAGNOSTICS(diag_ptr, diag_save);

/*
 * END:
 *  Drop to here if we have a problem, return to caller with the error status.
 */
end:
    return iret;
#elif defined(WIN32)
    return PSL_NORMAL;
#endif
}


/************************************************************************/
/*									*/
/*	TRM_REFRESH_SCREEN - Refresh the Screen				*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_REFRESH_SCREEN(tcb_ptr, line, column,		*/
/*				     depth, width)			*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   line	    = Line #						*/
/*	   column   = Column #						*/
/*	   depth    = Depth						*/
/*	   width    = Width						*/
/*									*/
/*	TRM_REFRESH_SCREEN should only be called by code using the old  */
/*	old screen image routines and is therefore not used in UNIX.    */
/*									*/
/************************************************************************/
unsigned long	TRM_REFRESH_SCREEN(struct tcb *tcb_ptr, ...)

{
/*
 *  Put on diagnostic stack messages as this routine should not be called.
 */
    PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps,
		     "TRM_REFRESH_SCREEN called in UNIX");
    if ((tcb_ptr->tcb_mscflgb & TCMS_OSC) != 0)
    {
	PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps,
			 "Old screen image rtns flag set in TCB");
    }
    else
    {
	PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps,
			 "Old screen image rtns flag not set in TCB");
    }

    (void)wrefresh(stdscr);
/*
 *  Exit here with error
 */
    return PSL_BADPARAM;
}


/************************************************************************/
/*									*/
/*	TRM_PUTOUT - Flush the Output Buffer				*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_PUTOUT(tcb_ptr)					*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*									*/
/************************************************************************/
unsigned long	TRM_PUTOUT(struct tcb *tcb_ptr)

{
/*
 *  Local variables
 */
unsigned long	    diag_save;		/* Diagnostic stack context */
unsigned long	    iret;		/* PSl return status */

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_PUTOUT");

/*
 *  Trace this routine if required
 */
    TRM__TRC_PUTOUT(tcb_ptr);

    iret = SCR_REFRESH(tcb_ptr, 0);
    if (!(psl_test_status(iret)))
    {
	TRM__SIGNAL_ERR(tcb_ptr, "SCR_REFRESH", iret, 0);
	goto end;
    }

/*
 *  Return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);
    iret = PSL_NORMAL;

end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_FLUSH - Flush the output buffer                          	*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   TRM_FLUSH(tcb_ptr)						*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*									*/
/*	N.B. There is no output buffer in the UNIX version of the TRM   */
/*	routines.							*/
/*									*/
/************************************************************************/
unsigned long	TRM_FLUSH(struct tcb *tcb_ptr)

{
/*
 *  Local variables
 */
unsigned long	    iret;		/* PSL Routine Status */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 int	    rtnsts;		/* curses return status */

/*
 *  Assume success
 */
    iret = PSL_NORMAL;
/*
 *  Trace this routine if required
 */
    TRM__TRC_FLUSH(tcb_ptr);

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_FLUSH");

/*
 *  Do a curses refresh to make any changes appear on the terminal
 */
    rtnsts = refresh();
#ifndef HPV1020                                                 
    if (rtnsts == ERR)
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "refresh()", iret, 0);
	return iret;                                                
    }
#endif                                                          
/*
 *  Return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_SCROLL - Scroll some of the screen                        	*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   TRM_SCROLL(tcb_ptr, line, column, depth, width,		*/
/*		     direction, distance)				*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   line	    = start line #					*/
/*	   column   = start column #					*/
/*	   depth    = depth						*/
/*	   width    = width						*/
/*	   direction= direction to scroll				*/
/*	   distance = distance to scroll				*/
/*									*/
/************************************************************************/
unsigned long	TRM_SCROLL (struct tcb *tcb_ptr,
			    long line,				     
			    long column,			     
			    long depth,				     
			    long width,				     
			    unsigned char direction,
			    long distance)			     
{
/*
 *  Local Data
 */
unsigned long	    iret;		/* PSL Routine Status */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 int	    rtnsts;		/* curses return status */
	 Boolean    param_error;	/* Parameter problem? */
	 WINDOW	    *scrwin_ptr;	/* Scroll window pointer */

#ifndef SCROLLING_SUPPORT                                
unsigned long	    isub;		/* Loop control */	     
#else
	 int	    signed_dist;	/* Signed distance of scroll */
#endif

/*
 *  Assume the parameters are OK initially
 */
    iret = PSL_NORMAL;
    param_error = PFALSE;

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM_SCROLL");

/*
 *  Trace this routine if required
 */
    TRM__TRC_SCROLL(tcb_ptr, line, column, depth, width, direction, distance);

/*
 *  If line or column values are negative then adjust them
 */
    if (line < 0) line = tcb_ptr->tcb_maxlinw + 1 + line;
					/* Reset line number */
    if (column < 0) column = tcb_ptr->tcb_maxcolw + 1 + column;
					/* Reset column number */

/*
 *  If start line is zero or
 *     start line is still negative after adjusting or
 *     depth of area extends off the screen then error.
 */
    if ((line < 0) ||			/* Line negative after adjusting */
	(line == 0) ||
	((line + depth - 1) > tcb_ptr->tcb_maxlinw))		      
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid Line Number %ld for Depth %ld",
			   line,
			   depth);
	param_error = PTRUE;
    }

/*
 *  If start column is zero or
 *     start column is still negative after adjusting or
 *     width of area extends off screen then error
 */
    if ((column < 0) ||			/* Column negative after adjusting*/
	(column == 0) ||
	((column + width - 1) > tcb_ptr->tcb_maxcolw))	
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid Column Number %ld for Width %ld",
			   column,
			   width);
	param_error = PTRUE;
	goto end;
    }

/*
 *  If a scroll direction was asked for that was not supported this is a
 *  parameter error.
 */
    if ((direction != TSCR_UP) && (direction != TSCR_DOWN))	   
    {
	if ((direction == TSCR_LEFT) || (direction == TSCR_RIGHT))
	{
	    if ((tcb_ptr->tcb_trmchal & TRMDET_RCT) != 0)
	    {
		PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
				   "No Horizontal scrolling under unix");
	    }
	    else
	    {
		PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			"Horizontal scrolling unsupported for terminal");
	    }
	}
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			   "Invalid scroll direction supplied");
	param_error = PTRUE;
    }
    else				/* If scroll UP or DOWN direction */
    {
	if ((tcb_ptr->tcb_trmchal & TRMDET_SCR) != 0)
	{
	    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			       "Vertical scrolling unsupported for terminal");
	    param_error = PTRUE;
	}
    }

/*
 *  If there was a parameter problem; report it and exit.
 */
    if (param_error)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "TRM_SCROLL", iret, 0);
	goto end;
    }

/*
 *  Now adjust line and column number for curses
 */
    line -= 1;
    column -= 1;

/*
 *  Set up the signed distance using the direction of the scroll and the
 *  distance provided by the caller, scrolling upwards is positive and
 *  downwards negative. Horizontal scrolls cannot be performed under unix.
 */
#ifdef SCROLLING_SUPPORT                                       
    if (direction == TSCR_UP)
    {
	signed_dist = (int)distance;
    }
    else
    {
	signed_dist = 0 - (int)distance;
    }
#endif

/*
 *  If not doing a full screen scroll set up a curses window of the area to be
 *  scrolled.
 */
    scrwin_ptr = subwin(stdscr, depth, width, line, column);
    if (scrwin_ptr == NULL)		/* Error has occurred if ptr NULL */
    {
	iret = PSL_CURSES;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    "Unexpected error from subwin(stdscr,%d,%d,%d,%d) in TRM_SCROLL",
	    depth, width, line, column);
	TRM__SIGNAL_ERR(tcb_ptr, "subwin()", iret, 0);
	goto end;
    }

/*
 *  Set the window up to be able to use the scrolling functionality. These
 *  routines always OK so we don't check the return value.
 */
    (void)idlok(scrwin_ptr, TRUE);	/* Inset delete line OK */
    (void)scrollok(scrwin_ptr, TRUE);	/* Scrolling OK */

/*
 *  Perform the scroll using a curses routine, hopefully this will do a
 *  hardware scroll.
 */
#ifdef SCROLLING_SUPPORT	      				     
    rtnsts = wscrl(scrwin_ptr, signed_dist);
    if (rtnsts == ERR)			/* If an error occurred */
    {
	iret = PSL_CURSES;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		"Unexpected error from wscrl(ptr, %d) in TRM_SCROLL",
		signed_dist);
	TRM__SIGNAL_ERR(tcb_ptr, "wscrl()", iret, 0);
	goto end;
    }

/*
 *	HP doesn't support negative scrolling
 */
#else /* No SCROLLING_SUPPORT */
    for (isub = 0; isub < distance; isub++)
    {
	rtnsts = scroll(scrwin_ptr);
					/* Scroll up 1 line */
	if (rtnsts == ERR)		/* If an error occurred */
	{
	    iret = PSL_CURSES;
	    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
			"Unexpected error from scroll() in TRM_SCROLL");
	    TRM__SIGNAL_ERR(tcb_ptr, "scroll()", iret, 0);
	    goto end;
	}
    }

#endif
/*
 *  Refresh the scrolled subwindow.
 */
    rtnsts = wrefresh(scrwin_ptr);
#ifndef HPV1020                                                       
    if (rtnsts == ERR)			/* If an error occurred */
    {
	iret = PSL_CURSES;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		"Unexpected error from wrefresh(ptr) in TRM_SCROLL");
	TRM__SIGNAL_ERR(tcb_ptr, "wrefresh()", iret, 0);
	goto end;
    }
#endif                                                                
    
/*
 *  Delete the temporary scroll window
 */
    rtnsts = delwin(scrwin_ptr);
    if (rtnsts == ERR)			/* If an error occurred */
    {
	iret = PSL_CURSES;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		"Unexpected error from delwin(ptr) in TRM_SCROLL");
	TRM__SIGNAL_ERR(tcb_ptr, "delwin()", iret, 0);
	goto end;
    }

/*
 *  Return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

/*
 * END: - drop to here with any problems or premature finish.
 *  Exit here with status
 */
end:
    return iret;
}

/************************************************************************/
/*									*/
/*	TRM_GETTYP - Get the terminal type name				*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	  TRM_GET_TYPDET(trmtyp, trmnam_ptr, trmnamlen)			*/
/*									*/
/*	Where:								*/
/*									*/
/*	  trmtyp	-   Terminal type (TRTY_xxx)			*/
/*			    This is ignored on UNIX and the TERM	*/
/*			    environment variable is used instead	*/
/*	  trmnam_ptr	-   Ptr to returned terminal name		*/
/*			    This is null terminated			*/
/*	  trmnamlen	-   Length of buffer				*/
/*									*/
/*  Note that if the PSLTERM/TERM environment variables are defined	*/
/*  then the terminal name string will be set to nulls.			*/
/*									*/
/************************************************************************/
void TRM_GETTYP(unsigned char trmtyp,
		char *trmnam_ptr,
		long trmnamlen)
{
    long	len;
#ifdef WIN32
/*
 *  In NT the terminal type is hard-wired
 */
    const char *trmenv_ptr = WIN32_TRMTYP;
#elif defined(UNIX)
/* -41X061*/
    char	*trmenv_ptr;		/* Ptr to TERM environment variable */
#endif

#ifndef WIN32                                                         
/*
 *  Assume the terminal is undefined.
 */
    trmnam_ptr[0] = '\0';

/*
 *  In UNIX the trmtyp parameter is just ignored and instead we get the
 *  PSLTERM/TERM environment variable value.
 *
 *  Note
 *
 *  [1] PSLTERM overrides TERM.
 *  [2] If TRM_INIT has been called previously then "terminal_name" will
 *      already have been set up. This code is in readiness for passing the
 *      terminal name into TRM_INIT.
 */
    trmenv_ptr = NULL;			/* Assume we can't find the trm name */
    if (strlen(terminal_name) != 0)	/* If TRM_INIT has already set it up */
    {
        trmenv_ptr = terminal_name;	/* ... use value from TRM_INIT */
    }
    if (trmenv_ptr == NULL)		/* If we still have no trm name */
    {
	trmenv_ptr = getenv("PSLTERM");	/* ... try and get PSLTERM env var */
    }
    if (trmenv_ptr == NULL)		/* If we still have no trm name */
    {
	trmenv_ptr = getenv("TERM");	/* ... try and get TERM env var */
    }

/*
 *  If we just could not get a terminal name then return with trmnam containing
 *  the null string.
 */
    if (trmenv_ptr == NULL) return;

#endif /* !WIN32 */                                                   
    
/*
 *  Only copy as many characters as there is room for in the passed buffer.
 */
    len = strlen(trmenv_ptr);
    if ((len + 1) > trmnamlen) len = trmnamlen - 1;

    (void) memcpy(trmnam_ptr, trmenv_ptr, len);
    trmnam_ptr[len] = '\0';
    return;
}
 #ifdef UNIX
/************************************************************************/
/*									*/
/*	TRM_GET_TYPDET - Get details from terminal type code		*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	  return_val = TRM_GET_TYPDET(trmtyp, item_code)		*/
/*									*/
/*	Where:								*/
/*									*/
/*	  return_val	-   Returned information.			*/
/*	  trmtyp	-   Terminal type (TRTY_xxx)			*/
/*	  item_code	-   Item code defining the item of data		*/
/*			    required. These are a subset of those	*/
/*			    defined in TRMDEF.				*/
/*									*/
/*	    TRMDET_ANS	    - Is trm ANSI-compatible?			*/
/*	    TRMDET_ASZ	    - Does trm support auto-sizing?		*/
/*	    TRMDET_ATT	    - Does trm support attributes?		*/
/*	    TRMDET_COL	    - Does trm support colour?			*/
/*	    TRMDET_DEP	    - (unsigned short) terminal depth		*/
/*	    TRMDET_RCT	    - Does trm support rectangular operations?	*/
/*	    TRMDET_SCR	    - Does trm support scroll areas?		*/
/*	    TRMDET_WID	    - (unsigned short) terminal width		*/
/*	    TRMDET_132	    - Does trm support 132 columns		*/
/*									*/
/*  Note that if the item code or the terminal type is invalid, the	*/
/*  return value is set to zero.					*/
/*									*/
/************************************************************************/
unsigned long TRM_GET_TYPDET(unsigned long trmtyp,
			      unsigned long item_code)
{
/*
 *  Local Data,
 */
	 int	    status;		/* Status variable */
unsigned long	    ival;		/* Returned value */

/*
 *  Assume no value to return.
 */
    ival = 0;

    setupterm(0, 1, &status);
    if (status != 1) return 0;

    switch(item_code)
    {
	case (TRMDET_ANS):
	{
	    ival = 0;			/* UNIX does not know this */
	    break;
	}
	case (TRMDET_ASZ):
	{
	    ival = 0;			/* UNIX does not know this */
	    break;
	}
	case (TRMDET_ATT):
	{
	    if (tgetstr("me", TGETSTRARG) != NULL)
	    {
		ival = 1;
	    }
	    else
	    {
		ival = 0;
	    }
	    break;
	}
	case (TRMDET_COL):
	{
#ifdef CURSES_COLOUR_SUPPORT
	    if (has_colors())		/* If colour supported */
	    {
		ival = 1;
	    }
	    else
	    {
		ival = 0;
	    }
#else
	    ival = 0;
#endif
	    break;
	}
	case (TRMDET_DEP):
	{
	    ival = lines;
	    break;
	}
	case (TRMDET_RCT):
	{
	    ival = 0;			/* UNIX does not know this */
	    break;
	}
	case (TRMDET_SCR):
	{
#ifdef SCROLLING_SUPPORT                                             
	    if (has_il() != ERR)
	    {
		ival = 1;
	    }
	    else
	    {
		ival = 0;
	    }
#else
	    ival = 0;
#endif
	    break;
	}
	case (TRMDET_WID):
	{
	    ival = columns;
	    break;
	}
	case (TRMDET_132):
	{
	    if ((init_1string[0] != '\0') && (init_2string[0] != '\0'))
	    {
		ival = 1;
	    }
	    else
	    {
		ival = 0;
	    }
	    break;
	}
	default:
	{
	    ival = 0;
	    break;
	}
    }

    resetterm();			/* Clear up after setupterm() */
    return ival;
}
#endif
 
/************************************************************************/
/*									*/
/*	TRM_COLOUR - Set Global Colour					*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_COLOUR(tcb_ptr, colour, type)			*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   colour   = Colour Required (Byte)				*/
/*	   type	 = Global Type (0 = Screen; 1 = Border)			*/
/*									*/
/************************************************************************/
unsigned long	TRM_COLOUR(struct tcb *tcb_ptr,
			   char colour,				      
			   unsigned char colour_type)
{
/*
 *  Trace this routine if required
 */
    TRM__TRC_COLOUR(tcb_ptr, colour, colour_type);

/*
 *  Exit here with success
 */
    return PSL_NORMAL;
}


/************************************************************************/
/*									*/
/*	TRM_RESET_CHRSET - Reset the Character Set			*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_RESET_CHRSET(tcb_ptr, char_set)			*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   char_set = Required Character Set				*/
/*									*/
/************************************************************************/
unsigned long	TRM_RESET_CHRSET(struct tcb *tcb_ptr,
				 unsigned char char_set)
{
/*
 *  Trace this routine if required
 */
    TRM__TRC_RESCHR(tcb_ptr, char_set);

/*
 *  If we've been specifically asked to switch to the alternate character set
 *  then do so, otherwise reset to the ordinary curses character set.
 */
    if (char_set == CURSES_ALT_CHRSET)
    {
	(void)attron(A_ALTCHARSET);	/* Turn on alternate character set */
	tcb_ptr->tcb_chrsetb = CURSES_ALT_CHRSET;
    }
    else				/* Any other chrset, should always...*/
    {					/* be CURSES_CHRSET though. */
	(void)attroff(A_ALTCHARSET);	/* Turn off alternate character set */
	tcb_ptr->tcb_chrsetb = CURSES_CHRSET;
    }

/*
 *  Exit here with success
 */
    return PSL_NORMAL;
}

/************************************************************************/
/*									*/
/*	GOOD_TCOL - Check to see if valid colour			*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = GOOD_TCOL(tcol_ptr)					*/
/*									*/
/*	Where:								*/
/*									*/
/*	   iret		= Boolean return value, FALSE if invalid colour	*/
/*									*/
/*	   tcol_ptr	= address of variable containing colour.	*/
/*									*/
/************************************************************************/
static Boolean GOOD_TCOL(char *col_ptr)				      
{
/*
 *  Check to see if the colour is outside the min and max values and therefore
 *  invalid.
 */
    if ((*col_ptr > TCOL_MAX) || (*col_ptr < TCOL_MIN))
    {
/*
 *	Exit with false to say this is an invalid colour value
 */
	return PFALSE;
    }
/*
 *  Else we have a valid colour but if it is one of the 8 unsupported on unix
 *  we change it to one of the supported ones
 */
    else
    {
	if (*col_ptr > TCOL_WHITE)
	{
	    if (*col_ptr == TCOL_GREY)
	    {
		*col_ptr = TCOL_BLACK;
	    }
	    else if (*col_ptr == TCOL_MAUVE)
	    {
		*col_ptr = TCOL_RED;
	    }
	    else if (*col_ptr == TCOL_LIME)
	    {
		*col_ptr = TCOL_GREEN;
	    }
	    else if (*col_ptr == TCOL_ORANGE)
	    {
		*col_ptr = TCOL_YELLOW;
	    }
	    else if (*col_ptr == TCOL_LIGHTBLUE)
	    {
		*col_ptr = TCOL_BLUE;
	    }
	    else if (*col_ptr == TCOL_VIOLET)
	    {
		*col_ptr = TCOL_MAGENTA;
	    }
	    else if (*col_ptr == TCOL_TURQUOISE)
	    {
		*col_ptr = TCOL_CYAN;
	    }
	    else if (*col_ptr == TCOL_CREAM)
	    {
		*col_ptr = TCOL_WHITE;
	    }
	}
/*
 *	Exit with true to say the colour value is correct
 */
	return PTRUE;
    }
}



/************************************************************************/
/*									*/
/*	TRM__ATTRSET - Set up the curses attributes			*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   TRM_ATTRSET(trm_attr)					*/
/*									*/
/*	Where:								*/
/*									*/
/*	   trm_attr	= PSL attributes to set				*/
/*									*/
/************************************************************************/
static	 void	TRM__ATTRSET(unsigned long trm_attr)
{
/*
 *  Start by switching all curses attributes except the character set off to
 *  get us into a known state.
 */
    (void)attroff(A_STANDOUT | A_UNDERLINE | A_REVERSE |
		  A_BLINK | A_DIM | A_BOLD); /* Switch off all attributes */
/*
 *  Then go through the psl attributes starting the equivalent curses attribute
 */
    if (((trm_attr & TCAT_INT) == 0)		/* If bold */
     && ((trm_attr & TCAT_INV) != 0))		/* AND inverse */
    {
	attron(A_STANDOUT);			/* Output in standout */
    }
    else
    {
	if	((trm_attr & TCAT_INT) != 0)	/* If half intensity */
	{
	    (void)attron(A_DIM);		/* Output in dim */
	}
	else
	{
	    (void)attron(A_BOLD);		/* Output in bold */
	}

	if	((trm_attr & TCAT_INV) != 0)	/* If inverse video */
	{
	    (void)attron(A_REVERSE);		/* Output in reverse video */
	}

	if	((trm_attr & TCAT_BLK) != 0)	/* If blinking */
	{
	    (void)attron(A_BLINK);		/* Output blinking */
	}

	if	((trm_attr & TCAT_UND) != 0)	/* If underlined */
	{
	    (void)attron(A_UNDERLINE);	/* Output underlined */
	}
    }
/*
 *  Exit here.
 */
    return;
}


/************************************************************************/
/*									*/
/*	CONVERT_FKEY - Convert Function Key to DFK Symbol		*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   fkynum = CONVERT_FKEY(tcbp, inputchar)                       */
/*									*/
/*	Where:								*/
/*									*/
/*	   fkynum    = Returned Function Key Number			*/
/*         tcbp      = Ptr to terminal control block                    */
/*	   inputchar = Character Returned by CURSES			*/
/*									*/
/************************************************************************/
static	 short	CONVERT_FKEY(struct tcb *tcbp,
                             int localchar)
{
/*
 *  Local variables
 */
    unsigned short	    isub;		/* Loop variable */
    long                    table;              /* table to use */    
    
/*
 *  If the user hit the PSL_HARD_KEY then this is taken as a prefix for another
 *  key. i.e. If PSL_HARD_KEY is CTRL/X
 *  CTRL/X x might be DFK_EXIT.
 */
    if (localchar == PSL_HARD_KEY)              /* CTRL/x */
    {
/*
 *	Now that we know the PSL hard key has been hit, display some text to
 *	show this so the user knows he is in the middle of a control sequence.
 */
#ifdef HARDKEY_CONFIRMATION
        (void)TRM_OUTPUT(tcbp, PSL_HARD_KEY_TEXT, sizeof(PSL_HARD_KEY_TEXT),
		   	 tcbp->tcb_maxlinw, 1, 0, TCOL_BLACK, TCOL_WHITE);
        (void)refresh();
#endif
        localchar = getch();                    /* Get another character */
/*
 *	Now the second key of the sequence has been hit erase the hard key
 *	confirmation text.
 */
#ifdef HARDKEY_CONFIRMATION
        (void)TRM_OUTPUT(tcbp, PSL_HARD_KEY_BLANK, sizeof(PSL_HARD_KEY_BLANK),
		   	 tcbp->tcb_maxlinw, 1, 0, TCOL_BLACK, TCOL_WHITE);
        (void)refresh();
#endif
	(void)SCR_INVALIDATE(tcbp, tcbp->tcb_maxlinw, 1, 1, 6);
        table = 1;                              /* Switch to the second table */
    }
    else
    {
        table = 0;                              /* Use first table */
    }
/*
 *  Loop through the function key table looking for our entry,
 *  when found exit the routine with the value for this set up as return value.
 */
#ifdef WIN32
    if (localchar == '\r') localchar = KEY_ENTER;   
#endif
    for (isub = 0; isub < fkymax; isub++)
    {
	if (localchar == fkytab[table][isub][0]) return fkytab[table][isub][1];
    }
/*
 *  Get to here if the function key wasn't in the table so we exit returning
 *  a value stating the key was invalid
 */
    return DFK_UNDEF;
}


#ifdef CURSES_COLOUR_SUPPORT
/************************************************************************/
/*									*/
/*	TRM__COLPR_INIT - Initialization routine for colour pairs	*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	  TRM__COLPR_INIT(tcb_ptr);					*/
/*									*/
/*	This routine initializes curses colour pairs so that in future   */
/*	when any colours are needed they can be referenced using the 	*/
/*	curses COLOR_PAIR with the macro COLPR_NUM, as such:		*/
/*									*/
/*	  COLOR_PAIR(COLPR_NUM(fore_tcol, back_tcol))			*/
/*									*/
/*	This routine relies on us knowing the numbers for TCOL_BLACK	*/
/*	to WHITE, and as such would be wrong if these were changed.	*/
/*									*/
/************************************************************************/
static void TRM__COLPR_INIT(struct tcb *tcb_ptr)
{
/*
 *  Local variables
 */
unsigned long	    diag_save;		/* Diagnostic stack context */
	 int	    rtnsts;		/* curses return status */
static	 short	    curs_col[] =
			{COLOR_BLACK,	/* 0 = TCOL_BLACK - 1 */
		 	 COLOR_RED,	/* 1 = TCOL_RED - 1 */
			 COLOR_GREEN,	/* 2 = TCOL_GREEN - 1 */
			 COLOR_YELLOW,	/* 3 = TCOL_YELLOW - 1 */
			 COLOR_BLUE,	/* 4 = TCOL_BLUE - 1 */
			 COLOR_MAGENTA,	/* 5 = TCOL_MAGENTA - 1 */
			 COLOR_CYAN,	/* 6 = TCOL_CYAN - 1 */
			 COLOR_WHITE};	/* 7 = TCOL_WHITE - 1 */
	 short	    fore_tcol;		/* Foreground TCOL loop variable */
	 short	    back_tcol;		/* Background TCOL loop variable */

/*
 *  Add diagnostic for this function. We know that when this routine is called
 *  a local trap has been set up, so we can simply signal any errors.
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps,
				 "During TRM__COLPR_INIT");

/*
 *  Check to see if terminal we are dealing with has enough colour pairs
 */
    if (COLOR_PAIRS < 64)		/* Not enough colour pairs available */
    {
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		"Need at least 64 colour pairs, only have %d",
		COLOR_PAIRS);
	TRM__SIGNAL_ERR(tcb_ptr, "TRM__COLPR_INIT", PSL_CURSES,
			"Not enough colour pairs available");
    }

/*
 *  Loop through the possible combinations of foreground and background colour
 *  setting up a colour pair for each.
 */
    for (fore_tcol = 1;			/* TCOL_BLACK */
	 fore_tcol <= 8;		/* TCOL_WHITE */
	 fore_tcol++)
    {
	for (back_tcol = 1;		/* TCOL_BLACK */
	     ((back_tcol < 8) ||	/* TCOL_WHITE */
	      ((back_tcol == 8) && (fore_tcol < 8))); /* No white on white */
	     back_tcol++)
	{
	    rtnsts = init_pair(COLPR_NUM(fore_tcol, back_tcol),
			       curs_col[fore_tcol-1],
			       curs_col[back_tcol-1]);
	    if (rtnsts == ERR)		/* If error */
	    {
		PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Trying to initialise colour pair %d",
		    COLPR_NUM(fore_tcol, back_tcol));
		TRM__SIGNAL_ERR(tcb_ptr, "init_pair()", PSL_CURSES, NULL);
	    }
	}
    }
/*
 *  Return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

/*
 *  Exit here
 */
    return;
}
#endif							


/************************************************************************/
/*									*/
/*	PSLCHR_INIT - Set up character set tables for curses.		*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	  PSLCHR_INIT();						*/
/*									*/
/*									*/
/************************************************************************/
static unsigned long PSLCHR_INIT(struct tcb *tcb_ptr)
{
/*
 *  Local variables
 */
unsigned long	    iret;		/* PSL Routine Status */
unsigned long	    diag_save;		/* Diagnostic stack context */
unsigned char	    *chrset_tbl;	/* Pointer to curses chrset table */
unsigned char	    *std_dectbl;	/* Ptr curses standard decode table */
unsigned char	    *alt_dectbl;	/* Ptr curses alternate decode table */

/*
 *  Assume success
 */
    iret = PSL_NORMAL;

/*
 *  Add diagnostic for this function. We know that when this routine is called
 *  a local trap has been set up, so we can simply signal any errors.
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps,
				 "During PSLCHR_INIT");

#ifdef UNIX
#ifndef ALT_CHRSET_SUPPORT               
    SETUP_HP_ACS_CHARS ();
#endif
#endif

/*
 *  Get pointers to the tables we want to alter
 */
    chrset_tbl = (unsigned char *)PSLCHR_GETTBL(CHRTBL_CHRSETTBL,
						TRTY_CURSES,	
						0);
    if (chrset_tbl == NULL)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "PSLCHR_GETTBL", iret,
			"Getting the character set table");
	goto end;
    }

    std_dectbl = (unsigned char *)PSLCHR_GETTBL(CHRTBL_DECTBL,
						TRTY_CURSES,	
						CURSES_CHRSET);
    if (std_dectbl == NULL)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "PSLCHR_GETTBL", iret,
			"Getting the standard decode table");
	goto end;
    }

    alt_dectbl = (unsigned char *)PSLCHR_GETTBL(CHRTBL_DECTBL,
						TRTY_CURSES,	
						CURSES_ALT_CHRSET);
    if (alt_dectbl == NULL)
    {
	iret = PSL_BADPARAM;
	TRM__SIGNAL_ERR(tcb_ptr, "PSLCHR_GETTBL", iret,
			"Getting the alternate decode table");
	goto end;
    }

/*
 *  Set the first character in the character set to 0. This gets around
 *  a problem in SCR_REFRESH which relys on this being 0.
 */  
    alt_dectbl[PSLCHR_DEFAULT] = 0;

/*
 *  Find if ACS_ULCORNER character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_ULCORNER & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_BOXLEFTTP] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_BOXLEFTTP] =
			(unsigned char)(ACS_ULCORNER & A_CHARTEXT);
	std_dectbl[PSLCHR_BOXLEFTTP] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_BOXLEFTTP] =
			(unsigned char)(ACS_ULCORNER & A_CHARTEXT);
    }

/*
 *  Find if ACS_LLCORNER character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_LLCORNER & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_BOXLEFTBT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_BOXLEFTBT] =
			(unsigned char)(ACS_LLCORNER & A_CHARTEXT);
	std_dectbl[PSLCHR_BOXLEFTBT] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_BOXLEFTBT] =
			(unsigned char)(ACS_LLCORNER & A_CHARTEXT);
    }

/*
 *  Find if ACS_URCORNER character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_URCORNER & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_BOXRIGHTTP] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_BOXRIGHTTP] =
			(unsigned char)(ACS_URCORNER & A_CHARTEXT);
	std_dectbl[PSLCHR_BOXRIGHTTP] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_BOXRIGHTTP] =
			(unsigned char)(ACS_URCORNER & A_CHARTEXT);
    }

/*
 *  Find if ACS_LRCORNER character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_LRCORNER & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_BOXRIGHTBT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_BOXRIGHTBT] =
			(unsigned char)(ACS_LRCORNER & A_CHARTEXT);
	std_dectbl[PSLCHR_BOXRIGHTBT] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_BOXRIGHTBT] =
			(unsigned char)(ACS_LRCORNER & A_CHARTEXT);
    }

/*
 *  Find if ACS_RTEE character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_RTEE & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_TJUNCTRIGHT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_TJUNCTRIGHT] =
			(unsigned char)(ACS_RTEE & A_CHARTEXT);
	std_dectbl[PSLCHR_TJUNCTRIGHT] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_TJUNCTRIGHT] =
			(unsigned char)(ACS_RTEE & A_CHARTEXT);
    }

/*
 *  Find if ACS_LTEE character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_LTEE & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_TJUNCTLEFT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_TJUNCTLEFT] =
			(unsigned char)(ACS_LTEE & A_CHARTEXT);
	std_dectbl[PSLCHR_TJUNCTLEFT] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_TJUNCTLEFT] =
			(unsigned char)(ACS_LTEE & A_CHARTEXT);
    }

/*
 *  Find if ACS_BTEE character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_BTEE & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_TJUNCTBT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_TJUNCTBT] =
			(unsigned char)(ACS_BTEE & A_CHARTEXT);
	std_dectbl[PSLCHR_TJUNCTBT] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_TJUNCTBT] =
			(unsigned char)(ACS_BTEE & A_CHARTEXT);
    }

/*
 *  Find if ACS_TTEE character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_TTEE & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_TJUNCTTP] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_TJUNCTTP] =
			(unsigned char)(ACS_TTEE & A_CHARTEXT);
	std_dectbl[PSLCHR_TJUNCTTP] =0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_TJUNCTTP] =
			(unsigned char)(ACS_TTEE & A_CHARTEXT);
    }

/*
 *  Find if ACS_HLINE character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_HLINE & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_HORIZLINE] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_HORIZLINE] = (unsigned char)(ACS_HLINE & A_CHARTEXT);
	std_dectbl[PSLCHR_HORIZLINE] = 0;
	chrset_tbl[PSLCHR_HRZLINRGT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_HRZLINRGT] = (unsigned char)(ACS_HLINE & A_CHARTEXT);
	std_dectbl[PSLCHR_HRZLINRGT] = 0;
	chrset_tbl[PSLCHR_HRZLINLFT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_HRZLINLFT] = (unsigned char)(ACS_HLINE & A_CHARTEXT);
	std_dectbl[PSLCHR_HRZLINLFT] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_HORIZLINE] = (unsigned char)(ACS_HLINE & A_CHARTEXT);
	std_dectbl[PSLCHR_HRZLINRGT] = (unsigned char)(ACS_HLINE & A_CHARTEXT);
	std_dectbl[PSLCHR_HRZLINLFT] = (unsigned char)(ACS_HLINE & A_CHARTEXT);
    }

/*
 *  Find if ACS_VLINE character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_VLINE & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_VERTLINE] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_VERTLINE] = (unsigned char)(ACS_VLINE & A_CHARTEXT);
	std_dectbl[PSLCHR_VERTLINE] = 0;
	chrset_tbl[PSLCHR_VERTLINTP] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_VERTLINTP] = (unsigned char)(ACS_VLINE & A_CHARTEXT);
	std_dectbl[PSLCHR_VERTLINTP] = 0;
	chrset_tbl[PSLCHR_VERTLINBT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_VERTLINBT] = (unsigned char)(ACS_VLINE & A_CHARTEXT);
	std_dectbl[PSLCHR_VERTLINBT] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_VERTLINE] = (unsigned char)(ACS_VLINE & A_CHARTEXT);
	std_dectbl[PSLCHR_VERTLINTP] = (unsigned char)(ACS_VLINE & A_CHARTEXT);
	std_dectbl[PSLCHR_VERTLINBT] = (unsigned char)(ACS_VLINE & A_CHARTEXT);
    }

/*
 *  Find if ACS_PLUS character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_PLUS & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_INTERSECT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_INTERSECT] = (unsigned char)(ACS_PLUS & A_CHARTEXT);
	std_dectbl[PSLCHR_INTERSECT] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_INTERSECT] = (unsigned char)(ACS_PLUS & A_CHARTEXT);
    }

/*
 *  Find if ACS_S1 character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_S1 & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_HZSCAN1] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_HZSCAN1] = (unsigned char)(ACS_S1 & A_CHARTEXT);
	std_dectbl[PSLCHR_HZSCAN1] = 0;
	chrset_tbl[PSLCHR_HZSCAN3] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_HZSCAN3] = (unsigned char)(ACS_S1 & A_CHARTEXT);
	std_dectbl[PSLCHR_HZSCAN3] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_HZSCAN1] = (unsigned char)(ACS_S1 & A_CHARTEXT);
	std_dectbl[PSLCHR_HZSCAN3] = (unsigned char)(ACS_S1 & A_CHARTEXT);
    }

/*
 *  Find if ACS_S9 character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_S9 & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_HZSCAN7] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_HZSCAN7] = (unsigned char)(ACS_S9 & A_CHARTEXT);
	std_dectbl[PSLCHR_HZSCAN7] = 0;
	chrset_tbl[PSLCHR_HZSCAN9] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_HZSCAN9] = (unsigned char)(ACS_S9 & A_CHARTEXT);
	std_dectbl[PSLCHR_HZSCAN9] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_HZSCAN7] = (unsigned char)(ACS_S9 & A_CHARTEXT);
	std_dectbl[PSLCHR_HZSCAN9] = (unsigned char)(ACS_S9 & A_CHARTEXT);
    }

/*
 *  Find if ACS_DIAMOND character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_DIAMOND & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_DIAMOND] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_DIAMOND] = (unsigned char)(ACS_DIAMOND & A_CHARTEXT);
	std_dectbl[PSLCHR_DIAMOND] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_DIAMOND] = (unsigned char)(ACS_DIAMOND & A_CHARTEXT);
    }

/*
 *  Find if ACS_CKBOARD character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_CKBOARD & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_BLODGE] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_BLODGE] = (unsigned char)(ACS_CKBOARD & A_CHARTEXT);
	std_dectbl[PSLCHR_BLODGE] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_BLODGE] = (unsigned char)(ACS_CKBOARD & A_CHARTEXT);
    }

/*
 *  Find if ACS_DEGREE character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_DEGREE & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_DEGREE] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_DEGREE] = (unsigned char)(ACS_DEGREE & A_CHARTEXT);
	std_dectbl[PSLCHR_DEGREE] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_DEGREE] = (unsigned char)(ACS_DEGREE & A_CHARTEXT);
    }

/*
 *  Find if ACS_PLMINUS character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_PLMINUS & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_PLUSMINUS] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_PLUSMINUS] =
			(unsigned char)(ACS_PLMINUS & A_CHARTEXT);
	std_dectbl[PSLCHR_PLUSMINUS] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_PLUSMINUS] =
			(unsigned char)(ACS_PLMINUS & A_CHARTEXT);
    }

/*
 *  Find if ACS_BULLET character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_BULLET & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_PERIODCENTERED] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_PERIODCENTERED] =
			(unsigned char)(ACS_BULLET & A_CHARTEXT);
	std_dectbl[PSLCHR_PERIODCENTERED] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_PERIODCENTERED] =
			(unsigned char)(ACS_BULLET & A_CHARTEXT);
    }

/*
 *  Find if ACS_LARROW character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_LARROW & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_ARROWLEFT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_ARROWLEFT] =
			(unsigned char)(ACS_LARROW & A_CHARTEXT);
	std_dectbl[PSLCHR_ARROWLEFT] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_ARROWLEFT] =
			(unsigned char)(ACS_LARROW & A_CHARTEXT);
    }

/*
 *  Find if ACS_RARROW character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_RARROW & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_ARROWRIGHT] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_ARROWRIGHT] =
			(unsigned char)(ACS_RARROW & A_CHARTEXT);
	std_dectbl[PSLCHR_ARROWRIGHT] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_ARROWRIGHT] =
			(unsigned char)(ACS_RARROW & A_CHARTEXT);
    }

/*
 *  Find if ACS_DARROW character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_DARROW & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_ARROWDOWN] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_ARROWDOWN] =
			(unsigned char)(ACS_DARROW & A_CHARTEXT);
	std_dectbl[PSLCHR_ARROWDOWN] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_ARROWDOWN] =
			(unsigned char)(ACS_DARROW & A_CHARTEXT);
    }

/*
 *  Find if ACS_UARROW character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_UARROW & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_ARROWUP] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_ARROWUP] =
			(unsigned char)(ACS_UARROW & A_CHARTEXT);
	std_dectbl[PSLCHR_ARROWUP] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_ARROWUP] =
			(unsigned char)(ACS_UARROW & A_CHARTEXT);
    }

/*
 *  Find if ACS_LANTERN character is available in the alternate character set,
 *  and if so set the psl characters up to change to that character.
 */
    if ((ACS_LANTERN & A_ALTCHARSET) != 0)
    {
	chrset_tbl[PSLCHR_CURRENCY] = CURSES_ALT_CHRSET;
	alt_dectbl[PSLCHR_CURRENCY] = (unsigned char)(ACS_LANTERN & A_CHARTEXT);
	std_dectbl[PSLCHR_CURRENCY] = 0;
    }
    else		/* else use standard charset equivalent */
    {
	std_dectbl[PSLCHR_CURRENCY] = (unsigned char)(ACS_LANTERN & A_CHARTEXT);
    }

/*
 *  Return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

end:
/*
 * END: - Drop to here if any errors.
 *  Return here
 */
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_SET_TERMINAL - Set terminal back into curses mode		*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_SET_TERMINAL(tcb_ptr);				*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*									*/
/************************************************************************/
unsigned long TRM_SET_TERMINAL(struct tcb *tcb_ptr)
{
/*
 *  Local data
 */
	 long	    rtnsts;		/* Curses Routine Status */
unsigned long	    iret;   		/* PSL return status */
static	char	    trminfo[256];	/* New TERMINFO var, must be static */
	 char	    pslroot[256];	/* psl_root location */	      

    iret = PSL_NORMAL;
/*
 *  Reset the TERMINFO database back to the PSL one before we return to curses.
 */
    PSL_ADDROOT ("/lib/terminfo", pslroot, sizeof (pslroot)); 

    (void)strcpy(trminfo, "TERMINFO=");
    (void)strcat(trminfo, pslroot);				      
    rtnsts = putenv(trminfo);
    if (rtnsts != 0)
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "putenv()", iret, NULL);
	goto end;
    }
/*
 *
 *  wrefresh() takes us back into curses mode. We clear the screen first to
 *  stop the user seeing multiple refreshes of useless screens.
 */
    (void)clear();						      
    (void)wrefresh(stdscr);
#ifdef UNIX
    (void)putp(reset_1string);		/* Reset to correct state */ 
#endif

/*
 *  Clear flag for TCB to show we are back in curses and therefore can report
 *  errors via the TCB again.
 */
    (*tcb_ptr).tcb_trmflgw &= (~TCTM_OUT);

#ifdef UNIX
/*
 *  Reset the terminal back to the size in the TCB.
 */
    if (tcb_ptr->tcb_maxcolw == 80)
    {
	rtnsts = putp(init_2string);
    }
    else
    {
	rtnsts = putp(init_1string);
    }
    if (rtnsts == ERR)
    {
	iret = PSL_CURSES;
	TRM__SIGNAL_ERR(tcb_ptr, "putp(init_nstring)", iret, NULL);
	goto end;
    }
#endif

end:
    return iret;
}


/************************************************************************/
/*									*/
/*	TRM_RESET_TERMINAL - Set terminal out of curses mode		*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM_RESET_TERMINAL(tcb_ptr, flags);			*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr  = Pointer to Terminal Control Block			*/
/*	   flags    = TRST_CLEAR - clear terminal screen		*/
/*		      TRST_POSCUR - position cursor (ignored in UNIX)	*/
/*		      TRST_WID132 - force terminal into 132 columns	*/
/*									*/
/*	This function resets the terminal to its original state		*/
/*	(with the exception of the contents of the flags parameter).	*/
/*	It restores the TERMINFO environment variable to its		*/
/*	pre-TRM_INIT() state, exits curses and signals an out of curses	*/
/*	state so PSL signals can be handled sensibly.			*/
/*									*/
/************************************************************************/
unsigned long TRM_RESET_TERMINAL(struct tcb *tcb_ptr,
				 unsigned long flags)
{
/*
 *  Local data
 */
static	char	    trminfo[256];	/* New TERMINFO var, must be static */
unsigned long	    iret;   		/* PSL return status */
	 long	    rtnsts;		/* Curses Routine Status */
static	 char	    newcolsenv[20];	/* New COLS environment var */

    iret = PSL_NORMAL;
/*
 *  If the flags parameter requires it clear the screen.
 */
    if ((flags & TRST_CLEAR) != 0)
    {
	rtnsts = clear();
	if (rtnsts != OK)
	{
	    iret = PSL_CURSES;
	    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		    "Unexpected Error from clear() in TRM_RESET_TERMINAL");
	    TRM__SIGNAL_ERR(tcb_ptr, "clear()", iret, 0);
	    goto end;
	}
    }

#ifdef UNIX
/*
 *  If the flags parameter requires the terminal in 132 column mode then do so
 *  now.
 */
    if ((flags & TRST_WID132) != 0)
    {
	rtnsts = putp(init_1string);
	if (rtnsts == ERR)
	{
	    iret = PSL_CURSES;
	    TRM__SIGNAL_ERR(tcb_ptr, "putp(init_2string)", iret, NULL);
	    goto end;
	}
/*
 *	Change the COLS environment variable to 132 so that new applications
 *	will pick up the change in terminal size.
 */
	(void)strcpy(newcolsenv, "COLUMNS=132");
	rtnsts = putenv(newcolsenv);
	if (rtnsts != 0)
	{
	    iret = PSL_CURSES;
	    TRM__SIGNAL_ERR(tcb_ptr, "putenv()", iret, NULL);
	    goto end;
	}
    }
#endif

/*
 *  Refresh screen to flush any changes
 */
    (void) refresh();		

/*
 *  Exit curses temporarily?
 */
    endwin();

/*
 *  Set flag for TCB to show we are out of curses and therefore cannot report
 *  errors via the TCB.
 */
    (*tcb_ptr).tcb_trmflgw |= TCTM_OUT;

/*
 *  Reset the terminfo database - note that the user may not have defined
 *  TERMINFO before starting a PSL utility. In this case we remove our
 *  definition of TERMINFO from the environment.
 */
#ifdef TERMINFOENV_REQUIRED
    if (terminfo_database[0] != '\0')
    {
	(void)strcpy(trminfo, "TERMINFO=");
	(void)strcat(trminfo, terminfo_database);
	rtnsts = putenv(trminfo);
	if (rtnsts != 0)
	{
	    iret = PSL_CURSES;
	    TRM__SIGNAL_ERR(tcb_ptr, "putenv()", iret, NULL);
	    goto end;
	}
    }
    else
    {
	iret = REMOVE_ENVVAR(tcb_ptr, "TERMINFO");
    }
#endif /* TERMINFOENV_REQUIRED */

end:

    return iret;
}

#ifdef UNIX
#ifndef ALT_CHRSET_SUPPORT                                            
/************************************************************************/
/*									*/
/*	SETUP_HP_ACS_CHARS - Setup HP alternative character-set graphics*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   SETUP_HP_ACS_CHARS();					*/
/*									*/
/*	In SVR4 curses the terminfo database contains a string entry	*/
/*	"acsc" which contains the translations for graphics characters	*/
/*	in the alternative character set. This string contains pairs of */
/*	characters where the first character of a pair represents a	*/
/*	particular graphics character and the second is the character	*/
/*	in the alternative character set which produces this graphic.	*/
/*	If a terminal cannot display a particular graphic it is omitted */
/*	from the acsc string and curses initialization routines		*/
/*	substitute a default e.g. a "+" for corner graphics.		*/
/*	This whole process is fully automatic and the programmer only	*/
/*	has to ensure that the terminfo database is correct for a	*/
/*	particular terminal.						*/
/*									*/
/*	Currently on the HP this string is not available and graphics	*/
/*	characters are not supported. This function is used to		*/
/*	replicate the graphics character initialization on the SUN for	*/
/*	the HP.	As the HP does not support the "acsc" string we use	*/
/*	"is3" (init_3string) instead.					*/
/*									*/
/************************************************************************/
static void SETUP_HP_ACS_CHARS(void)
{
	 char	    *acs_strptr;	/* Ptr to acsc key string */
	 char	    acs_string[256];	/* Terminal acs string */
	 Boolean    acs_present;	/* acs string present in terminfo DB */
unsigned long	    isub;		/* Loop variable */
	 char	    *chr_ptr;		/* Useful pointer to char */

/*
 *  We start by setting up the default values in case the terminfo entry for
 *  the current terminal does not contain all or any of the alternate character
 *  set graphics translations.
 */
    for (isub = 0; isub < ACSC_LENGTH; isub++)
    {
	acs_map[acsckey[isub]] = acscdef[isub];	/* Setup the default */
    }

/*
 *  First we must ascertain if the terminfo entry for the current terminal
 *  contains the acs_chars string. If it is missing every character must be
 *  defaulted.
 */
    acs_string[0] = '\0';		/* Assume no "acsc" in terminfo */
    (void)strcpy(acs_string, init_3string);
					/* Get terminfo acsc string */
    if (acs_string[0] == '\0')		/* String missing */
    {
	acs_present = PFALSE;
    }
    else
    {
	acs_present = PTRUE;
    }

/*
 *  Now we loop around each possible key character setting up the translation.
 */
    for (isub = 1,
	 acs_strptr = acsckey;		/* Start with 1st key character */
	 isub <= ACSC_LENGTH; isub++,
	 acs_strptr++)
    {
	if (acs_present)		/* If acsc string in terminfo */
	{
/*
 *	    See if the current character is in terminfo acs_chars
 */
	    chr_ptr = strchr(acs_string, *acs_strptr);
	}
	else				/* No acs_chars string in terminfo */
	{
	    chr_ptr = NULL;		/* Character not found */
	}
/*
 *	If we have found the key character in the terminfo acsc_chars
 *	string then the next character is the translation. Otherwise we just
 *	leave the default in place.
 */
	if (chr_ptr == NULL)		/* We use the default */
	{
	    /*EMPTY*/;
	}
	else				/* We use terminfo translation */
	{
	    acs_map[*chr_ptr] = (A_ALTCHARSET | chr_ptr[1]);
	}
    }
}
#endif	/*HP*/

/************************************************************************/
/*									*/
/*	REMOVE_ENVVAR - Remove environment variable			*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = REMOVE_ENVVAR(tcb_ptr, envvar_ptr);			*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr	= Pointer to Terminal Control Block		*/
/*	   envvar_ptr	= Pointer to environment variable to remove	*/
/*									*/
/*  This function removes the specified environment variable from the	*/
/*  environ external variable.						*/
/*									*/
/************************************************************************/
static unsigned long REMOVE_ENVVAR(struct tcb *tcb_ptr,
				   const char *envvar_ptr)
{
/*
 *  Local data
 */
	 size_t	size;			/* Return value from strlen() */
	 char	**ep;			/* Ptr to a environment variable */
unsigned long	iret;			/* PSL return status */

    size = strlen(envvar_ptr);		/* Get length of env to remove */

/*
 *  Loop through the environment variables
 */
    for (ep = environ; *ep != NULL; ep++)
    {

/*
 *	If we find the environment variable to remove
 */
	if ((memcmp(*ep, envvar_ptr, size) == 0) && (*ep)[size] == '=')
	{

/*
 *	    Loop through the remainder of the environment variables moving
 *	    them up the array over the top the variable to remove.
 */
	    while (ep[1] != NULL)				      
	    {
		ep[0] = ep[1];
		ep++;
	    }
	    *ep = NULL;
	    return PSL_NORMAL;
	}
    }

/*
 *  If we did not find this variable then return an error.
 */
    iret = PSL_BADPARAM;
    PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
		       "Can't find environment variable %s",
		       envvar_ptr);
    TRM__SIGNAL_ERR(tcb_ptr, "clear()", iret, 0);

    return iret;
}
#endif

#ifdef UNIX
/************************************************************************/
/*									*/
/*	CHECK_FNKEYS - Check function keys defined.			*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = CHECK_FNKEYS(tcb_ptr);				*/
/*									*/
/*	Where:								*/
/*									*/
/*	   tcb_ptr	= Pointer to Terminal Control Block		*/
/*									*/
/*  This function checks that the required function keys are defined.	*/
/*									*/
/************************************************************************/
static unsigned long CHECK_FNKEYS(struct tcb *tcb_ptr)
{
unsigned long	    iret;		/* PSL return status */

/*
 *  Assume all function keys are defined.
 */
    iret = PSL_NORMAL;

/*
 *  Because we use different function keys on the HP to the SUN we must have
 *  two sets of tests.
 */
#if (!defined(HPV1020) && !defined(AIXV41)) && (defined(HP) || defined(IBM))
					     		      	      
    if (key_f30[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<DEFAULT>");
    }
    if (key_backspace[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<DELETE>");
    }
    if (key_f31[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<EDIT>");
    }
    if (key_f32[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<ENTER>");
    }
    if (key_f33[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<ERASE>");
    }
    if (key_f34[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<EXIT>");
    }
    if (key_f35[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<EXPAND>");
    }
    if (key_f36[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<FKEY-HELP>");
    }
    if (key_down[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<DOWN>");
    }
    if (key_left[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<LEFT>");
    }
    if (key_up[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<UP>");
    }
    if (key_right[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<RIGHT>");
    }
    if (key_f37[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<HELP>");
    }
    if (key_ic[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<INSERT>");
    }
    if (key_f38[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<MOVE>");
    }
    if (key_npage[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<NEXT-PAGE>");
    }
    if (key_f39[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<NEXT-SEQ>");
    }
    if (key_f40[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<OPTIONS>");
    }
    if (key_ppage[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<PREVIOUSE-PAGE>");
    }
    if (key_f41[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<PREVIOUS-SEQ>");
    }
    if (key_f42[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<PRINT>");
    }
    if (key_f43[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<QUIT>");
    }
    if (key_f44[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<REFRESH>");
    }
    if (key_f45[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<SELECT>");
    }
#elif defined(SUN) || defined(NCR) || defined(AIXV41) || defined(HPV1020)
    if (key_copy[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<DEFAULT>");
    }
    if (key_backspace[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<DELETE>");
    }
    if (key_redo[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<EDIT>");
    }
    if (key_enter[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<ENTER>");
    }
    if (key_cancel[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<ERASE>");
    }
    if (key_exit[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<EXIT>");
    }
    if (key_open[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<EXPAND>");
    }
    if (key_shelp[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<FKEY-HELP>");
    }
    if (key_down[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<DOWN>");
    }
    if (key_left[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<LEFT>");
    }
    if (key_up[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<UP>");
    }
    if (key_right[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<RIGHT>");
    }
    if (key_help[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<HELP>");
    }
    if (key_ic[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<INSERT>");
    }
    if (key_move[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<MOVE>");
    }
    if (key_npage[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<NEXT-PAGE>");
    }
    if (key_next[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<NEXT-SEQ>");
    }
    if (key_options[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<OPTIONS>");
    }
    if (key_ppage[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<PREVIOUSE-PAGE>");
    }
    if (key_previous[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<PREVIOUS-SEQ>");
    }
    if (key_print[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<PRINT>");
    }
    if (key_end[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<QUIT>");
    }
    if (key_refresh[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<REFRESH>");
    }
    if (key_select[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<SELECT>");
    }
#elif defined(OSF)
    if (key_f21[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<HELP>");
    }
    if (key_f22[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<MOVE>");
    }
    if (key_f23[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<NEXT-SEQ>");
    }
    if (key_f24[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<OPTIONS>");
    }
    if (key_f25[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<PREVIOUS-SEQ>");
    }
    if (key_f26[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<PRINT>");
    }
    if (key_f27[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<QUIT>");
    }
    if (key_f28[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<REFRESH>");
    }
    if (key_f29[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<SELECT>");
    }
    if (key_f30[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<DEFAULT>");
    }
    if (key_backspace[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<DELETE>");
    }
    if (key_f31[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<EDIT>");
    }
    if (key_f32[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<ENTER>");
    }
    if (key_f33[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<ERASE>");
    }
    if (key_f34[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<EXIT>");
    }
    if (key_f35[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<EXPAND>");
    }
    if (key_f36[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<FKEY-HELP>");
    }
    if (key_down[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<DOWN>");
    }
    if (key_left[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<LEFT>");
    }
    if (key_up[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<UP>");
    }
    if (key_right[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<RIGHT>");
    }
    if (key_ic[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<INSERT>");
    }
    if (key_ppage[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<PREVIOUSE-PAGE>");
    }
    if (key_npage[0] == '\0')
    {
	iret = PSL_MISSFNKEYDEF;
	PSL_PUTDIAG(tcb_ptr->tcb_errstkps, 1, iret, "<NEXT-PAGE>");
    }
#else
#error Platform unsupported at line __LINE__
#error do not know how to check for function keys
#endif /* SUN or NCR */


    return iret;
}
#endif /*UNIX*/

#ifdef UNIX
/************************************************************************/
/*									*/
/*	TRM__TIMEOUT - Set timed-out input				*/
/*									*/
/*	Calling Convention:						*/
/*									*/
/*	   iret = TRM__TIMEOUT(tcb_ptr, timeout_val);			*/
/*									*/
/*	Where:								*/
/*									*/
/*	  tcb_ptr     = Pointer to Terminal Control Block		*/
/*	  timeout_val = Timeout in Seconds				*/
/*									*/
/*	Setting timed-out inputs has raised a number of			*/
/*	incompatibilities between the different platforms. On the SUN	*/
/*	the curses implementation includes a timeout() function.	*/
/*	Unfortunately, this is bug ridden and this function contains	*/
/*	a number of patches supplied by SUN. On the HP the curses	*/
/*	implementation does not support timeout() and time outs are	*/
/*	achieved by changing the terminal characteristics.		*/
/*									*/
/*	Above comment nolonger applies as I have removed all use of	*/
/*	timeout() and Sun's work-around (but comment worth leaving in)	*/
/*									*/
/************************************************************************/
static unsigned long TRM__TIMEOUT(struct tcb *tcb_ptr,
				  long timeout_val)
{

struct	 termios    newtty;		/* New terminal characteristics */
	 int	    rtnsts;		/* Curses Routine Status */
unsigned long	    iret;		/* PSL return status */
unsigned long	    diag_save;		/* Diagnostic stack context */


    iret = PSL_NORMAL;			/* Assume success */

/*
 *  Add diagnostic for this function
 */
    diag_save = PSL_ADD_STATDIAG(tcb_ptr->tcb_errstkps, "During TRM__TIMEOUT");

/*
 *  We set time-out input by altering the terminal characteristics.
 *  We, therefore need the current terminal characteristics.
 */
    rtnsts = tcgetattr(0, &newtty);
    if (rtnsts != 0)
    {
	iret = PSL_UNIXERR;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    "Unexpected Error from tcgetattr()\n");
	TRM__SIGNAL_ERR(tcb_ptr, "tcgetattr()", iret, 0);
	goto end;
    }
    newtty.c_lflag &= ~ICANON;		/* Non-canonical mode */

/*
 *  Setup the timeout for the input.
 */
    if (timeout_val == 0)		/* Infinite timeout */
    {
	newtty.c_cc[VMIN] = 1;		/* Need at least one char from read */

	newtty.c_cc[VTIME] = 0;		/* No timeout */

    }
    else if (timeout_val > 32)		/* If timeout too large */
    {
	newtty.c_cc[VMIN] = 0;		/* Start timing from getch() */
	newtty.c_cc[VTIME] = (timeout_val * 10);		    
					/* timeout */
    }
    else
    {
	newtty.c_cc[VMIN] = 0;		/* Start timing from getch() */
	newtty.c_cc[VTIME] = (timeout_val * 10);		    
					/* timeout (1/10 s) */
    }
    rtnsts = tcsetattr(0, TCSANOW, &newtty);
    if (rtnsts != 0)
    {
	iret = PSL_UNIXERR;
	PSL_ADD_DIAGNOSTIC(tcb_ptr->tcb_errstkps,
	    "Unexpected Error from tcsetattr()\n");
	TRM__SIGNAL_ERR(tcb_ptr, "tcsetattr()", iret, 0);
	goto end;
    }

/*
 *  Return diagnostics stack to its pre-function state
 */
    PSL_REMOVE_DIAGNOSTICS(tcb_ptr->tcb_errstkps, diag_save);

/*
 *  This label is branched to with an error or dropped into from above with
 *  a success status.
 */
end:
    return iret;
}
#endif

#ifdef OSF
/*
 * Alarm Signal handler. This routine is used in OSF to get around inadequacies
 * in the 'getch' routine.  Since this routine has no concept of timeout, we
 * request an alarm signal at the timeout period.  This brings us here.
 * If timer is still active, (ie. stuck in getch), we jump back into TRM_INPUT
 * so that we can feign a returned timeout and continue.
 */
static void SIGALRM_HANDLER (int signo)
{
/*
 *  If timer is still active then we still haven't had an input character...
 */
    if (osf_tmr_active)
    {
	osf_tmr_active = PFALSE;	/* Mark timer as inactive	*/
	longjmp (osf_alarm, 1);         /* Jump back into TRM_INPUT	*/
    }
/*
 *  If the timer was inactive, it will have been marked as such because
 *  the getch routine returned a character.  We could be anywhere in the
 *  code by now and don't want to disturb anything, so just return.
 */
    return;
}
#endif /*OSF*/
#endif /*UNIX*/
